@@ -89,9 +89,8 @@ class AuthenticationBlock:
         if validate_credentials:
             self._validate_api_client_auth()
         anyscale.conf.CLI_TOKEN = self.credentials
-        self.log.info(
+        self.log.debug(
             f"Loaded Anyscale authentication token from {cli_token_location}.",
-            block_label=self.block_label,
         )
         self._warn_credential_file_permissions(cli_token_location)
         self.log.close_block(self.block_label)
@@ -188,8 +187,10 @@ class AuthenticationBlock:
         except ApiExceptionInternal as e:
             if e.status == 401:
                 raise click.ClickException(
-                    f"Your user credentials are invalid for {anyscale.util.get_endpoint('')}. Please go to "
-                    f"{anyscale.util.get_endpoint('/credentials')} and follow the instructions to properly "
+                    f"Your user credentials for {anyscale.util.get_endpoint('')} "
+                    "are invalid or have expired. "
+                    "Please run `anyscale login` or follow the instructions at "
+                    f"{anyscale.util.get_endpoint('/credentials')} to properly "
                     "set your credentials."
                 )
             else:
