@@ -2,8 +2,9 @@ from contextlib import contextmanager
 import os
 import sys
 import time
-from typing import Optional
+from typing import Any, Optional
 
+import click
 import colorama  # type: ignore
 from halo import Halo
 
@@ -11,6 +12,24 @@ from halo import Halo
 _process_start_time = time.time()
 
 
+def bold(text: str, color: Optional[int] = None) -> str:
+    """Outputs the text into ANSI-supported bolded text.
+
+    `colorama.Style.BRIGHT` corresponds to bold"""
+    return f"{colorama.Style.BRIGHT}{color if color else ''}{text}{colorama.Style.RESET_ALL}"
+
+
+# 2 is used for consistency across other indenting / padding in CLI
+INDENT = " " * 2
+
+
+def pad_string(text: str, padding: int = 10) -> str:
+    """Pads the remainder of text with spaces (`spaces == padding - len(text)`)
+
+    NOTE: `padding` should be longer than `len(text)` for this to be useful"""
+    return f"{text: <{padding}}"
+
+
 class BlockLogger:
     """
     Logger class for the CLI. Supports formatting in blocks if `block_label` is provided to
@@ -19,12 +38,17 @@ class BlockLogger:
     """
 
     def __init__(
-        self, log_output: bool = True, t0: float = _process_start_time,
+        self,
+        log_output: bool = True,
+        t0: float = _process_start_time,
+        spinner_manager: Any = None,
     ) -> None:
         self.t0 = t0
         # Flag to disable all terminal output from CLILogger (useful for SDK)
         self.log_output = log_output
         self.current_block: Optional[str] = None
+        self.spinner_manager = spinner_manager
+        self.indent_level: int = 0
 
     def open_block(
         self, block_label: str, block_title: Optional[str] = None, auto_close=False
@@ -66,8 +90,8 @@ class BlockLogger:
         print(file=sys.stderr)
 
     @staticmethod
-    def highlight(word: str) -> str:
-        return f"{colorama.Style.BRIGHT}{colorama.Fore.MAGENTA}{word}{colorama.Style.RESET_ALL}"
+    def highlight(text: str) -> str:
+        return bold(text, colorama.Fore.MAGENTA)
 
     def zero_time(self) -> None:
         self.t0 = time.time()
@@ -80,6 +104,7 @@ class BlockLogger:
             assert (
                 self.current_block == block_label
             ), f"Attempting to log to block {block_label}, but block {self.current_block} is currently open."
+            print(INDENT * self.indent_level, end="", file=sys.stderr)
             print(
                 *msg, file=sys.stderr,
             )
@@ -94,6 +119,7 @@ class BlockLogger:
                 end="",
                 file=sys.stderr,
             )
+            print(INDENT * self.indent_level, end="", file=sys.stderr)
             print(
                 *msg, file=sys.stderr,
             )
@@ -111,6 +137,7 @@ class BlockLogger:
                 ),
                 end="",
             )
+            print(INDENT * self.indent_level, end="", file=sys.stderr)
             print(*msg)
 
     def warning(self, *msg: str) -> None:
@@ -144,17 +171,50 @@ class BlockLogger:
         return output
 
     def error(self, *msg: str) -> None:
+        prefix_msg = f"(anyscale +{self._time_string()})"
+        self.print_red_error_message(prefix_msg, end_char="")
+
+        print(*msg, file=sys.stderr)
+
+    def confirm_missing_permission(self, msg):
+        if self.spinner_manager:
+            self.spinner_manager.stop()
+
+        self.warning(msg)
+
+        self.print_red_error_message(
+            "[DANGER] To continue without these permissions press 'y', or press 'N' to abort.",
+            end_char="",
+        )
+
+        click.confirm(
+            "", abort=True,
+        )
+
+        if self.spinner_manager:
+            self.spinner_manager.start()
+
+    def print_red_error_message(self, error_msg, end_char="\n"):
         print(
-            "{}{}(anyscale +{}){} ".format(
+            "{}{}{} {}".format(
                 colorama.Style.BRIGHT,
                 colorama.Fore.RED,
-                self._time_string(),
+                error_msg,
                 colorama.Style.RESET_ALL,
             ),
-            end="",
+            end=end_char,
             file=sys.stderr,
         )
-        print(*msg, file=sys.stderr)
+
+    @contextmanager
+    def indent(self):
+        """ Indent all output within the context
+        """
+        try:
+            self.indent_level += 1
+            yield
+        finally:
+            self.indent_level -= 1
 
 
 class LogsLogger(BlockLogger):
