@@ -1,6 +1,4 @@
-import copy
 import difflib
-import ipaddress
 import pprint
 from typing import Any, Dict, List, Optional
 
@@ -10,7 +8,9 @@ from botocore.exceptions import ClientError
 from anyscale.aws_iam_policies import (
     AMAZON_ECR_READONLY_ACCESS_POLICY_NAME,
     ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE,
+    ANYSCALE_IAM_PERMISSIONS_SERVICE_STEADY_STATE,
     get_anyscale_aws_iam_assume_role_policy,
+    get_anyscale_iam_permissions_ec2_restricted,
 )
 from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client.models.create_cloud_resource import (
@@ -27,6 +27,10 @@ from anyscale.util import (  # pylint:disable=private-import
     filter_actions_associated_with_role,
     filter_actions_from_policy_document,
 )
+from anyscale.utils.network_verification import (
+    AWS_SUBNET_CAPACITY,
+    AWS_VPC_CAPACITY,
+)
 from anyscale.utils.s3 import verify_s3_access
 
 
@@ -36,20 +40,6 @@ from anyscale.utils.s3 import verify_s3_access
 DEFAULT_RAY_IAM_ROLE = "ray-autoscaler-v1"
 
 
-class CAPACITY_THRESHOLDS:
-    """
-    These are various constants for resources capacity we want to ensure we meet.
-    """
-
-    class AWS_VPC:
-        HOSTS_MIN: int = ipaddress.ip_network("10.0.0.0/24").num_addresses
-        HOSTS_WARN: int = ipaddress.ip_network("10.0.0.0/20").num_addresses
-
-    class AWS_SUBNET:
-        HOSTS_MIN: int = ipaddress.ip_network("10.0.0.0/28").num_addresses
-        HOSTS_WARN: int = ipaddress.ip_network("10.0.0.0/24").num_addresses
-
-
 def compare_dicts_diff(d1: Dict[Any, Any], d2: Dict[Any, Any]) -> str:
     """Returns a string representation of the difference of the two dictionaries.
     Example:
@@ -83,6 +73,7 @@ def verify_aws_vpc(
     boto3_session: boto3.Session,
     logger: BlockLogger,
     ignore_capacity_errors: bool = False,  # TODO: Probably don't do this forever. Its kinda hacky
+    strict: bool = False,  # strict is currently unused # noqa: ARG001
 ) -> bool:
     logger.info("Verifying VPC ...")
     if not cloud_resource.aws_vpc_id:
@@ -102,31 +93,15 @@ def verify_aws_vpc(
         raise e
 
     # Verify that the VPC has "enough" capacity.
-    return aws_vpc_has_enough_capacity(vpc, logger) or ignore_capacity_errors
-
-
-def aws_vpc_has_enough_capacity(vpc, logger: BlockLogger) -> bool:
-    cidr_block = ipaddress.ip_network(vpc.cidr_block, strict=False)
-
-    if cidr_block.num_addresses < CAPACITY_THRESHOLDS.AWS_VPC.HOSTS_MIN:
-        logger.error(
-            f"The provided vpc ({vpc.id})'s CIDR block ({cidr_block}) is too"
-            f" small. We want at least {CAPACITY_THRESHOLDS.AWS_VPC.HOSTS_MIN} addresses,"
-            f" but this vpc only has {cidr_block.num_addresses}. Please reach out to"
-            f" support if this is an issue!"
+    if (
+        AWS_VPC_CAPACITY.verify_network_capacity(
+            cidr_block_str=vpc.cidr_block, resource_name=vpc.id, logger=logger
         )
-        return False
-    elif cidr_block.num_addresses < CAPACITY_THRESHOLDS.AWS_VPC.HOSTS_WARN:
-        logger.warning(
-            f"The provided vpc ({vpc.id})'s CIDR block ({cidr_block}) is probably"
-            f" too small. We suggest at least {CAPACITY_THRESHOLDS.AWS_VPC.HOSTS_WARN}"
-            f" addresses, but this vpc only supports up to"
-            f" {cidr_block.num_addresses} addresses."
-        )
-    else:
+        or ignore_capacity_errors
+    ):
         logger.info(f"VPC {vpc.id} verification succeeded.")
-
-    return True
+        return True
+    return False
 
 
 def _get_subnets_from_subnet_ids(subnet_ids: List[str], region: str) -> List[Any]:
@@ -141,6 +116,7 @@ def verify_aws_subnets(  # noqa: PLR0911
     is_private_network: bool,
     logger: BlockLogger,
     ignore_capacity_errors: bool = False,  # TODO: Probably don't do this forever. Its kinda hacky
+    strict: bool = False,
 ) -> bool:
     """Verify the subnets cloud resource of a cloud."""
 
@@ -159,8 +135,6 @@ def verify_aws_subnets(  # noqa: PLR0911
             subnet_id_with_az.subnet_id
             for subnet_id_with_az in cloud_resource.aws_subnet_ids_with_availability_zones
         ]
-    elif cloud_resource.aws_subnet_ids and len(cloud_resource.aws_subnet_ids) > 0:
-        subnet_ids = cloud_resource.aws_subnet_ids
     else:
         logger.error("Missing subnet IDs.")
         return False
@@ -173,6 +147,7 @@ def verify_aws_subnets(  # noqa: PLR0911
         return False
 
     subnets = _get_subnets_from_subnet_ids(subnet_ids=subnet_ids, region=region)
+    subnet_azs = set()
 
     for subnet, subnet_id in zip(subnets, subnet_ids):
         # Verify subnet exists
@@ -182,7 +157,9 @@ def verify_aws_subnets(  # noqa: PLR0911
 
         # Verify the Subnet has "enough" capacity.
         if (
-            not aws_subnet_has_enough_capacity(subnet, logger)
+            not AWS_SUBNET_CAPACITY.verify_network_capacity(
+                cidr_block_str=subnet.cidr_block, resource_name=subnet.id, logger=logger
+            )
             and not ignore_capacity_errors
         ):
             return False
@@ -199,31 +176,32 @@ def verify_aws_subnets(  # noqa: PLR0911
             logger.warning(
                 f"The subnet {subnet_id} does not have the 'Auto-assign Public IP' option enabled. This is not currently supported."
             )
+            if strict:
+                return False
 
         # Success!
         logger.info(f"Subnet {subnet.id}'s verification succeeded.")
+        subnet_azs.add(subnet.availability_zone)
 
-    logger.info(f"Subnets {cloud_resource.aws_subnet_ids} verification succeeded.")
+    if len(subnet_azs) < 2:
+        logger.error(
+            "Subnets should be in at least 2 Availability Zones. This is required for Anyscale services to function properly."
+        )
+        return False
+
+    logger.info(
+        f"Subnets {cloud_resource.aws_subnet_ids_with_availability_zones} verification succeeded."
+    )
     return True
 
 
-def associate_subnets_with_azs(
-    cloud_resource: CreateCloudResource, region: str
-) -> CreateCloudResource:
-    """This function mutates the aws_subnet_ids field in cloud_resource. It combines the subnets with its availability zone.
+def associate_aws_subnets_with_azs(
+    aws_subnet_ids: List[str], region: str
+) -> List[SubnetIdWithAvailabilityZoneAWS]:
+    """This function combines the subnets with its availability zone.
+    """
 
-    Invariant: After this function, cloud_resource will always have subnet_ids_with_availability_zones instead of aws_subnet_ids."""
-    cloud_resource = copy.deepcopy(cloud_resource)
-
-    if (
-        cloud_resource.aws_subnet_ids_with_availability_zones is not None
-        and len(cloud_resource.aws_subnet_ids_with_availability_zones) > 0
-    ):
-        return cloud_resource
-
-    subnets = _get_subnets_from_subnet_ids(
-        subnet_ids=cloud_resource.aws_subnet_ids, region=region
-    )
+    subnets = _get_subnets_from_subnet_ids(subnet_ids=aws_subnet_ids, region=region)
 
     # combine subnet and its availability zone
     subnet_ids_with_availability_zones = [
@@ -232,34 +210,8 @@ def associate_subnets_with_azs(
         )
         for subnet in subnets
     ]
-    cloud_resource.aws_subnet_ids_with_availability_zones = (
-        subnet_ids_with_availability_zones
-    )
-    cloud_resource.aws_subnet_ids = None
 
-    return cloud_resource
-
-
-def aws_subnet_has_enough_capacity(subnet, logger: BlockLogger) -> bool:
-    cidr_block = ipaddress.ip_network(subnet.cidr_block, strict=False)
-
-    if cidr_block.num_addresses < CAPACITY_THRESHOLDS.AWS_SUBNET.HOSTS_MIN:
-        logger.error(
-            f"The provided Subnet ({subnet.id})'s CIDR block ({cidr_block}) is too"
-            f" small. We want at least {CAPACITY_THRESHOLDS.AWS_SUBNET.HOSTS_MIN} addresses,"
-            f" but this subnet only has {cidr_block.num_addresses}. Please reach out to"
-            f" support if this is an issue!"
-        )
-        return False
-    elif cidr_block.num_addresses < CAPACITY_THRESHOLDS.AWS_SUBNET.HOSTS_WARN:
-        logger.warning(
-            f"The provided Subnet ({subnet.id})'s CIDR block ({cidr_block}) is probably"
-            f" too small. We suggest at least {CAPACITY_THRESHOLDS.AWS_SUBNET.HOSTS_WARN}"
-            f" addresses, but this subnet only supports up to"
-            f" {cidr_block.num_addresses} addresses."
-        )
-
-    return True
+    return subnet_ids_with_availability_zones
 
 
 def _get_roles_from_cloud_resource(
@@ -285,16 +237,30 @@ def _get_roles_from_cloud_resource(
     return roles
 
 
-def verify_aws_iam_roles(
+def verify_aws_iam_roles(  # noqa: PLR0911
     cloud_resource: CreateCloudResource,
     boto3_session: boto3.Session,
     anyscale_aws_account: str,
     logger: BlockLogger,
+    cloud_id: str,
+    strict: bool = False,
+    _use_strict_iam_permissions: bool = False,
 ) -> bool:
+
     logger.info("Verifying IAM roles ...")
     if not cloud_resource.aws_iam_role_arns:
         logger.error("Missing IAM role arns.")
         return False
+    accounts = [
+        AwsRoleArn.from_string(role).account_id
+        for role in cloud_resource.aws_iam_role_arns
+    ]
+    if len(set(accounts)) != 1:
+        logger.error(
+            f"All IAM roles must be in the same AWS account: {cloud_resource.aws_iam_role_arns}"
+        )
+        return False
+
     roles = _get_roles_from_cloud_resource(cloud_resource, boto3_session, logger)
     if roles is None:
         return False
@@ -317,19 +283,46 @@ def verify_aws_iam_roles(
                 assume_role_policy_document, expected_assume_role_policy_document
             )
         )
+        if strict:
+            return False
+
+    # Verify EC2 steady state permissions
+    # If permissions are missing, log warning message
+    anyscale_iam_permissions_ec2 = ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE
+    if _use_strict_iam_permissions:
+        anyscale_iam_permissions_ec2 = get_anyscale_iam_permissions_ec2_restricted(
+            cloud_id
+        )
 
     allow_actions_expected = filter_actions_from_policy_document(
-        ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE
+        anyscale_iam_permissions_ec2
     )
     allow_actions_on_role = filter_actions_associated_with_role(
         boto3_session, anyscale_iam_role
     )
     allow_actions_missing = allow_actions_expected - allow_actions_on_role
+
     if allow_actions_missing:
         logger.warning(
-            f"IAM role {anyscale_iam_role.arn} does not have sufficient permissions. We suggest adding these actions to ensure that cluster management works properly: {allow_actions_missing}."
+            f"IAM role {anyscale_iam_role.arn} does not have sufficient permissions for cluster management. We suggest adding these actions to ensure that cluster management works properly: {allow_actions_missing}. "
         )
 
+    # Verify Service Steady State permissions
+    # If service permissions are missing, display confirmation message to user if they would like to continue
+    allow_actions_expected = filter_actions_from_policy_document(
+        ANYSCALE_IAM_PERMISSIONS_SERVICE_STEADY_STATE
+    )
+    allow_actions_missing = allow_actions_expected - allow_actions_on_role
+    if allow_actions_missing:
+        logger.print_red_error_message(
+            "[SERVICES V2] Permissions are missing to enable services v2 "
+        )
+        logger.confirm_missing_permission(
+            f"For IAM role {anyscale_iam_role.arn}, we suggest adding the following actions:\n{pprint.pformat(allow_actions_missing)}.\n"
+        )
+        if strict:
+            return False
+
     # verifying data plane role: ray autoscaler role
     cluster_node_role = roles[1]
     policy_names = [
@@ -339,6 +332,8 @@ def verify_aws_iam_roles(
         logger.warning(
             f"Dataplane role {cluster_node_role.arn} does not contain policy {AMAZON_ECR_READONLY_ACCESS_POLICY_NAME}. This is safe to ignore if you are not pulling custom Docker Images from an ECR repository."
         )
+        if strict:
+            return False
 
     if (
         len(
@@ -360,10 +355,11 @@ def verify_aws_iam_roles(
     return True
 
 
-def verify_aws_security_groups(
+def verify_aws_security_groups(  # noqa: PLR0912
     cloud_resource: CreateCloudResource,
     boto3_session: boto3.Session,
     logger: BlockLogger,
+    strict: bool = False,
 ) -> bool:
     logger.info("Verifying security groups ...")
     if not cloud_resource.aws_security_groups:
@@ -371,29 +367,35 @@ def verify_aws_security_groups(
         return False
 
     ec2 = boto3_session.resource("ec2")
-    anyscale_security_group_arn = cloud_resource.aws_security_groups[0]
-    anyscale_security_group = ec2.SecurityGroup(anyscale_security_group_arn)
-    try:
-        anyscale_security_group.load()
-    except ClientError as e:
-        if e.response["Error"]["Code"] == "InvalidGroup.NotFound":
-            log_resource_not_found_error(
-                "Security group", anyscale_security_group_arn, logger
-            )
-            return False
-        raise e
 
-    # Now we only have one security group defining inbound rules.
-    # 443 is for HTTPS ingress
-    # 22 is for SSH
-    inbound_ip_permissions = anyscale_security_group.ip_permissions
-    expected_open_ports = [443, 22]
+    aws_security_group_ids = cloud_resource.aws_security_groups
+    anyscale_security_groups = []
 
-    inbound_ip_permissions_with_specific_port = [
+    for anyscale_security_group_id in aws_security_group_ids:
+        anyscale_security_group = ec2.SecurityGroup(anyscale_security_group_id)
+        try:
+            anyscale_security_group.load()
+        except ClientError as e:
+            if e.response["Error"]["Code"] == "InvalidGroup.NotFound":
+                log_resource_not_found_error(
+                    "Security group", anyscale_security_group_id, logger
+                )
+                return False
+            raise e
+        anyscale_security_groups.append(anyscale_security_group)
+
+    expected_open_ports = [443, 22]  # 443 is for HTTPS ingress, 22 is for SSH
+
+    inbound_ip_permissions = [
+        ip_permission
+        for anyscale_security_group in anyscale_security_groups
+        for ip_permission in anyscale_security_group.ip_permissions
+    ]
+    inbound_ip_permissions_with_specific_port = {
         ip_permission["FromPort"]
         for ip_permission in inbound_ip_permissions
         if "FromPort" in ip_permission
-    ]
+    }
     inbound_sg_rule_with_self = []  # type: ignore
     for sg_rule in inbound_ip_permissions:
         if sg_rule.get("IpProtocol") == "-1":
@@ -407,35 +409,38 @@ def verify_aws_security_groups(
         ):
             missing_open_ports.append(port)
     if missing_open_ports:
-        # TODO (sluo): update the doc link to our website once the page is up.
         logger.warning(
-            f"Security group with id {anyscale_security_group_arn} does not contain inbound permission for ports: {missing_open_ports}. These ports are used for interaction with the clusters from Anyscale UI. Please make sure to configure them according to https://docs.google.com/document/d/12QE0nZwZELvR6ocW_mDISzA568VGOQgwYXECNlvCcVk"
+            f"Security groups {aws_security_group_ids} do not contain inbound permission for ports: {missing_open_ports}. These ports are used for interaction with the clusters from Anyscale UI. Please make sure to configure them according to https://docs.anyscale.com/user-guide/onboard/clouds/deploy-on-aws#appendix-detailed-resource-requirements"
         )
+        if strict:
+            return False
 
     if not any(
-        sg_rule.get("GroupId") == anyscale_security_group_arn
+        sg_rule.get("GroupId") in aws_security_group_ids
         for sg_rule in inbound_sg_rule_with_self
     ):
         logger.error(
-            f"Security group with id {anyscale_security_group_arn} does not contain inbound permission for all ports for traffic from the same security group."
+            f"Security groups {aws_security_group_ids} do not contain inbound permission for all ports for traffic from the same security group."
         )
         return False
 
     if len(inbound_ip_permissions_with_specific_port) > len(expected_open_ports):
         logger.warning(
-            f"Security group with id {anyscale_security_group_arn} allows access to more than {expected_open_ports}. This may not be safe by default."
+            f"Security groups {aws_security_group_ids} allows access to more than {expected_open_ports}. This may not be safe by default."
         )
+        if strict:
+            return False
 
-    logger.info(
-        f"Security group {cloud_resource.aws_security_groups} verification succeeded."
-    )
+    logger.info(f"Security group {aws_security_group_ids} verification succeeded.")
     return True
 
 
-def verify_aws_s3(
+def verify_aws_s3(  # noqa: PLR0911, PLR0912
     cloud_resource: CreateCloudResource,
     boto3_session: boto3.Session,
+    region: str,
     logger: BlockLogger,
+    strict: bool = False,
 ) -> bool:
     logger.info("Verifying S3 ...")
     if not cloud_resource.aws_s3_id:
@@ -474,9 +479,11 @@ def verify_aws_s3(
     except ClientError as e:
         if e.response["Error"]["Code"] == "NoSuchCORSConfiguration":
             logger.warning(
-                f"S3 bucket {bucket_name} does not have CORS rules. This is safe to ignore if you are not using Anyscale UI. Otherwise please create the correct CORS rule for Anyscale according to https://docs.google.com/document/d/12QE0nZwZELvR6ocW_mDISzA568VGOQgwYXECNlvCcVk"
+                f"S3 bucket {bucket_name} does not have CORS rules. This is safe to ignore if you are not using Anyscale UI. Otherwise please create the correct CORS rule for Anyscale according to https://docs.anyscale.com/cloud-deployment/aws/manage-clouds#s3"
             )
             cors_rules = []
+            if strict:
+                return False
         else:
             raise e
 
@@ -490,8 +497,24 @@ def verify_aws_s3(
 
     if not has_correct_cors_rule:
         logger.warning(
-            f"S3 bucket {bucket_name} does not have the correct CORS rule for Anyscale. This is safe to ignore if you are not using Anyscale UI. Otherwise please create the correct CORS rule for Anyscale according to https://docs.google.com/document/d/12QE0nZwZELvR6ocW_mDISzA568VGOQgwYXECNlvCcVk"
+            f"S3 bucket {bucket_name} does not have the correct CORS rule for Anyscale. This is safe to ignore if you are not using Anyscale UI. Otherwise please create the correct CORS rule for Anyscale according to https://docs.anyscale.com/anyscale-cloud-administration/deploy-on-aws#appendix-detailed-resource-requirements"
         )
+        if strict:
+            return False
+
+    returned_bucket_location = boto3_session.client("s3").get_bucket_location(
+        Bucket=bucket_name
+    )["LocationConstraint"]
+
+    # LocationConstraint is `None` if the bucket is located in us-east-1
+    bucket_region = returned_bucket_location or "us-east-1"
+    if bucket_region != region:
+        logger.warning(
+            f"S3 bucket {bucket_name} is in region {bucket_region}, but this cloud is being set up in {region}."
+            "This can result in degraded cluster launch & logging performance as well as additional cross-region costs."
+        )
+        if strict:
+            return False
 
     roles = _get_roles_from_cloud_resource(cloud_resource, boto3_session, logger)
     if roles is None:
@@ -501,11 +524,15 @@ def verify_aws_s3(
         logger.warning(
             f"S3 Bucket {bucket_name} does not appear to have correct permissions for the Anyscale Control Plane role {roles[0].name}"
         )
+        if strict:
+            return False
 
     if not verify_s3_access(boto3_session, s3_bucket, roles[1], logger):
         logger.warning(
             f"S3 Bucket {bucket_name} does not appear to have correct permissions for the Data Plane role {roles[1].name}"
         )
+        if strict:
+            return False
     logger.info(f"S3 {cloud_resource.aws_s3_id} verification succeeded.")
     return True
 
@@ -529,10 +556,11 @@ def _get_network_interfaces_from_mount_targets(
     return network_interfaces
 
 
-def verify_aws_efs(  # noqa: PLR0911, PLR0912
+def verify_aws_efs(  # noqa: PLR0911, PLR0912, C901
     cloud_resource: CreateCloudResource,
     boto3_session: boto3.Session,
     logger: BlockLogger,
+    strict: bool = False,
 ) -> bool:
     logger.info("Verifying EFS ...")
     if not cloud_resource.aws_efs_id:
@@ -547,8 +575,6 @@ def verify_aws_efs(  # noqa: PLR0911, PLR0912
             subnet_id_with_az.subnet_id
             for subnet_id_with_az in cloud_resource.aws_subnet_ids_with_availability_zones
         ]
-    elif cloud_resource.aws_subnet_ids and len(cloud_resource.aws_subnet_ids) > 0:
-        subnet_ids = cloud_resource.aws_subnet_ids
     else:
         logger.error("Missing subnet IDs.")
         return False
@@ -575,15 +601,27 @@ def verify_aws_efs(  # noqa: PLR0911, PLR0912
     mount_targets_response = client.describe_mount_targets(
         FileSystemId=cloud_resource.aws_efs_id
     )
-    if not mount_targets_response.get("MountTargets"):
+    mount_targets = mount_targets_response.get("MountTargets")
+    if not mount_targets:
         logger.warning(
             f"EFS with id {cloud_resource.aws_efs_id} does not contain mount targets."
         )
         return False
 
+    # verify the mount target ID stored in our database is still valid
+    mount_target_ips = [mount_target["IpAddress"] for mount_target in mount_targets]
+    if cloud_resource.aws_efs_mount_target_ip and (
+        cloud_resource.aws_efs_mount_target_ip not in mount_target_ips
+    ):
+        logger.error(
+            f"Mount target registered with the cloud no longer exists. EFS ID: {cloud_resource.aws_efs_id} IP address: {cloud_resource.aws_efs_mount_target_ip}. Please make sure you have the correct AWS credentials set. If the EFS mount target has been deleted, please recreate the cloud or contact Anyscale for support."
+        )
+        return False
+
     network_interfaces = _get_network_interfaces_from_mount_targets(
         mount_targets_response, boto3_session, logger
     )
+
     expected_security_group_id = cloud_resource.aws_security_groups[0]
     for subnet_id in subnet_ids:
         contains_subnet_id_security_group = False
@@ -603,6 +641,28 @@ def verify_aws_efs(  # noqa: PLR0911, PLR0912
             logger.warning(
                 f"EFS with id {cloud_resource.aws_efs_id} does not contain network interface with subnet id {subnet_id} and security group id {cloud_resource.aws_security_groups[0]}."
             )
+            if strict:
+                return False
+    try:
+        backup_policy_response = client.describe_backup_policy(
+            FileSystemId=cloud_resource.aws_efs_id
+        )
+        backup_policy_status = backup_policy_response.get("BackupPolicy", {}).get(
+            "Status", ""
+        )
+        if backup_policy_status != "ENABLED":
+            logger.warning(
+                f"EFS {cloud_resource.aws_efs_id} backup policy is not enabled."
+            )
+            if strict:
+                return False
+    except ClientError as e:
+        if e.response["Error"]["Code"] == "PolicyNotFound":
+            logger.warning(f"EFS {cloud_resource.aws_efs_id} backup policy not found.")
+            if strict:
+                return False
+        else:
+            raise e
 
     logger.info(f"EFS {cloud_resource.aws_efs_id} verification succeeded.")
     return True
@@ -612,6 +672,7 @@ def verify_aws_cloudformation_stack(
     cloud_resource: CreateCloudResource,
     boto3_session: boto3.Session,
     logger: BlockLogger,
+    strict: bool = False,  # strict is currently unused # noqa: ARG001
 ) -> bool:
     logger.info("Verifying CloudFormation stack ...")
     if not cloud_resource.aws_cloudformation_stack_id:
