@@ -1,3 +1,4 @@
+import re
 from typing import List, Optional
 
 import click
@@ -95,31 +96,17 @@ def cloud_set_default(
 )
 @click.option("--name", "-n", help="Name of the cloud.", required=True, prompt="Name")
 @click.option(
-    "--yes", "-y", is_flag=True, default=False, help="Don't ask for confirmation."
-)
-@click.option(
-    "--gce",
-    is_flag=True,
-    default=False,
-    hidden=True,
-    help="GCE is the VM stack on GCP.",
-)
-@click.option(
-    "--gcp-folder-id",
-    help="Folder ID where the 'Anyscale'  folder will be created.",
+    "--project-id",
+    help="Globally Unique project ID for GCP clouds (e.g., my-project-abc123)",
     required=False,
-    type=int,
+    type=str,
 )
 @click.option(
-    "--vpc-peering-ip-range", help="IP range for VPC peering.", required=False
-)
-@click.option(
-    "--vpc-peering-target-project-id",
-    help="Project ID of the target VPC.",
+    "--functional-verify",
+    help="Verify the cloud is functional. This will check that the cloud can launch workspace/service.",
     required=False,
-)
-@click.option(
-    "--vpc-peering-target-vpc-id", help="VPC ID of the target VPC.", required=False
+    is_flag=False,
+    flag_value="workspace",
 )
 @click.option(
     "--anyscale-managed",
@@ -138,59 +125,36 @@ def setup_cloud(  # noqa: PLR0913
     provider: str,
     region: str,
     name: str,
-    yes: bool,
-    gce: bool,
-    anyscale_managed: bool,
+    project_id: str,
+    functional_verify: Optional[str],
+    anyscale_managed: bool,  # noqa: ARG001
     use_legacy_stack: bool,
-    gcp_folder_id: Optional[int],
-    vpc_peering_ip_range: Optional[str],
-    vpc_peering_target_project_id: Optional[str],
-    vpc_peering_target_vpc_id: Optional[str],
 ) -> None:
+    # TODO (congding): remove `anyscale_managed` in the future, now keeping it for compatibility
     cluster_management_stack_version = (
         ClusterManagementStackVersions.V1
         if use_legacy_stack
-        else ClusterManagementStackVersions.V2,
+        else ClusterManagementStackVersions.V2
     )
-    if provider == "aws":  # only provides Anyscale Managed option for AWS
-        if (
-            vpc_peering_ip_range
-            or vpc_peering_target_vpc_id
-            or vpc_peering_target_project_id
-        ):
-            raise click.ClickException(
-                "VPC peering option is is only available for GCP."
-            )
+    if provider == "aws":
         CloudController().setup_managed_cloud(
             provider=provider,
             region=region,
             name=name,
-            yes=yes,
-            folder_id=gcp_folder_id,
+            functional_verify=functional_verify,
             cluster_management_stack_version=cluster_management_stack_version,
         )
     elif provider == "gcp":
-        if anyscale_managed:
-            CloudController().setup_managed_cloud(
-                provider=provider,
-                region=region,
-                name=name,
-                yes=yes,
-                folder_id=gcp_folder_id,
-                cluster_management_stack_version=cluster_management_stack_version,
-            )
-        else:
-            CloudController().setup_cloud(
-                provider=provider,
-                region=region,
-                name=name,
-                yes=yes,
-                gce=gce,
-                folder_id=gcp_folder_id,
-                vpc_peering_ip_range=vpc_peering_ip_range,
-                vpc_peering_target_project_id=vpc_peering_target_project_id,
-                vpc_peering_target_vpc_id=vpc_peering_target_vpc_id,
-            )
+        if not project_id:
+            project_id = click.prompt("GCP Project ID", type=str)
+        CloudController().setup_managed_cloud(
+            provider=provider,
+            region=region,
+            name=name,
+            project_id=project_id,
+            functional_verify=functional_verify,
+            cluster_management_stack_version=cluster_management_stack_version,
+        )
 
 
 @cloud_cli.command(
@@ -275,58 +239,6 @@ def cloud_config_get(
     )
 
 
-@cloud_cli.command(
-    name="secrets",
-    help="[Deprecated] Allow clusters started in this cloud to access secrets manager.",
-    hidden=True,
-)
-@click.argument("cloud-name", required=False)
-@click.option(
-    "--name",
-    "-n",
-    help="Allow clusters started in this cloud (identified by cloud name) to access secrets manager.",
-    type=str,
-)
-@click.option(
-    "--cloud-id",
-    "--id",
-    help="Allow clusters started in this cloud (identified by cloud id) to access secrets manager.",
-    required=False,
-)
-@click.option(
-    "--yes", "-y", is_flag=True, default=False, help="Don't ask for confirmation."
-)
-@click.option(
-    "--write-permissions",
-    is_flag=True,
-    default=False,
-    help="Allow clusters starting in this cloud to write to secrets manager as well.",
-)
-def cloud_secrets(
-    cloud_name: Optional[str],
-    name: Optional[str],
-    cloud_id: Optional[str],
-    write_permissions: bool,
-    yes: bool,
-) -> None:
-    log.warning(
-        "`anyscale cloud secrets` has been deprecated. Please instead use the cloud provider "
-        "API to add the appropriate policies to access the secrets manager to your "
-        "dataplane IAM role. We will remove this command at the end of Q1 2023."
-    )
-    if cloud_name and name and cloud_name != name:
-        raise click.ClickException(
-            "The positional argument CLOUD_NAME and the keyword argument --name "
-            "were both provided. Please only provide one of these two arguments."
-        )
-    CloudController().experimental_setup_secrets(
-        cloud_name=cloud_name or name,
-        cloud_id=cloud_id,
-        write_permissions=write_permissions,
-        yes=yes,
-    )
-
-
 @cloud_cli.command(
     name="register", help="Register an anyscale cloud with your own resources."
 )
@@ -381,6 +293,63 @@ def cloud_secrets(
 @click.option(
     "--s3-bucket-id", help="S3 bucket ID.", required=False, type=str,
 )
+@click.option(
+    "--project-id",
+    help="Globally Unique project ID for GCP clouds (e.g., my-project-abc123)",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--vpc-name", help="VPC name for GCP clouds", required=False, type=str,
+)
+@click.option(
+    "--subnet-names",
+    help="Comma separated list of subnet names for GCP clouds",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--filestore-instance-id",
+    help="Filestore instance ID for GCP clouds.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--filestore-location",
+    help="Filestore location for GCP clouds.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--anyscale-service-account-email",
+    help="Anyscale service account email for GCP clouds.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--instance-service-account-email",
+    help="Instance service account email for GCP clouds.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--provider-name",
+    help="Workload Identity Federation provider name for Anyscale access.",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--firewall-policy-names",
+    help="Filewall policy names for GCP clouds",
+    required=False,
+    type=str,
+)
+@click.option(
+    "--cloud-storage-bucket-name",
+    help="Cloud storage bucket name for GCP clouds",
+    required=False,
+    type=str,
+)
 @click.option(
     "--private-network", help="Use private network.", is_flag=True, default=False,
 )
@@ -391,6 +360,16 @@ def cloud_secrets(
     hidden=True,
     help="Whether to use the legacy stack.",
 )
+@click.option(
+    "--functional-verify",
+    help="Verify the cloud is functional. This will check that the cloud can launch workspace/service.",
+    required=False,
+    is_flag=False,
+    flag_value="workspace",
+)
+@click.option(
+    "--yes", "-y", is_flag=True, default=False, help="Skip asking for confirmation."
+)
 def register_cloud(  # noqa: PLR0913
     provider: str,
     region: str,
@@ -402,12 +381,23 @@ def register_cloud(  # noqa: PLR0913
     instance_iam_role_id: str,
     security_group_ids: str,
     s3_bucket_id: str,
+    project_id: str,
+    vpc_name: str,
+    subnet_names: str,
+    filestore_instance_id: str,
+    filestore_location: str,
+    anyscale_service_account_email: str,
+    instance_service_account_email: str,
+    provider_name: str,
+    firewall_policy_names: str,
+    cloud_storage_bucket_name: str,
+    functional_verify: Optional[str],
     private_network: bool,
     use_legacy_stack: bool,
+    yes: bool,
 ) -> None:
-
+    missing_args: List[str] = []
     if provider == "aws":
-        missing_args: List[str] = []
         for resource in [
             (vpc_id, "--vpc-id"),
             (subnet_ids, "--subnet-ids"),
@@ -433,15 +423,70 @@ def register_cloud(  # noqa: PLR0913
             instance_iam_role_id=instance_iam_role_id,
             security_group_ids=security_group_ids.split(","),
             s3_bucket_id=s3_bucket_id,
+            functional_verify=functional_verify,
             private_network=private_network,
             cluster_management_stack_version=ClusterManagementStackVersions.V1
             if use_legacy_stack
             else ClusterManagementStackVersions.V2,
+            yes=yes,
         )
     elif provider == "gcp":
-        # TODO (allenyin): implement this
-        raise click.ClickException(
-            "Registering cloud for GCP is not currently supported"
+        # Keep the parameter naming ({resource}_name or {resource}_id) consistent with GCP to reduce confusion for customers
+        for resource in [
+            (project_id, "--project-id"),
+            (vpc_name, "--vpc-name"),
+            (subnet_names, "--subnet-names"),
+            (filestore_instance_id, "--filestore-instance-id"),
+            (filestore_location, "--filestore-location"),
+            (anyscale_service_account_email, "--anyscale-service-account-email"),
+            (instance_service_account_email, "--instance-service-account-email"),
+            (provider_name, "--provider-name"),
+            (firewall_policy_names, "--firewall-policy-names"),
+            (cloud_storage_bucket_name, "--cloud-storage-bucket-name"),
+        ]:
+            if resource[0] is None:
+                missing_args.append(resource[1])
+
+        if len(missing_args) > 0:
+            raise click.ClickException(f"Please provide a value for {missing_args}")
+
+        if project_id[0].isdigit():
+            # project ID should start with a letter
+            raise click.ClickException(
+                "Please provide a valid project ID. Note that project ID is not project number, see https://cloud.google.com/resource-manager/docs/creating-managing-projects#before_you_begin for details."
+            )
+
+        if (
+            re.search(
+                "projects/[0-9]*/locations/global/workloadIdentityPools/.+/providers/.+",
+                provider_name,
+            )
+            is None
+        ):
+            raise click.ClickException(
+                "Please provide a valid, fully qualified provider name. Example: projects/<project number>/locations/global/workloadIdentityPools/<pool name>/providers/<provider id>"
+            )
+
+        CloudController().register_gcp_cloud(
+            region=region,
+            name=name,
+            project_id=project_id,
+            vpc_name=vpc_name,
+            subnet_names=subnet_names.split(","),
+            filestore_instance_id=filestore_instance_id,
+            filestore_location=filestore_location,
+            anyscale_service_account_email=anyscale_service_account_email,
+            instance_service_account_email=instance_service_account_email,
+            # TODO (allenyin): use provider_name instead of provider_id everywhere.
+            provider_id=provider_name,
+            firewall_policy_names=firewall_policy_names.split(","),
+            cloud_storage_bucket_name=cloud_storage_bucket_name,
+            functional_verify=functional_verify,
+            private_network=private_network,
+            cluster_management_stack_version=ClusterManagementStackVersions.V1
+            if use_legacy_stack
+            else ClusterManagementStackVersions.V2,
+            yes=yes,
         )
     else:
         raise click.ClickException(
@@ -458,8 +503,25 @@ def register_cloud(  # noqa: PLR0913
     help="Verify cloud by cloud id, alternative to cloud name.",
     required=False,
 )
+@click.option(
+    "--functional-verify",
+    help="Verify the cloud is functional. This will check that the cloud can launch workspace/service.",
+    required=False,
+    is_flag=False,
+    flag_value="workspace",
+)
+@click.option(
+    "--strict",
+    is_flag=True,
+    default=False,
+    help="Strict Verify. Treat warnings as failures.",
+)
 def cloud_verify(
-    cloud_name: Optional[str], name: Optional[str], cloud_id: Optional[str]
+    cloud_name: Optional[str],
+    name: Optional[str],
+    cloud_id: Optional[str],
+    functional_verify: Optional[str],
+    strict: bool = False,
 ) -> bool:
     if cloud_name and name and cloud_name != name:
         raise click.ClickException(
@@ -468,5 +530,8 @@ def cloud_verify(
         )
 
     return CloudController().verify_cloud(
-        cloud_name=cloud_name or name, cloud_id=cloud_id
+        cloud_name=cloud_name or name,
+        cloud_id=cloud_id,
+        functional_verify=functional_verify,
+        strict=strict,
     )
