@@ -214,11 +214,33 @@ def start(  # noqa: PLR0913
         "context will be used if it exists. Otherwise a default project will be used."
     ),
 )
+@click.option(
+    "--cloud-id",
+    required=False,
+    default=None,
+    help=(
+        "Use cloud ID to disambiguate only when selecting a cluster to terminate with `--name`"
+        "that doesn't belong to any project. This requires cloud isolation to be enabled."
+    ),
+    hidden=True,
+)
+@click.option(
+    "--cloud",
+    required=False,
+    default=None,
+    help=(
+        "Use cloud to disambiguate only when selecting a cluster to terminate with `--name`"
+        "that doesn't belong to any project. This requires cloud isolation to be enabled."
+    ),
+    hidden=True,
+)
 def terminate(
     name: Optional[str],
     cluster_id: Optional[str],
     project_id: Optional[str],
     project: Optional[str],
+    cloud_id: Optional[str],
+    cloud: Optional[str],
 ) -> None:
     cluster_controller = ClusterController()
     cluster_controller.terminate(
@@ -226,6 +248,8 @@ def terminate(
         cluster_id=cluster_id,
         project_id=project_id,
         project_name=project,
+        cloud_id=cloud_id,
+        cloud_name=cloud,
     )
 
 
@@ -265,11 +289,33 @@ def terminate(
         "context will be used if it exists. Otherwise a default project will be used."
     ),
 )
+@click.option(
+    "--cloud-id",
+    required=False,
+    default=None,
+    help=(
+        "Use cloud ID to disambiguate only when selecting a cluster to archive with `--name`"
+        "that doesn't belong to any project. This requires cloud isolation to be enabled."
+    ),
+    hidden=True,
+)
+@click.option(
+    "--cloud",
+    required=False,
+    default=None,
+    help=(
+        "Use cloud to disambiguate only when selecting a cluster to archive with `--name`"
+        "that doesn't belong to any project. This requires cloud isolation to be enabled."
+    ),
+    hidden=True,
+)
 def archive(
     name: Optional[str],
     cluster_id: Optional[str],
     project_id: Optional[str],
     project: Optional[str],
+    cloud_id: Optional[str],
+    cloud: Optional[str],
 ) -> None:
     cluster_controller = ClusterController()
     cluster_controller.archive(
@@ -277,6 +323,8 @@ def archive(
         cluster_id=cluster_id,
         project_id=project_id,
         project_name=project,
+        cloud_id=cloud_id,
+        cloud_name=cloud,
     )
 
 
@@ -351,6 +399,28 @@ def archive(
     help="Max items to show in list.",
     callback=validate_non_negative_arg,
 )
+@click.option(
+    "--cloud-id",
+    required=False,
+    default=None,
+    help=(
+        "Use cloud ID to disambiguate only when selecting a cluster to list with `--name`"
+        "that doesn't belong to any project. This requires cloud isolation to be enabled."
+        "Note: This command doesn't support filtering clusters by cloud."
+    ),
+    hidden=True,
+)
+@click.option(
+    "--cloud",
+    required=False,
+    default=None,
+    help=(
+        "Use cloud to disambiguate only when selecting a cluster to list with `--name`"
+        "that doesn't belong to any project. This requires cloud isolation to be enabled."
+        "Note: This command doesn't support filtering clusters by cloud."
+    ),
+    hidden=True,
+)
 def list(  # noqa: A001, PLR0913
     name: Optional[str],
     cluster_id: Optional[str],
@@ -360,6 +430,8 @@ def list(  # noqa: A001, PLR0913
     include_inactive: bool,
     include_archived: bool,
     max_items: int,
+    cloud_id: Optional[str],
+    cloud: Optional[str],
 ) -> None:
     cluster_controller = ClusterController()
     cluster_controller.list(
@@ -371,4 +443,6 @@ def list(  # noqa: A001, PLR0913
         include_inactive=include_inactive,
         include_archived=include_archived,
         max_items=max_items,
+        cloud_id=cloud_id,
+        cloud_name=cloud,
     )
