@@ -149,7 +149,7 @@ def terminate(job_id: Optional[str], name: Optional[str]) -> None:
     job_controller.terminate(job_id=job_id, job_name=name)
 
 
-@job_cli.command(name="logs", help="Print the logs of a job.")
+@job_cli.command(name="logs")
 @click.option("--job-id", "--id", required=False, help="Unique ID of the job.")
 @click.option("--name", "-n", required=False, help="Name of the job.")
 @click.option(
@@ -161,10 +161,32 @@ def terminate(job_id: Optional[str], name: Optional[str]) -> None:
     is_flag=True,
     help="Whether to follow the log.",
 )
-def logs(job_id: Optional[str], name: Optional[str], follow: bool = False,) -> None:
-    job_controller = JobController()
+@click.option(
+    "--all-attempts",
+    is_flag=True,
+    default=False,
+    help="Show logs for all job attempts.",
+)
+def logs(
+    job_id: Optional[str],
+    name: Optional[str],
+    follow: bool = False,
+    all_attempts: bool = False,
+) -> None:
+    """Print the logs of a job.
+
+    By default from the latest job attempt.
+
+    Example usage:
+
+        anyscale job logs --id prodjob_123
+
+        anyscale job logs --id prodjob_123 -f
+
+        anyscale job logs -n my-job --all-attempts"""
+    job_controller = JobController(raise_structured_exception=True)
     job_controller.logs(
-        job_id=job_id, job_name=name, should_follow=follow,
+        job_id=job_id, job_name=name, should_follow=follow, all_attempts=all_attempts,
     )
 
 
@@ -213,7 +235,10 @@ def wait(
     )
 
 
-@job_cli.group("output", help="Interact with Production Job outputs.")
+@job_cli.group(
+    "output",
+    help="DEPRECATED: Interact with Production Job outputs. Job outputs do not work in V2 stack and has been removed from the UI. The CLI commands will be removed in the future.",
+)
 def output_cli() -> None:
     pass
 
@@ -223,6 +248,9 @@ def output_cli() -> None:
 @click.option("--name", "-n", required=False, help="Name of the job.")
 def get_output(job_id: Optional[str], name: Optional[str]) -> None:
     """Get the output of a Production Job."""
+    log.warning(
+        "DEPRECATION WARNING: Job outputs is a deprecated feature. This does not work in V2 stack and will be removed completely in the future."
+    )
     job_controller = JobController()
     job_controller.retrieve_output(job_name=name, job_id=job_id)
 
@@ -253,6 +281,9 @@ def write_output(file, input_str) -> None:
         cat output.json | anyscale job output write
 
     """
+    log.warning(
+        "DEPRECATION WARNING: Job outputs is a deprecated feature. This does not work in V2 stack and will be removed completely in the future."
+    )
     data = None
     if input_str and file:
         raise click.ClickException("Cannot specify both --file and an input string.")
