@@ -58,6 +58,12 @@ option_worker_only = click.option(
 option_head_only = click.option(
     "--head-only", is_flag=True, help="Download logs of only the head node."
 )
+option_unpack_combined_logs = click.option(
+    "--unpack/--no-unpack",
+    default=True,
+    help="Whether to unpack the combined-worker.log after downloading.",
+    hidden=True,
+)
 
 option_ttl = click.option(
     "--ttl",
@@ -97,6 +103,7 @@ def log_cli() -> None:
 @option_instance_id
 @option_worker_only
 @option_head_only
+@option_unpack_combined_logs
 @option_download_dir
 @option_ttl
 @option_parallelism
@@ -110,6 +117,7 @@ def anyscale_logs_cluster(  # noqa: PLR0913
     instance_id: Optional[str],
     worker_only: bool,
     head_only: bool,
+    unpack: bool,
     # list files config
     ttl: int,
     # download files config
@@ -143,6 +151,7 @@ def anyscale_logs_cluster(  # noqa: PLR0913
             ttl_seconds=ttl,
             download_dir=download_dir,
             parallelism=parallelism,
+            unpack=unpack,
         )
 
     else:
