@@ -1,7 +1,7 @@
 import json
 import os
 import sys
-from typing import Any
+from typing import Any, Tuple
 
 import click
 import requests
@@ -137,7 +137,12 @@ def terminate(name: str) -> None:
     cluster_id = workspace.cluster_id
 
     cluster_controller.terminate(
-        cluster_name=None, cluster_id=cluster_id, project_id=None, project_name=None,
+        cluster_name=None,
+        cluster_id=cluster_id,
+        project_id=None,
+        project_name=None,
+        cloud_id=None,
+        cloud_name=None,
     )
 
 
@@ -272,11 +277,17 @@ def run(command: str, web_terminal: bool, as_job: bool, no_push: bool,) -> None:
         )
         print()
     else:
-        workspace_controller.run_cmd(f"cd ~/{dir_name} && {command}")
+        workspace_controller.run_cmd(cmd=f"cd ~/{dir_name} && {command}", args=[])
 
 
-@workspace_cli.command(name="ssh", help="ssh into a workspace")
-def ssh() -> None:
+@workspace_cli.command(
+    name="ssh",
+    help="ssh into a workspace, you can also pass args to the ssh command. E.g. 'anyscale workspace ssh -- -L 8888:localhost:8888",
+)
+@click.argument(
+    "args", nargs=-1, required=False, type=click.UNPROCESSED,
+)
+def ssh(args: Tuple[str]) -> None:
     """
     ssh into a running workspace.
     """
@@ -284,7 +295,43 @@ def ssh() -> None:
 
     workspace_controller = WorkspaceController()
     dir_name = workspace_controller.get_workspace_dir_name()
-    workspace_controller.run_cmd(f"cd ~/{dir_name} && /bin/bash")
+    workspace_controller.run_cmd(f"cd ~/{dir_name} && /bin/bash", args=list(args))
+
+
+@workspace_cli.command(
+    name="list", help="prints information about existing workspaces", hidden=True,
+)
+def list_command() -> None:
+    workspace_controller = WorkspaceController()
+    workspace_controller.list()
+
+
+@workspace_cli.command(name="cp")
+@click.argument(
+    "remote_path",
+    nargs=1,
+    required=True,
+    type=click.Path(
+        readable=False  # we don't want to check the readability of a remote path against local file
+    ),
+)
+@click.argument(
+    "local_path", nargs=1, required=True, type=click.Path(writable=True),
+)
+def copy_command(remote_path, local_path) -> None:
+    """
+        Copy a file or a directory from workspace to local file system.
+
+        Examples
+
+            anyscale workspace cp /mnt/shared_objects/foo.py /tmp/copy_of_foo.py
+
+            anyscale workspace cp "~/default/README.md" ~/Downlaods
+
+            anyscale workspace cp "/tmp/" ~/Downlaods
+
+    """
+    _do_copy(remote_path, local_path)
 
 
 def _do_pull(pull_git_state):
@@ -294,10 +341,14 @@ def _do_pull(pull_git_state):
     # a repack prior to pulling. Otherwise, the pulled git repo may not be fully
     # functional locally. A repack is expensive, but we assume pulls aren't frequent.
     if pull_git_state:
-        workspace_controller.run_cmd(f"cd ~/{dir_name} && git repack -a -d")
+        workspace_controller.run_cmd(
+            f'cd ~/{dir_name} && [[ -n "$ENABLE_MOUNT_SNAPSHOT_UTIL" ]] && python -m snapshot_util repack_git_repos || git repack -a -d -q',
+            args=[],
+        )
+
     workspace_root = get_workspace_root_or_throw()
     workspace_controller.run_rsync(
-        workspace_root,
+        local_path=workspace_root,
         down=True,
         rsync_filters=[".gitignore"],
         rsync_excludes=[".anyscale.yaml", ".git/objects/info/alternates"]
@@ -309,7 +360,7 @@ def _do_push(push_git_state):
     workspace_controller = WorkspaceController()
     workspace_root = get_workspace_root_or_throw()
     workspace_controller.run_rsync(
-        workspace_root,
+        local_path=workspace_root,
         down=False,
         rsync_filters=[".gitignore"],
         # TODO(ekl) to efficiently push the git state, we need to do this in two
@@ -321,6 +372,17 @@ def _do_push(push_git_state):
     )
 
 
+def _do_copy(remote_path: str, local_path: str):
+    workspace_controller = WorkspaceController()
+    workspace_controller.run_rsync(
+        local_path=local_path,
+        remote_path=remote_path,
+        down=True,
+        rsync_filters=[],
+        rsync_excludes=[],
+    )
+
+
 def _check_local():
     if "ANYSCALE_WORKING_DIR" in os.environ:
         _exit_error(
@@ -336,8 +398,7 @@ def _check_workspace():
 
 
 def get_workspace_from_name(name: str) -> Any:
-    """Get a workspace from its name.
-    """
+    """Get a workspace from its name."""
 
     # Find the workspace by name
     auth_api_client = get_auth_api_client()
