@@ -32,7 +32,6 @@ import sys
 from types import ModuleType
 from typing import Any, Dict, List, Optional, Tuple, Union
 from urllib.parse import parse_qs, urlparse
-import uuid
 
 from packaging import version
 import requests
@@ -49,8 +48,6 @@ from anyscale.connect_utils.start_interactive_session import (  # pylint:disable
     _get_interactive_shell_frame,
     start_interactive_session_block,
 )
-from anyscale.controllers.exec_controller import ExecController
-from anyscale.controllers.session_controller import SessionController
 import anyscale.project
 from anyscale.sdk.anyscale_client.sdk import AnyscaleSDK
 from anyscale.shared_anyscale_utils.util import slugify
@@ -61,9 +58,6 @@ from anyscale.utils.connect_helpers import AnyscaleClientContext, get_cluster
 # Max number of auto created clusters.
 MAX_CLUSTERS = 40
 
-# Default minutes for autosuspend.
-DEFAULT_AUTOSUSPEND_TIMEOUT = 120
-
 # The paths to exclude when syncing the working directory in runtime env.
 EXCLUDE_DIRS = [".git", "__pycache__", "venv"]
 EXCLUDE_PATHS = [".anyscale.yaml", "session-default.yaml"]
@@ -164,7 +158,7 @@ class ClientBuilder:
         >>> [0, 1, 4, 9, 16]
     """
 
-    def __init__(
+    def __init__(  # noqa: PLR0912
         self,
         address: Optional[str] = None,
         anyscale_sdk: AnyscaleSDK = None,
@@ -186,12 +180,6 @@ class ClientBuilder:
             self._log = log
         else:
             self._log = BlockLogger()
-        self._log.warning(
-            "DEPRECATION WARNING: Anyscale Connect will be deprecated in future Anyscale versions. "
-            "Please use workspaces: "
-            "https://docs.anyscale.com/user-guide/develop-and-debug/workspaces or Ray Jobs "
-            "https://docs.anyscale.com/user-guide/develop-and-debug/ray-jobs-support instead."
-        )
         if anyscale_sdk:
             self._anyscale_sdk = anyscale_sdk
         else:
@@ -209,6 +197,7 @@ class ClientBuilder:
             )
             configure_open_api_client_headers(self._anyscale_sdk.api_client, "connect")
         api_client = auth_api_client.api_client
+
         configure_open_api_client_headers(api_client.api_client, "connect")
         anyscale_api_client = auth_api_client.anyscale_api_client
         self._api_client = api_client
@@ -231,8 +220,6 @@ class ClientBuilder:
         self._os: Any = _os
         self._requests: Any = requests
         self._in_shell = _is_in_shell()
-        self._session_controller = SessionController()
-        self._exec_controller = ExecController()
 
         self._log.open_block("ParseArgs", block_title="Parsing Ray Client arguments")
 
@@ -751,8 +738,6 @@ class ClientBuilder:
             self._fill_config_from_env("cluster_env")
 
         if self._autosuspend_timeout is None:
-            # set to default autosuspend timeout
-            self._autosuspend_timeout = DEFAULT_AUTOSUSPEND_TIMEOUT
             # override to env variable if available
             self._fill_config_from_env("autosuspend")
 
@@ -811,6 +796,22 @@ class ClientBuilder:
             >>> ray.init("anyscale://cluster_name?update=True")
         """
 
+        feature_flag_on = self._api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
+            "anyscale_connect_enabled_cli"
+        ).result.is_on
+
+        if not feature_flag_on:
+            raise RuntimeError(
+                "Anyscale Connect is disabled for your organization. Contact support for more details."
+            )
+        else:
+            self._log.warning(
+                "DEPRECATION WARNING: Anyscale Connect will be deprecated in future Anyscale versions. "
+                "Please use workspaces: "
+                "https://docs.anyscale.com/user-guide/develop-and-debug/workspaces or Ray Jobs "
+                "https://docs.anyscale.com/user-guide/develop-and-debug/ray-jobs-support instead."
+            )
+
         _allow_multiple_clients = (
             os.environ.get("ANYSCALE_ALLOW_MULTIPLE_CLIENTS") != "0"
         )
@@ -863,8 +864,7 @@ class ClientBuilder:
         self._prepare_cluster_block = create_prepare_cluster_block(
             project_id=project_id,
             cluster_name=self._cluster_name,
-            autosuspend_timeout=self._autosuspend_timeout
-            or DEFAULT_AUTOSUSPEND_TIMEOUT,
+            autosuspend_timeout=self._autosuspend_timeout,
             allow_public_internet_traffic=self._allow_public_internet_traffic,
             needs_update=self._needs_update,
             cluster_compute_name=self._cluster_compute_name,
@@ -1124,7 +1124,9 @@ class ClientBuilder:
             raise TypeError("The type of cluster_env must be either a str or a dict.")
         return self
 
-    def download_results(self, *, remote_dir: str, local_dir: str) -> None:
+    def download_results(
+        self, *, remote_dir: str, local_dir: str  # NOQA: ARG002
+    ) -> None:
         """Specify a directory to sync down from the cluster head node.
 
         IMPORTANT: the data is downloaded immediately after this call.
@@ -1149,7 +1151,7 @@ class ClientBuilder:
                 'to a cluster via ray.client("anyscale://...").connect()'
             )
 
-        self._download_results(remote_dir, local_dir)
+        raise RuntimeError("Downloading results is not supported on anyscale V2")
 
     def autosuspend(
         self,
@@ -1175,7 +1177,7 @@ class ClientBuilder:
         """
         if enabled:
             if hours is None and minutes is None:
-                timeout = DEFAULT_AUTOSUSPEND_TIMEOUT
+                timeout = None
             else:
                 timeout = 0
                 if hours is not None:
@@ -1278,23 +1280,6 @@ class ClientBuilder:
         self._initial_scale = to_request
         return self
 
-    def _download_results(self, remote_dir: str, local_dir: str) -> None:
-        # Determine the cluster's cluster ID by querying for the Anyscale Environment Variable
-        # that is set on cluster startup. We achieve this by running a remote function against the cluster
-        # using Ray Client.
-        try:
-            ray_id = self._ray.remote(
-                lambda: os.environ.get("ANYSCALE_SESSION_ID")
-            ).remote()
-            cluster_id = self._ray.get(ray_id)
-            cluster_name = self._anyscale_sdk.get_session(cluster_id).result.name
-        except Exception as e:  # noqa: BLE001
-            self._log.error(f"Failed to get ClusterID with error: {e}")
-            raise RuntimeError(
-                "Unable to get ClusterID for `download_results`."
-            ) from None
-        self._session_controller.pull(cluster_name, source=remote_dir, target=local_dir)
-
     def _get_cluster_or_die(self, project_id: str, session_name: str) -> Session:
         """Query Anyscale for the given cluster's metadata."""
         cluster_found = get_cluster(self._anyscale_sdk, project_id, session_name)
@@ -1302,39 +1287,6 @@ class ClientBuilder:
             raise RuntimeError(f"Failed to locate cluster: {session_name}")
         return cluster_found
 
-    def _exec_self_in_head_node(self) -> None:
-        """Run the current main file in the head node."""
-        cur_file = os.path.abspath(sys.argv[0])
-        # TODO(ekl) it would be nice to support keeping the original file name,
-        # but "anyscale push" isn't escaping e.g., spaces correctly.
-        tmp_file = f"/tmp/anyscale-connect-{uuid.uuid4().hex}.py"
-        cur_dir = os.getcwd()
-        try:
-            assert self._project_dir is not None
-            os.chdir(self._project_dir)
-
-            self._session_controller.push(
-                self._cluster_name,
-                source=cur_file,
-                target=tmp_file,
-                config=None,
-                all_nodes=False,
-            )
-
-            self._exec_controller.anyscale_exec(
-                self._cluster_name,  # type: ignore
-                screen=False,
-                tmux=False,
-                port_forward=(),  # type: ignore
-                sync=False,
-                stop=False,
-                terminate=False,
-                commands=["python", tmp_file] + sys.argv[1:],
-            )
-        finally:
-            os.chdir(cur_dir)
-        self._os._exit(0)  # pylint:disable=private-use  # noqa: SLF001
-
     def _exec_self_in_local_docker(self) -> None:
         """Run the current main file in a local docker image."""
         cur_file = os.path.abspath(sys.argv[0])
