@@ -10,11 +10,7 @@ from anyscale.authenticate import get_auth_api_client
 from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client.models.build import Build
 from anyscale.client.openapi_client.models.session import Session
-from anyscale.cloud import (
-    get_cloud_id_and_name,
-    get_last_used_cloud,
-    get_organization_default_cloud,
-)
+from anyscale.cluster_compute import get_selected_cloud_id_or_default
 from anyscale.conf import MINIMUM_RAY_VERSION
 from anyscale.links import DOCS_CLUSTER
 from anyscale.sdk.anyscale_client import (
@@ -55,6 +51,8 @@ BUILD_STEPS = [
     "pip uninstall -y ray",
 ]
 
+# Default minutes for autosuspend.
+DEFAULT_AUTOSUSPEND_TIMEOUT = 120
 
 # Default docker images to use for connect clusters.
 def _get_base_image(image: str, ray_version: str, cpu_or_gpu: str) -> str:
@@ -77,7 +75,7 @@ class PrepareClusterBlock:
         self,
         project_id: str,
         cluster_name: Optional[str],
-        autosuspend_timeout: int,
+        autosuspend_timeout: Optional[int],
         allow_public_internet_traffic: Optional[bool],
         needs_update: bool,
         cluster_compute_name: Optional[str],
@@ -156,7 +154,11 @@ class PrepareClusterBlock:
             else:
                 build_id = existing_terminated_cluster.build_id
 
-            cloud_id = self._get_cloud_id(self.project_id, self.cloud_name)
+            cloud_id = get_selected_cloud_id_or_default(
+                api_client=self.api_client,
+                anyscale_api_client=self.anyscale_api_client,
+                cloud_name=self.cloud_name,
+            )
             if (
                 (existing_terminated_cluster is not None)
                 and (existing_terminated_cluster.cloud_id is not None)
@@ -246,7 +248,7 @@ class PrepareClusterBlock:
         build_id: str,
         compute_template_id: str,
         cluster_name: Optional[str],
-        autosuspend_timeout: int,
+        autosuspend_timeout: Optional[int],
         allow_public_internet_traffic: Optional[bool],
     ) -> str:
         """Create/Start a cluster based on its current state and passed args.
@@ -258,7 +260,7 @@ class PrepareClusterBlock:
             cluster_name (Optional[str]): If specified, the given cluster
                 will be created or updated as needed. Otherwise the cluster
                 name will be picked automatically.
-            autosuspend_timeout (int): Autosuspend value of cluster
+            autosuspend_timeout (Optional[int]): Autosuspend value of cluster
             allow_public_internet_traffic (bool): Whether to allow public internet
                 traffic to serve endpoints
         Returns:
@@ -462,6 +464,9 @@ class PrepareClusterBlock:
                 f"Starting cluster {BlockLogger.highlight(cluster_name)}:",
                 block_label=self.block_label,
             )
+            # User didn't specify idle timeout, use the default autosuspend timeout
+            if idle_timeout is None:
+                idle_timeout = DEFAULT_AUTOSUSPEND_TIMEOUT
             create_cluster_data = CreateCluster(
                 name=cluster_name,
                 project_id=project_id,
@@ -783,7 +788,11 @@ class PrepareClusterBlock:
                 cluster_compute_class = ComputeTemplateConfig(**cluster_compute_dict)
                 config_object = cluster_compute_class
             else:
-                cloud_id = self._get_cloud_id(project_id, cloud_name)
+                cloud_id = get_selected_cloud_id_or_default(
+                    api_client=self.api_client,
+                    anyscale_api_client=self.anyscale_api_client,
+                    cloud_name=cloud_name,
+                )
                 config_object = self.anyscale_api_client.get_default_compute_config(
                     cloud_id
                 ).result
@@ -844,21 +853,6 @@ class PrepareClusterBlock:
             self.log.debug(f"Error comparing cluster compute: {e}")
             return False
 
-    def _get_cloud_id(self, project_id: str, cloud_name: Optional[str]) -> str:
-        """Returns the cloud id from cloud name.
-        If cloud name is not provided, returns the default cloud name if exists in organization.
-        If default cloud name does not exist returns last used cloud.
-        """
-
-        if cloud_name is None:
-            default_cloud_name = get_organization_default_cloud(self.api_client)
-            if default_cloud_name:
-                cloud_name = default_cloud_name
-            else:
-                cloud_name = get_last_used_cloud(project_id, self.anyscale_api_client)
-        cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_name=cloud_name)
-        return cloud_id
-
     def _validate_new_cluster_compute_and_env_match_existing_cluster(
         self, project_id: str, running_cluster: Session, print_warnings: bool = True
     ) -> bool:
@@ -942,7 +936,7 @@ class PrepareClusterBlock:
 def create_prepare_cluster_block(  # noqa: PLR0913
     project_id: str,
     cluster_name: Optional[str],
-    autosuspend_timeout: int,
+    autosuspend_timeout: Optional[int],
     allow_public_internet_traffic: Optional[bool],
     needs_update: bool,
     cluster_compute_name: Optional[str],
