@@ -10,10 +10,16 @@ class BaseController:
         - Authenticating and getting internal and external API clients
     """
 
-    def __init__(self, initialize_auth_api_client: bool = True) -> None:
+    def __init__(
+        self,
+        initialize_auth_api_client: bool = True,
+        raise_structured_exception: bool = False,
+    ) -> None:
         self.initialize_auth_api_client = initialize_auth_api_client
         if self.initialize_auth_api_client:
-            self.auth_api_client = get_auth_api_client()
+            self.auth_api_client = get_auth_api_client(
+                raise_structured_exception=raise_structured_exception
+            )
             self.api_client = self.auth_api_client.api_client
             self.anyscale_api_client = self.auth_api_client.anyscale_api_client
 
