@@ -7,9 +7,11 @@ import tabulate
 import yaml
 
 from anyscale.cli_logger import BlockLogger
+from anyscale.cloud import get_cloud_id_and_name, get_last_used_cloud
 from anyscale.cluster_compute import (
     get_cluster_compute_from_name,
     get_default_cluster_compute,
+    get_selected_cloud_id_or_default,
 )
 from anyscale.cluster_env import (
     get_build_from_cluster_env_identifier,
@@ -59,7 +61,7 @@ class ClusterController(BaseController):
         self.log = log
         self.log.open_block("Output")
 
-    def start(  # noqa: PLR0913
+    def start(  # noqa: PLR0912, PLR0913
         self,
         cluster_name: Optional[str],
         cluster_id: Optional[str],
@@ -109,9 +111,35 @@ class ClusterController(BaseController):
                 "`--ray-version` should be specified when `--docker` is used."
             )
 
-        project_id, cluster_name = self._get_project_id_and_cluster_name(
-            cluster_id, project_id, cluster_name, project_name
+        # Try to get project id and cluster name of cluster per arguments to
+        # see if the cluster already exists. If the cluster doesn't already exist,
+        # get the project id and cluster name of a cluster to create.
+        cluster_compute_id, cluster_compute_config = None, None
+        if cluster_compute_name:
+            cluster_compute_id = get_cluster_compute_from_name(
+                cluster_compute_name, self.api_client
+            ).id
+        elif cluster_compute_file:
+            cluster_compute_config = self._load_cluster_compute(cluster_compute_file)
+        # parent_cloud_id used to search for existing cluster in default project
+        # if cloud isolation is enabled and we are unable to find the cluster with the
+        # provided project_id.
+        parent_cloud_id = get_selected_cloud_id_or_default(
+            self.api_client,
+            self.anyscale_api_client,
+            cluster_compute_id=cluster_compute_id,
+            cluster_compute_config=cluster_compute_config,
+            cloud_id=None,
+            cloud_name=cloud_name,
         )
+        project_id, cluster_name = self._get_project_id_and_cluster_name(
+            cluster_id,
+            project_id,
+            cluster_name,
+            project_name,
+            cloud_id=parent_cloud_id,
+        )
+
         cluster_list = self.anyscale_api_client.search_clusters(
             {
                 "project_id": project_id,
@@ -139,9 +167,9 @@ class ClusterController(BaseController):
             existing_cluster,
             project_id,
         )
-        cloud_name = self.anyscale_api_client.get_cloud(
+        cloud = self.anyscale_api_client.get_cloud(
             cluster_compute.config.cloud_id
-        ).result.name
+        ).result
 
         # Whether configurations that could require a restart were passed
         passed_cluster_env = bool(cluster_env_name)
@@ -171,7 +199,7 @@ class ClusterController(BaseController):
             f"\t\t\tCluster environment: {cluster_environment_display}\n"
             f"\t\t\tCluster compute: {cluster_compute.name} (id={cluster_compute.id})\n"
             f"\t\t\tProject: {project_id}\n"
-            f"\t\t\tCloud: {cloud_name}\n"
+            f"\t\t\tCloud: {cloud.name}\n"
             f"\t\t\tURL: {url}"
         )
         if needs_start:
@@ -201,18 +229,18 @@ class ClusterController(BaseController):
         cluster_id: Optional[str],
         project_id: Optional[str],
         project_name: Optional[str],
+        cloud_id: Optional[str],
+        cloud_name: Optional[str],
     ) -> None:
-        if cluster_name is not None and cluster_id is not None:
-            raise click.ClickException(
-                "`cluster-name` and `--cluster-id` cannot both be specified. Please only provide one"
-                "of these two arguments."
-            )
-        if cluster_name is None and cluster_id is None:
-            raise click.ClickException(
-                "Please specity one of `cluster-name` or `--cluster-id`."
+        self._validate_args_to_find_cluster(
+            cluster_name, cluster_id, project_name, project_id, cloud_name, cloud_id
+        )
+        if cloud_id or cloud_name:
+            cloud_id, _ = get_cloud_id_and_name(
+                api_client=self.api_client, cloud_id=cloud_id, cloud_name=cloud_name,
             )
         project_id, cluster_name = self._get_project_id_and_cluster_name(
-            cluster_id, project_id, cluster_name, project_name
+            cluster_id, project_id, cluster_name, project_name, cloud_id=cloud_id
         )
 
         cluster_list = self.anyscale_api_client.search_clusters(
@@ -235,18 +263,18 @@ class ClusterController(BaseController):
         cluster_id: Optional[str],
         project_id: Optional[str],
         project_name: Optional[str],
+        cloud_id: Optional[str],
+        cloud_name: Optional[str],
     ) -> None:
-        if cluster_name is not None and cluster_id is not None:
-            raise click.ClickException(
-                "`--name` and `--cluster-id` cannot both be specified. Please only provide one "
-                "of these two arguments."
-            )
-        if cluster_name is None and cluster_id is None:
-            raise click.ClickException(
-                "Please specity one of `--name` or `--cluster-id`."
+        self._validate_args_to_find_cluster(
+            cluster_name, cluster_id, project_name, project_id, cloud_name, cloud_id
+        )
+        if cloud_id or cloud_name:
+            cloud_id, _ = get_cloud_id_and_name(
+                api_client=self.api_client, cloud_id=cloud_id, cloud_name=cloud_name,
             )
         project_id, cluster_name = self._get_project_id_and_cluster_name(
-            cluster_id, project_id, cluster_name, project_name
+            cluster_id, project_id, cluster_name, project_name, cloud_id
         )
 
         cluster_list = self.anyscale_api_client.search_clusters(
@@ -272,6 +300,8 @@ class ClusterController(BaseController):
         include_archived: bool,
         max_items: int,
         project_name: Optional[str],
+        cloud_id: Optional[str],
+        cloud_name: Optional[str],
     ) -> None:
         if (
             project_id is not None or project_name is not None
@@ -294,8 +324,17 @@ class ClusterController(BaseController):
             )
         elif cluster_name is not None:
             # Find cluster name in project if cluster_name specified
+            self._validate_args_to_find_cluster(
+                cluster_name, cluster_id, project_name, project_id, cloud_name, cloud_id
+            )
+            if cloud_id or cloud_name:
+                cloud_id, _ = get_cloud_id_and_name(
+                    api_client=self.api_client,
+                    cloud_id=cloud_id,
+                    cloud_name=cloud_name,
+                )
             project_id, cluster_name = self._get_project_id_and_cluster_name(
-                cluster_id, project_id, cluster_name, project_name
+                cluster_id, project_id, cluster_name, project_name, cloud_id=cloud_id,
             )
             cluster_list.extend(
                 self.api_client.list_decorated_clusters_api_v2_decorated_sessions_get(
@@ -315,13 +354,28 @@ class ClusterController(BaseController):
                         project_definition = load_project_or_throw()
                         project_id = get_project_id(project_definition.root)
                     except click.ClickException:
-                        default_project = (
-                            self.anyscale_api_client.get_default_project().result
-                        )
+                        parent_cloud_id = None
+                        feature_flag_on = self.api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
+                            "cloud-isolation-phase-1"
+                        ).result.is_on
+                        log_msg = "No project context detected or `--project-id` provided. Continuing without a project."
+                        if feature_flag_on:
+                            if not (cloud_id or cloud_name):
+                                cloud_name = get_last_used_cloud(
+                                    project_id=None,
+                                    anyscale_api_client=self.anyscale_api_client,
+                                )
+                            parent_cloud_id, _ = get_cloud_id_and_name(
+                                api_client=self.api_client,
+                                cloud_id=cloud_id,
+                                cloud_name=cloud_name,
+                            )
+                            log_msg = f"No project context detected or `--project-id` provided. Continuing without a project assuming cloud {parent_cloud_id}."
+                        default_project = self.anyscale_api_client.get_default_project(
+                            parent_cloud_id=parent_cloud_id
+                        ).result
                         project_id = default_project.id
-                        self.log.info(
-                            "No project context detected or `--project-id` provided. Continuing without a project."
-                        )
+                        self.log.info(log_msg)
                 search_clusters_query["project_id"] = project_id
             if not include_inactive:
                 search_clusters_query["state_filter"] = ["Running"]
@@ -376,12 +430,64 @@ class ClusterController(BaseController):
 
     # Helpers
 
+    def _validate_args_to_find_cluster(
+        self,
+        cluster_name: Optional[str],
+        cluster_id: Optional[str],
+        project_name: Optional[str],
+        project_id: Optional[str],
+        cloud_name: Optional[str],
+        cloud_id: Optional[str],
+    ):
+        """
+        Validate arguments to ensure we can find the cluster to take action on
+        (eg: for commands like archive and terminate).
+        """
+        if cluster_name is not None and cluster_id is not None:
+            raise click.ClickException(
+                "`--name` and `--cluster-id` cannot both be specified. Please only provide one "
+                "of these two arguments."
+            )
+        if cluster_name is None and cluster_id is None:
+            raise click.ClickException(
+                "Please specity one of `--name` or `--cluster-id`."
+            )
+        if cloud_name is not None and cloud_id is not None:
+            raise click.ClickException(
+                "`--cloud-name` and `--cloud-id` cannot both be specified. Please only provide one "
+                "of these two arguments."
+            )
+        feature_flag_on = self.api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
+            "cloud-isolation-phase-1"
+        ).result.is_on
+        if (
+            feature_flag_on
+            and cluster_name is not None
+            and (
+                project_id is None
+                and project_name is None
+                and cloud_id is None
+                and cloud_name is None
+            )
+        ):
+            raise click.ClickException(
+                "Please specify the project of the cluster through `--project-id` or `--project-name`. If the cluster "
+                "doesn't belong to any project, please specify the cloud id with `--cloud-id`."
+            )
+
+    def _load_cluster_compute(self, cluster_compute_file: str) -> ClusterComputeConfig:
+        with open(cluster_compute_file) as f:
+            config_dict: Dict[str, Any] = yaml.safe_load(f)
+        cluster_compute_config = ClusterComputeConfig(**config_dict)
+        return cluster_compute_config
+
     def _get_project_id_and_cluster_name(
         self,
         cluster_id: Optional[str],
         project_id: Optional[str],
         cluster_name: Optional[str],
         project_name: Optional[str],
+        cloud_id: Optional[str],
     ) -> Tuple[str, str]:
         """
         Get cluster name and project id. If cluster id is specified, any cluster can
@@ -400,6 +506,7 @@ class ClusterController(BaseController):
             project_id = get_and_validate_project_id(
                 project_id=project_id,
                 project_name=project_name,
+                parent_cloud_id=cloud_id,
                 api_client=self.api_client,
                 anyscale_api_client=self.anyscale_api_client,
             )
@@ -429,16 +536,14 @@ class ClusterController(BaseController):
         cluster_compute_file: Optional[str],
         cloud_name: Optional[str],
         existing_cluster: Optional[Cluster],
-        project_id: str,
+        project_id: Optional[str],
     ) -> ComputeTemplate:
         if cluster_compute_name:
             cluster_compute = get_cluster_compute_from_name(
                 cluster_compute_name, self.api_client
             )
         elif cluster_compute_file:
-            with open(cluster_compute_file) as f:
-                config_dict: Dict[str, Any] = yaml.safe_load(f)
-            cluster_compute_config = ClusterComputeConfig(**config_dict)
+            cluster_compute_config = self._load_cluster_compute(cluster_compute_file)
             name = f"cli-config-{datetime.now().isoformat()}"
             cluster_compute = self.anyscale_api_client.create_cluster_compute(
                 CreateClusterCompute(name=name, config=cluster_compute_config)
