@@ -17,14 +17,16 @@ from anyscale.client.openapi_client.models.schedule_config import (
     ScheduleConfig as APIScheduleConfig,
 )
 from anyscale.controllers.base_controller import BaseController
-from anyscale.controllers.job_controller import JobConfig
+from anyscale.models.job_model import JobConfig
+from anyscale.project import infer_project_id
 from anyscale.tables import SchedulesTable
 from anyscale.util import (
     AnyscaleEndpointFormatter,
     populate_dict_with_workspace_config_if_exists,
     validate_job_config_dict,
 )
-from anyscale.utils.runtime_env import override_runtime_env_for_local_working_dir
+from anyscale.utils.runtime_env import override_runtime_env_config
+from anyscale.utils.workload_types import Workload
 
 
 log = LogsLogger()
@@ -53,10 +55,6 @@ class ScheduleConfig(BaseModel):
 class CreateScheduleConfig(JobConfig):
     schedule: ScheduleConfig
 
-    @validator("runtime_env")
-    def rewrite_runtime_env(cls, runtime_env):
-        return override_runtime_env_for_local_working_dir(runtime_env)
-
 
 def load_yaml_file_as_dict(config_file: str) -> Dict[str, Any]:
     if not os.path.exists(config_file):
@@ -125,8 +123,28 @@ class ScheduleController(BaseController):
         schedule_config_dict = populate_dict_with_workspace_config_if_exists(
             schedule_config_dict, self.anyscale_api_client
         )
+        schedule_config: CreateScheduleConfig = CreateScheduleConfig.parse_obj(
+            schedule_config_dict
+        )
+
+        schedule_config.project_id = infer_project_id(
+            self.anyscale_api_client,
+            self.api_client,
+            self.log,
+            project_id=schedule_config.project_id,
+            cluster_compute_id=schedule_config.compute_config_id,
+            cluster_compute=schedule_config.compute_config,
+            cloud=schedule_config.cloud,
+        )
+        schedule_config.runtime_env = override_runtime_env_config(
+            runtime_env=schedule_config.runtime_env,
+            anyscale_api_client=self.anyscale_api_client,
+            api_client=self.api_client,
+            workload_type=Workload.SCHEDULED_JOBS,
+            compute_config_id=schedule_config.compute_config_id,
+            log=self.log,
+        )
 
-        schedule_config = CreateScheduleConfig.parse_obj(schedule_config_dict)
         return schedule_config
 
     def resolve_file_name_or_id(
