@@ -1,5 +1,5 @@
 import os
-from typing import Any, Dict, List, Optional
+from typing import List, Optional
 
 import click
 import tabulate
@@ -10,17 +10,30 @@ from anyscale.client.openapi_client import ComputeTemplate, ProductionJob
 from anyscale.client.openapi_client.models.decorated_list_service_api_model import (
     DecoratedListServiceAPIModel,
 )
+from anyscale.client.openapi_client.models.decorated_production_job import (
+    DecoratedProductionJob,
+)
+from anyscale.client.openapi_client.models.decorated_production_service_v2_api_model import (
+    DecoratedProductionServiceV2APIModel,
+)
+from anyscale.client.openapi_client.models.ha_job_states import HaJobStates
 from anyscale.controllers.base_controller import BaseController
 from anyscale.controllers.job_controller import JobController
-from anyscale.formatters.service_formatter import (
-    format_service_config,
-    format_service_config_v2,
-)
+from anyscale.formatters.service_formatter import format_service_config_v2
 from anyscale.models.service_model import ServiceConfig
 from anyscale.project import infer_project_id
 from anyscale.tables import ServicesTable
-from anyscale.util import get_endpoint, is_anyscale_workspace
-from anyscale.utils.runtime_env import override_runtime_env_for_local_working_dir
+from anyscale.util import (
+    get_endpoint,
+    poll,
+    populate_dict_with_workspace_config_if_exists,
+)
+from anyscale.utils.runtime_env import override_runtime_env_config
+from anyscale.utils.workload_types import Workload
+
+
+_POLL_SERVICE_VERSION_INFO_INTERVAL_IN_SECS = 3.0
+_POLL_SERVICE_VERSION_INFO_TIMEOUT_IN_SECS = 60.0 * 3
 
 
 def _is_v2_service_id(service_id: str) -> bool:
@@ -148,85 +161,23 @@ class ServiceController(BaseController):
 
         return service_id
 
-    def deploy(
-        self,
-        service_config_file: str,
-        name: Optional[str] = None,
-        description: Optional[str] = None,
-        healthcheck_url: Optional[str] = None,
-        is_entrypoint_cmd: Optional[bool] = False,
-        entrypoint: Optional[List[str]] = None,
-    ) -> None:
-        entrypoint = entrypoint or []
-        if is_anyscale_workspace() and is_entrypoint_cmd:
-            entrypoint = [service_config_file, *entrypoint]
-            config = self.generate_config_from_entrypoint(
-                entrypoint,
-                name=name,
-                description=description,
-                healthcheck_url=healthcheck_url,
-            )
-            project_id = infer_project_id(
-                config.project_id, self.anyscale_api_client, self.log
-            )
-
-            config.runtime_env = override_runtime_env_for_local_working_dir(
-                config.runtime_env
-            )
-            config.project_id = project_id
-            self.deploy_from_config(config)
-        elif len(entrypoint) == 0:
-            # Assume that service_config_file is a file and submit it.
-            config = self.generate_config_from_file(
-                service_config_file,
-                name=name,
-                description=description,
-                healthcheck_url=healthcheck_url,
-            )
-
-            project_id = infer_project_id(
-                config.project_id, self.anyscale_api_client, self.log
-            )
-
-            config.runtime_env = override_runtime_env_for_local_working_dir(
-                config.runtime_env
-            )
-            config.project_id = project_id
-            self.deploy_from_config(config)
-        elif len(entrypoint) != 0:
-            msg = (
-                "Within an Anyscale Workspace, `anyscale service deploy` takes either a file, or a command. To submit a command, use `anyscale service deploy -- my command`."
-                if is_anyscale_workspace()
-                else "`anyscale service deploy` takes one argument, a YAML file configuration. Please use `anyscale service deploy my_file`."
-            )
-            raise click.ClickException(msg)
-
-    def generate_config_from_entrypoint(
-        self,
-        entrypoint: List[str],
-        name: Optional[str] = None,
-        description: Optional[str] = None,
-        healthcheck_url: Optional[str] = "/healthcheck",
-    ) -> ServiceConfig:
-        config_dict = {
-            "entrypoint": " ".join(entrypoint),
-            "name": name,
-            "description": description,
-            "healthcheck_url": healthcheck_url,
-        }
-        return self._populate_service_config(config_dict)
-
     def generate_config_from_file(
         self,
         service_config_file,
         *,
         name: Optional[str] = None,
         description: Optional[str] = None,
-        healthcheck_url: Optional[str] = None,
         version: Optional[str] = None,
         canary_percent: Optional[int] = None,
         rollout_strategy: Optional[str] = None,
     ) -> ServiceConfig:
+        """
+        Given the input file path and user overrides, this method
+        constructs the expected service configuration.
+
+        Note that if the command is run from a workspace, some values
+        may be overwritten.
+        """
         service_config: ServiceConfig = self._read_service_config_file(
             service_config_file
         )
@@ -236,14 +187,11 @@ class ServiceController(BaseController):
         if description:
             service_config.description = description
 
-        if healthcheck_url:
-            service_config.healthcheck_url = healthcheck_url
-
         if version:
             service_config.version = version
 
-        if canary_percent:
-            service_config.canary_weight = canary_percent
+        if canary_percent is not None:
+            service_config.canary_percent = canary_percent
 
         if rollout_strategy:
             service_config.rollout_strategy = rollout_strategy
@@ -253,37 +201,6 @@ class ServiceController(BaseController):
 
         return service_config
 
-    def deploy_from_config(self, service_config: ServiceConfig):
-        apply_service_config = format_service_config(service_config)
-        # TODO: shawnp. Change to the new Services API
-        # once the new endpoint works for both Service v1 and v2.
-        create_production_service = apply_service_config.service_v1_config
-        if create_production_service:
-            service = self.api_client.apply_service_api_v2_decorated_ha_jobs_apply_service_put(
-                create_production_service
-            ).result
-        else:
-            # TODO: shawnp. This condition should never occur
-            raise RuntimeError("The Service configuration is missing")
-        service_id = service.id
-        current_state = service.state.current_state
-
-        maximum_uptime_minutes = self._get_maximum_uptime_minutes(service)
-        self.log.info(
-            f"Maximum uptime is {self._get_maximum_uptime_output(maximum_uptime_minutes)} for clusters launched by this service."
-            f"{self._get_additional_log_if_maximum_uptime_enabled(maximum_uptime_minutes)}"
-        )
-
-        self.log.info(
-            f"Service {service_id} has been deployed. Current state of service: {current_state}."
-        )
-        self.log.info(
-            f"Query the status of the service with `anyscale service list --service-id {service.id}`."
-        )
-        self.log.info(
-            f'View the service in the UI at {get_endpoint(f"/services/{service.id}")}.'
-        )
-
     def _get_maximum_uptime_minutes(self, service: ProductionJob) -> Optional[int]:
         compute_config: ComputeTemplate = self.api_client.get_compute_template_api_v2_compute_templates_template_id_get(
             service.config.compute_config_id
@@ -302,27 +219,6 @@ class ServiceController(BaseController):
             return " This may cause disruptions. To disable, update the compute config."
         return ""
 
-    def _populate_service_config(self, config_dict: Dict[str, Any]) -> ServiceConfig:
-        if (
-            "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID" in os.environ
-            and "ANYSCALE_SESSION_ID" in os.environ
-        ):
-            cluster = self.anyscale_api_client.get_cluster(
-                os.environ["ANYSCALE_SESSION_ID"]
-            ).result
-            # If the job configs are not specified, infer them from the workspace:
-            if "build_id" not in config_dict and "cluster_env" not in config_dict:
-                config_dict["build_id"] = cluster.cluster_environment_build_id
-            if "project_id" not in config_dict:
-                config_dict["project_id"] = cluster.project_id
-            if (
-                "compute_config" not in config_dict
-                and "compute_config_id" not in config_dict
-            ):
-                config_dict["compute_config_id"] = cluster.cluster_compute_id
-        service_config = ServiceConfig.parse_obj(config_dict)
-        return service_config
-
     def _read_service_config_file(self, service_config_file: str) -> ServiceConfig:
         if not os.path.exists(service_config_file):
             raise click.ClickException(f"Config file {service_config_file} not found.")
@@ -330,7 +226,10 @@ class ServiceController(BaseController):
         with open(service_config_file) as f:
             config_dict = yaml.safe_load(f)
 
-        return self._populate_service_config(config_dict)
+        updated_config_dict = populate_dict_with_workspace_config_if_exists(
+            config_dict, self.anyscale_api_client
+        )
+        return ServiceConfig.parse_obj(updated_config_dict)
 
     def rollout(
         self,
@@ -350,21 +249,121 @@ class ServiceController(BaseController):
             canary_percent=canary_percent,
             rollout_strategy=rollout_strategy,
         )
-        config.runtime_env = override_runtime_env_for_local_working_dir(
-            config.runtime_env
+
+        config.project_id = infer_project_id(
+            self.anyscale_api_client,
+            self.api_client,
+            self.log,
+            project_id=config.project_id,
+            cluster_compute_id=config.compute_config_id,
+            cluster_compute=config.compute_config,
+            cloud=config.cloud,
         )
+
+        if not config.ray_serve_config:
+            config.ray_serve_config = {}
+
+        self._overwrite_runtime_env_in_v2_ray_serve_config(config)
+
         service_v2_config = format_service_config_v2(config)
+
         service = self.api_client.apply_service_v2_api_v2_services_v2_apply_put(
             service_v2_config
         ).result
 
+        current_state = service.goal_state
         self.log.info(
-            f"Service {service.id} has been deployed. Service is transitioning towards: {service.goal_state}."
+            f"Service {service.id} has been deployed. Service is transitioning towards: {current_state}."
         )
         self.log.info(
-            f'View the service in the UI at {get_endpoint(f"/services/{service.id}")}.'
+            f'View the service in the UI at {get_endpoint(f"/services/{service.id}")}'
         )
 
+        if config.canary_percent is not None and len(service.versions) > 1:
+            if config.canary_percent == 100:
+                self.log.warning(
+                    "The canary percent has been set to 100%. NOTE that this will not complete the rollout. "
+                    "To complete the rollout, you will need to trigger an automatic rollout for the service. "
+                    "This can be achieved by removing the canary percent flag and executing the rollout command again."
+                )
+            self._log_canary_version_query(service)
+
+    def _overwrite_runtime_env_in_v2_ray_serve_config(self, config: ServiceConfig):
+        """Modifies config in place."""
+        ray_serve_config = config.ray_serve_config
+        if "applications" in ray_serve_config:
+            for ray_serve_app_config in ray_serve_config["applications"]:
+                ray_serve_app_config["runtime_env"] = override_runtime_env_config(
+                    runtime_env=ray_serve_app_config.get("runtime_env"),
+                    anyscale_api_client=self.anyscale_api_client,
+                    api_client=self.api_client,
+                    workload_type=Workload.SERVICES,
+                    compute_config_id=config.compute_config_id,
+                    log=self.log,
+                )
+
+        else:
+            ray_serve_config["runtime_env"] = override_runtime_env_config(
+                runtime_env=ray_serve_config.get("runtime_env"),
+                anyscale_api_client=self.anyscale_api_client,
+                api_client=self.api_client,
+                workload_type=Workload.SERVICES,
+                compute_config_id=config.compute_config_id,
+                log=self.log,
+            )
+
+    def _log_canary_version_query(self, service: DecoratedProductionServiceV2APIModel):
+        """
+        This method logs the canary version's query endpoint.
+
+        Since the url and token from the service object are not initially available,
+        we poll until they become available.
+        """
+        assert len(service.versions) > 1
+        # We assume versions are returned in order of creation.
+        canary_version = service.versions[1]
+        production_job_id = (
+            canary_version.production_job_ids[0]
+            if canary_version.production_job_ids
+            else None
+        )
+        if production_job_id:
+            canary_version_query_printed = False
+            for _ in poll(
+                interval_secs=_POLL_SERVICE_VERSION_INFO_INTERVAL_IN_SECS,
+                timeout_secs=_POLL_SERVICE_VERSION_INFO_TIMEOUT_IN_SECS,
+            ):
+                try:
+                    service_v1 = self.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
+                        production_job_id
+                    ).result
+                    url = service_v1.url
+                    token = service_v1.token
+                    service_version_current_state = service_v1.state.current_state
+                    if url and token:
+                        self.log.info()
+                        self.log.info(
+                            "You can query the service endpoint using the curl request below:"
+                        )
+                        self.log.info(f"curl -H 'Authorization: Bearer {token}' {url}")
+
+                        if service_version_current_state in {
+                            HaJobStates.PENDING,
+                            HaJobStates.AWAITING_CLUSTER_START,
+                        }:
+                            self.log.info(
+                                "Please note the canary version is still pending and may take several minutes to begin successfully returning requests."
+                            )
+                        canary_version_query_printed = True
+                        break
+                except Exception:  # noqa: BLE001
+                    # We keep re-retrying until the timeout
+                    pass
+            if not canary_version_query_printed:
+                self.log.info(
+                    "Please refer to the Service details page if you would like to query the canary version."
+                )
+
     def list(
         self,
         *,
@@ -414,49 +413,55 @@ class ServiceController(BaseController):
         services = []
         if service_id:
             if _is_v2_service_id(service_id):
-                service = self.api_client.get_service_api_v2_services_v2_service_id_get(
-                    service_id
-                ).result
-                services.append(service)
-            else:
-                service = self.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
-                    service_id
-                ).result
-                # We short-circuit if service v1 if the model is different from service v2
-                service_v1_res = [
+                service_v2_result: DecoratedProductionServiceV2APIModel = (
+                    self.api_client.get_service_api_v2_services_v2_service_id_get(
+                        service_id
+                    ).result
+                )
+                services.append(
                     [
-                        service.name,
-                        service.id,
-                        service.state.current_state,
-                        service.creator.email,
+                        service_v2_result.name,
+                        service_v2_result.id,
+                        service_v2_result.current_state,
+                        # TODO: change to email once https://github.com/anyscale/product/pull/18189 is merged
+                        service_v2_result.creator_id,
                     ]
-                ]
+                )
+            else:
+                service_v1_result: DecoratedProductionJob = self.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
+                    service_id
+                ).result
+                services.append(
+                    [
+                        service_v1_result.name,
+                        service_v1_result.id,
+                        service_v1_result.state.current_state,
+                        service_v1_result.creator.email,
+                    ]
+                )
 
-                table = tabulate.tabulate(
-                    service_v1_res,
-                    headers=["NAME", "ID", "CURRENT STATE", "CREATOR"],
-                    tablefmt="plain",
-                )
-                self.log.info(
-                    f'View your Services in the UI at {get_endpoint("/services")}'
-                )
-                self.log.info(f"Services:\n{table}")
-                return
         else:
-            services = self._get_services_by_name(
+            services_data = self._get_services_by_name(
                 name=name,
                 project_id=project_id,
                 created_by_me=created_by_me,
                 max_items=max_items,
             )
 
-        # TODO: shawnp. Add creator username once that's exposed
-        services_table = [
-            [service.name, service.id, service.current_state,] for service in services
-        ]
+            services = [
+                [
+                    service.name,
+                    service.id,
+                    service.current_state,
+                    service.creator.email,
+                ]
+                for service in services_data
+            ]
 
         table = tabulate.tabulate(
-            services_table, headers=["NAME", "ID", "CURRENT STATE"], tablefmt="plain",
+            services,
+            headers=["NAME", "ID", "CURRENT STATE", "CREATOR"],
+            tablefmt="plain",
         )
         self.log.info(f'View your Services in the UI at {get_endpoint("/services")}')
         self.log.info(f"Services:\n{table}")
@@ -483,7 +488,7 @@ class ServiceController(BaseController):
 
         self.log.info(f"Service {service.id} rollback initiated.")
         self.log.info(
-            f'View the service in the UI at {get_endpoint(f"/services/{service.id}")}.'
+            f'View the service in the UI at {get_endpoint(f"/services/{service.id}")}'
         )
 
     def terminate(self, service_id: str) -> None:
