@@ -55,12 +55,12 @@ def get_gcp_secret(secret_name: str, **kwargs) -> ProtectedString:
         secret_name: Key of your secret
         kwargs: Optional credentials passed in to authenticate instance
     """
-    import google
+    from google import auth
 
     secretmanager = try_import_gcp_secretmanager()
 
     client = secretmanager.SecretManagerServiceClient(**kwargs)
-    _, project_name = google.auth.default()
+    _, project_name = auth.default()
 
     name = f"projects/{project_name}/secrets/{secret_name}/versions/latest"
     response = client.access_secret_version(request={"name": name})
