@@ -1,5 +1,5 @@
 import os
-from typing import Any, Optional, Tuple
+from typing import Any, Dict, Optional, Tuple, Union
 
 import click
 from click import ClickException
@@ -10,6 +10,14 @@ from anyscale.authenticate import get_auth_api_client
 from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client import Project
 from anyscale.client.openapi_client.api.default_api import DefaultApi
+from anyscale.cloud import get_cloud_id_and_name
+from anyscale.cluster_compute import (
+    get_cluster_compute_from_name,
+    get_selected_cloud_id_or_default,
+)
+from anyscale.sdk.anyscale_client.models.cluster_compute_config import (
+    ClusterComputeConfig,
+)
 from anyscale.shared_anyscale_utils.util import slugify
 from anyscale.util import get_endpoint, PROJECT_NAME_ENV_VAR, send_json_request
 
@@ -231,6 +239,30 @@ def get_proj_id_from_name(
     return str(selected_project.id)
 
 
+def get_parent_cloud_id_and_name_of_project(
+    project_id: str, api_client: Optional[DefaultApi] = None,
+) -> Optional[Tuple[str, str]]:
+    """
+    Returns tuple (parent_cloud_id, parent_cloud_name) of project if cloud isolation is enabled.
+    Return (None, None) if cloud isolation isn't enabled or a project with this id doesn't exist.
+    """
+    if api_client is None:
+        api_client = get_auth_api_client().api_client
+
+    project = api_client.get_project_api_v2_projects_project_id_get(
+        project_id=project_id
+    ).result
+    cloud_isolation_ff_on = api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
+        "cloud-isolation-phase-1"
+    ).result.is_on
+    if project and cloud_isolation_ff_on:
+        parent_cloud_id, parent_cloud_name = get_cloud_id_and_name(
+            api_client, cloud_id=project.parent_cloud_id
+        )
+        return (parent_cloud_id, parent_cloud_name)
+    return None
+
+
 def write_project_file_to_disk(project_id: str, directory: str) -> None:
     with open(os.path.join(directory, ANYSCALE_PROJECT_FILE), "w") as f:
         f.write("{}".format(f"project_id: {project_id}"))
@@ -339,6 +371,7 @@ def validate_project_id(project_id, api_client: DefaultApi) -> Project:
 def get_and_validate_project_id(
     project_id: Optional[str],
     project_name: Optional[str],
+    parent_cloud_id: Optional[str],
     api_client: DefaultApi,
     anyscale_api_client: DefaultApi,
 ) -> str:
@@ -353,7 +386,9 @@ def get_and_validate_project_id(
             project_definition = load_project_or_throw()
             project_id = get_project_id(project_definition.root)
         except click.ClickException:
-            default_project = anyscale_api_client.get_default_project().result
+            default_project = get_default_project(
+                api_client, anyscale_api_client, parent_cloud_id=parent_cloud_id
+            )
             project_id = default_project.id
             log.info(
                 "No project context detected or `--project-id` provided. Continuing without a project."
@@ -362,16 +397,50 @@ def get_and_validate_project_id(
     return project_id
 
 
-def get_default_project(anyscale_api_client: Optional[DefaultApi] = None):
+def get_default_project(
+    api_client: Optional[DefaultApi] = None,
+    anyscale_api_client: Optional[DefaultApi] = None,
+    parent_cloud_id: Optional[str] = None,
+):
+    """
+    Gets the default project to use if no project has been selected.
+
+    parent_cloud_id: cloud_id to get a default project for. This field is
+        required if cloud isolation is enabled, because there are multiple
+        default projects in an organization, each associated with a
+        different cloud.
+    """
     anyscale_api_client = (
-        anyscale_api_client or get_auth_api_client().anyscale_api_client
+        anyscale_api_client or get_auth_api_client(log_output=False).anyscale_api_client
     )
-    default_project = anyscale_api_client.get_default_project().result
+    api_client = api_client or get_auth_api_client(log_output=False).api_client
+    if (
+        api_client.check_is_feature_flag_on_api_v2_userinfo_check_is_feature_flag_on_get(
+            "cloud-isolation-phase-1"
+        ).result.is_on
+        is False
+    ):
+        default_project = anyscale_api_client.get_default_project().result
+    else:
+        if not parent_cloud_id:
+            raise click.ClickException(
+                "Please specify a cloud for this command either through the cloud or "
+                "cluster compute arguments."
+            )
+        default_project = anyscale_api_client.get_default_project(
+            parent_cloud_id=parent_cloud_id
+        ).result
     return default_project
 
 
 def infer_project_id(
-    project_id: Optional[str], anyscale_api_client: DefaultApi, log: BlockLogger
+    anyscale_api_client: DefaultApi,
+    api_client: DefaultApi,
+    log: BlockLogger,
+    project_id: Optional[str],
+    cluster_compute_id: Optional[str] = None,
+    cluster_compute: Optional[Union[str, Dict[str, Any]]] = None,
+    cloud: Optional[str] = None,
 ) -> str:
     if not project_id:
         # Check directory of .anyscale.yaml to decide whether to use default project.
@@ -380,7 +449,27 @@ def infer_project_id(
             project_definition = ProjectDefinition(root_dir)
             project_id = get_project_id(project_definition.root)
         else:
-            default_project = anyscale_api_client.get_default_project().result
+            # TODO: add cloud compute config args here
+            if not cluster_compute_id and isinstance(cluster_compute, str):
+                cluster_compute_id = get_cluster_compute_from_name(
+                    cluster_compute, api_client
+                ).id
+            cluster_compute_config = (
+                ClusterComputeConfig(**cluster_compute)
+                if isinstance(cluster_compute, dict)
+                else None
+            )
+            parent_cloud_id = get_selected_cloud_id_or_default(
+                api_client,
+                anyscale_api_client,
+                cluster_compute_id=cluster_compute_id,
+                cluster_compute_config=cluster_compute_config,
+                cloud_id=None,
+                cloud_name=cloud,
+            )
+            default_project = get_default_project(
+                api_client, anyscale_api_client, parent_cloud_id=parent_cloud_id
+            )
             project_id = default_project.id
             log.info("No project specified. Continuing without a project.")
     return str(project_id)
