@@ -1,15 +1,26 @@
 import asyncio
 import contextlib
 import os
-import sys
+import logging
+import copy
 from time import sleep, time
 from typing import Any, Callable, Dict, List, Optional, Union
-import warnings
 
 from anyscale.api import configure_tcp_keepalive
-from anyscale.api_utils.job_logs_util import _get_logs_from_running_production_job, _get_job_logs_from_storage_bucket, _get_logs_from_running_job
-from anyscale.api_utils.job_util import _is_job_on_v2_cloud, _is_production_job_on_v2_cloud
-from anyscale.api_utils.exceptions.log_retrieval_errors import ExpectedLogRetrievalError, LogRetrievalTimeoutError, UnsupportedLogRetrievalMethodError
+from anyscale.api_utils.job_logs_util import (
+    _get_logs_from_running_production_job,
+    _get_job_logs_from_storage_bucket,
+    _get_logs_from_active_job_run,
+)
+from anyscale.api_utils.job_util import (
+    _is_job_on_v2_cloud,
+    _is_production_job_on_v2_cloud,
+)
+from anyscale.api_utils.exceptions.log_retrieval_errors import (
+    ExpectedLogRetrievalError,
+    LogRetrievalTimeoutError,
+    UnsupportedLogRetrievalMethodError,
+)
 from anyscale.api_utils.logs_util import _remove_ansi_escape_sequences
 from anyscale.sdk.anyscale_client.models.cluster_compute_config import (
     ClusterComputeConfig,
@@ -17,38 +28,81 @@ from anyscale.sdk.anyscale_client.models.cluster_compute_config import (
 from anyscale.sdk.anyscale_client.models.create_cluster_compute import (
     CreateClusterCompute,
 )
+from anyscale.sdk.anyscale_client.models.job import Job
+from anyscale.sdk.anyscale_client.models.job_status import JobStatus
+from anyscale.sdk.anyscale_client.models.production_job import ProductionJob
+from anyscale.sdk.anyscale_client.models.ha_job_states import HaJobStates
 from anyscale.utils.runtime_env import upload_and_rewrite_working_dir
 from anyscale.sdk import anyscale_client
 from anyscale.sdk.anyscale_client.api.default_api import DefaultApi
 from anyscale.version import __version__ as version
-from anyscale.sdk.anyscale_client.models.cloud import Cloud
 from anyscale.sdk.anyscale_client.models.create_cluster_environment import (
     CreateClusterEnvironment,
 )
 from anyscale.sdk.anyscale_client.models.create_byod_cluster_environment import (
     CreateBYODClusterEnvironment,
 )
+from anyscale.sdk.anyscale_client.models.apply_production_service_v2_model import (
+    ApplyProductionServiceV2Model,
+)
 from anyscale.sdk.anyscale_client.models.cluster import Cluster
-from anyscale.sdk.anyscale_client.models.session import Session
 from anyscale.sdk.anyscale_client.models.cluster_environment_build import (
     ClusterEnvironmentBuild,
 )
-from anyscale.sdk.anyscale_client.models.job import Job
+from anyscale.sdk.anyscale_client.models.jobs_logs import JobsLogs
 from anyscale.sdk.anyscale_client.models.jobslogs_response import JobslogsResponse
 from anyscale.shared_anyscale_utils.headers import RequestHeaders
-from anyscale.cli_logger import BlockLogger
+from anyscale.cli_logger import BlockLogger, pad_string
+from anyscale.cluster_compute import get_selected_cloud_id_or_default
 from anyscale.util import get_endpoint
 from anyscale.authenticate import AuthenticationBlock
 from anyscale_client.models import ClusterEnvironmentBuildStatus
 from anyscale.util import get_ray_and_py_version_for_default_cluster_env
+from anyscale.utils.deprecation_util import deprecated
+from anyscale.project import get_default_project
+
+logger = logging.getLogger(__file__)
 
 
 def _upload_and_rewrite_working_dir_in_create_production_job(create_production_job):
+    """
+    If a local working_dir and upload_path are specified, this method
+    will upload the working_dir and rewrites the working_dir to the remote uri.
+
+    Please note that this method mutates the model in-place.
+    """
     if create_production_job.config is not None:
-        runtime_env = create_production_job.config.get("runtime_env")
+        runtime_env = None
+        if isinstance(create_production_job.config, dict):
+            runtime_env = create_production_job.config.get("runtime_env")
+        elif hasattr(create_production_job.config, "runtime_env"):
+            runtime_env = create_production_job.config.runtime_env
         if runtime_env is not None:
             new_runtime_env = upload_and_rewrite_working_dir(runtime_env)
-            create_production_job.config["runtime_env"] = new_runtime_env
+            if isinstance(create_production_job.config, dict):
+                create_production_job.config["runtime_env"] = new_runtime_env
+            elif hasattr(create_production_job.config, "runtime_env"):
+                create_production_job.config.runtime_env = new_runtime_env
+
+
+def _upload_and_rewrite_working_dir_ray_serve_config(
+    ray_serve_config: Optional[Dict[str, Any]]
+) -> Dict[str, Any]:
+    """
+    If a local working_dir and upload_path are specified, this method
+    will upload the working_dir and rewrites the working_dir to the remote uri.
+
+    Please note that this method returns the new ray_serve_config.
+    """
+    if not ray_serve_config:
+        return ray_serve_config
+    res = copy.deepcopy(ray_serve_config)
+    if isinstance(ray_serve_config, dict):
+        runtime_env = ray_serve_config.get("runtime_env")
+        if runtime_env is not None:
+            new_runtime_env = upload_and_rewrite_working_dir(runtime_env)
+            res["runtime_env"] = new_runtime_env
+    return res
 
 
 def _is_create_byod_cluster_environment(obj: Any) -> bool:
@@ -175,11 +229,28 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
                 ).result
             build_operation_id = build.id
 
-        return self.wait_for_cluster_environment_build_operation(
+        build = self._wait_for_cluster_environment_build_operation(
             build_operation_id, poll_rate_seconds, timeout_seconds, log_output
         )
+        self.log.close_block()
 
-    def wait_for_cluster_environment_build_operation(
+        block_label = "BuildDetails"
+        self.log.open_block(block_label=block_label, block_title="Build Details")
+        with self.log.indent():
+            pad_len = len("Name:")
+            self.log.info(
+                pad_string("Name:", pad_len),
+                BlockLogger.highlight(f"{create_cluster_environment.name}:{build.revision}"),
+                block_label=block_label
+            )
+            self.log.info(
+                pad_string("ID:", pad_len),
+                BlockLogger.highlight(build.id),
+                block_label=block_label
+            )
+        return build
+
+    def _wait_for_cluster_environment_build_operation(
         self,
         operation_id: str,
         poll_rate_seconds: int = 15,
@@ -207,9 +278,9 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         operation = self.get_build(operation_id)
         url = get_endpoint(f"configurations/app-config-details/{operation_id}")
         self.log.info(
-            f"Waiting for cluster environment to build. View progress at {url}."
+            f"Waiting for cluster environment to build. View progress at {url}"
         )
-        self.log.info(f"status: {operation.result.status}")
+        self.log.info(f"Status: {operation.result.status}")
         while operation.result.status in IN_PROGRESS_BUILD_STATES:
             if timeout and time() > timeout:
                 raise Exception(
@@ -217,7 +288,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
                 )
 
             sleep(poll_rate_seconds)
-            self.log.info(f"status: {operation.result.status}")
+            self.log.info(f"Status: {operation.result.status}")
             operation = self.get_build(operation_id)
         if operation.result.status != ClusterEnvironmentBuildStatus.SUCCEEDED:
             raise Exception(
@@ -237,6 +308,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         timeout_seconds: Optional[int] = None,
         idle_timeout_minutes: Optional[int] = None,
         cluster_compute_config: Optional[ClusterComputeConfig] = None,
+        log_output: bool = False,
     ) -> Cluster:
         """
         Starts a Cluster in the specified Project.
@@ -255,6 +327,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             timeout_seconds - maximum number of seconds to wait for Cluster operation to complete before timing out; defaults to no timeout
             idle_timeout_minutes - Idle timeout (in minutes), after which the Cluster is terminated
             cluster_compute_config - One-off Cluster Compute that this Cluster will use.
+            log_output - Whether to show verbose logs during execution
 
         Returns:
             Cluster object
@@ -263,14 +336,21 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             Exception if starting Cluster fails or times out
         """
 
-        if not project_id:
-            default_project = self.get_default_project().result
-            search_project_id = default_project.id
-        else:
+        self.log.log_output = log_output
+        if project_id:
             search_project_id = project_id
+        else:
+            parent_cloud_id = get_selected_cloud_id_or_default(
+                cluster_compute_id=cluster_compute_id,
+                cluster_compute_config=cluster_compute_config,
+                cloud_id=None,
+                cloud_name=None,
+            )
+            default_project = get_default_project(parent_cloud_id=parent_cloud_id)
+            project_id = default_project.id
 
         clusters = self.search_clusters(
-            {"project_id": search_project_id, "name": {"equals": cluster_name}}
+            {"project_id": project_id, "name": {"equals": cluster_name}}
         ).results
 
         if clusters:
@@ -413,7 +493,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
     def fetch_job_logs(self, job_id: str) -> str:
         """
         Retrieves logs for a job run (eg. 'job_123').
-        
+
         This will try in following order until it finds a retrieval method that returns the logs:
         1. Ray Dashboard API on the cluster that the job run is on
            NOTE: The caller of this function must be able to reach the Ray dashboard host.
@@ -427,10 +507,15 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             str: All logs for the job run
                 (so far, if the job run is still running)
         """
+        job_run: Job = self.get_job(job_id).result
         get_job_logs_fns: List[Callable[..., str]] = [
-            lambda: asyncio.run(_get_logs_from_running_job(self, job_id)),
+            *(
+                [lambda: asyncio.run(_get_logs_from_active_job_run(self, job_id))] if job_run.status == JobStatus.RUNNING else []
+            ),
             lambda: self._fetch_job_logs_legacy(job_id),
-            lambda: asyncio.run(_get_job_logs_from_storage_bucket(self, job_run_id=job_id)),
+            lambda: asyncio.run(
+                _get_job_logs_from_storage_bucket(self, job_run_id=job_id)
+            ),
         ]
         logs = ""
         # Return logs from the first retrieval method that has non-empty logs
@@ -439,7 +524,7 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
                 logs = fn()
                 if logs:
                     return logs
-        return logs    
+        return logs
 
     def _fetch_job_logs_legacy(self, job_id: str) -> str:
         """
@@ -460,12 +545,6 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             raise UnsupportedLogRetrievalMethodError(
                 "Logs are not stored Anyscale-side on clouds set up with Anyscale v2."
             )
-        else:
-            warnings.warn(
-                "Fetching logs stored in Anyscale backend is deprecated. "
-                "Please consider moving to a cloud set up with Anyscale v2.",
-                DeprecationWarning
-            )
 
         # Timeout in 5 minutes
         timeout = time() + 300
@@ -482,8 +561,13 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         # Logs will be empty if on anyscale v1 and job logs are no longer in cache
         return _remove_ansi_escape_sequences(logs.logs)
 
+    @deprecated("`get_production_job_logs()` is deprecated. "
+                "Use `fetch_production_job_logs()` to retrieve the logs string instead.")
     def get_production_job_logs(self, job_id: str) -> JobslogsResponse:
         """
+        DEPRECATED: Please use `fetch_production_job_logs` to retrieve
+        the logs string instead.
+
         Retrieves logs for latest run of a Production Job.
 
         Args:
@@ -496,6 +580,14 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             Exception if we fail to fetch logs
         """
 
+        return JobslogsResponse(
+            result=JobsLogs(
+                logs=self.fetch_production_job_logs(job_id),
+                ready=True,
+            )
+        )
+
+    def _get_production_job_logs_legacy(self, job_id: str) -> JobslogsResponse:
         latest_job_run_id = self.get_production_job(
             production_job_id=job_id
         ).result.last_job_run_id
@@ -522,8 +614,11 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             str: All logs for the job
                 (so far, if the job is still running)
         """
+        job: ProductionJob = self.get_production_job(job_id).result
         get_job_logs_fns: List[Callable[..., str]] = [
-            lambda: asyncio.run(_get_logs_from_running_production_job(self, job_id)),
+            *(
+                [lambda: asyncio.run(_get_logs_from_running_production_job(self, job_id))] if job.state.current_state == HaJobStates.RUNNING else []
+            ),
             lambda: self._fetch_production_job_logs_legacy(job_id),
             lambda: asyncio.run(_get_job_logs_from_storage_bucket(self, job_id=job_id)),
         ]
@@ -556,26 +651,20 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
             raise UnsupportedLogRetrievalMethodError(
                 "Logs are not stored Anyscale-side on clouds set up with Anyscale v2."
             )
-        else:
-            warnings.warn(
-                "Fetching logs stored in Anyscale backend is deprecated. "
-                "Please consider moving to a cloud set up with Anyscale v2.",
-                DeprecationWarning
-            )
 
         # Timeout in 5 minutes
         timeout = time() + 300
 
-        logs = self.get_production_job_logs(job_id).result
+        logs = self._get_production_job_logs_legacy(job_id).result
 
         while not logs.ready:
             if time() > timeout:
                 raise Exception(f"Failed to get logs from Job {job_id}")
 
             sleep(15)
-            logs = self.get_production_job_logs(job_id).result
+            logs = self._get_production_job_logs_legacy(job_id).result
 
-        # Logs will be empty if on anyscale v1 and job logs are no longer in cache        
+        # Logs will be empty if on anyscale v1 and job logs are no longer in cache
         return _remove_ansi_escape_sequences(logs.logs)
 
     def create_job(self, create_production_job, **kwargs):
@@ -594,6 +683,27 @@ class AnyscaleSDK(DefaultApi):  # type: ignore
         )
         return super().apply_service(create_production_service, **kwargs)
 
+    def rollout_service_v2(self, apply_production_service_v2_model, **kwargs):
+        """
+        If a local working_dir and upload_path are specified in ray_serve_config,
+        this method uploads the local working_dir to the upload_path and rewrites
+        the working_dir in ray_serve_config.
+        """
+        if isinstance(apply_production_service_v2_model, dict):
+            ray_serve_config = apply_production_service_v2_model.get("ray_serve_config")
+            new_ray_serve_config = _upload_and_rewrite_working_dir_ray_serve_config(
+                ray_serve_config
+            )
+            apply_production_service_v2_model["ray_serve_config"] = new_ray_serve_config
+        elif hasattr(apply_production_service_v2_model, "ray_serve_config"):
+            ray_serve_config = apply_production_service_v2_model.ray_serve_config
+            new_ray_serve_config = _upload_and_rewrite_working_dir_ray_serve_config(
+                ray_serve_config
+            )
+            apply_production_service_v2_model.ray_serve_config = new_ray_serve_config
+
+        return super().rollout_service_v2(apply_production_service_v2_model, **kwargs)
+
     def get_cluster_token(self, project_id: str, cluster_name: str) -> str:
         """
         Retrieves cluster token for a Cluster.
