@@ -21,8 +21,7 @@ from typing import (
     TypeVar,
     Union,
 )
-from urllib.parse import quote as quote_sanitize, urlencode, urljoin
-import webbrowser
+from urllib.parse import urljoin
 
 import boto3
 from boto3.resources.base import ServiceResource as Boto3Resource
@@ -32,7 +31,6 @@ import click
 from packaging import version
 import requests
 from requests import Response
-import yaml
 
 from anyscale.authenticate import get_auth_api_client
 from anyscale.cli_logger import BlockLogger
@@ -47,7 +45,6 @@ from anyscale.conf import MINIMUM_RAY_VERSION
 from anyscale.sdk.anyscale_client.api.default_api import DefaultApi as BaseApi
 from anyscale.sdk.anyscale_client.models.compute_template import ComputeTemplate
 import anyscale.shared_anyscale_utils.conf as shared_anyscale_conf
-from anyscale.shared_anyscale_utils.util import get_container_name
 
 
 logger = logging.getLogger(__file__)
@@ -63,6 +60,11 @@ VPC_CIDR_RANGE = "10.0.0.0/16"
 
 DEFAULT_RAY_VERSION = "1.7.0"
 
+# The V2 stack has some delay before changing the state of a new cluster from
+# Terminated to StartingUp. Allow the terminated state for the duration of this
+# grace period before erroring.
+TERMINATED_STATE_GRACE_PERIOD_SECONDS = 60
+
 
 def confirm(msg: str, yes: bool) -> Optional[bool]:
     return None if yes else click.confirm(msg, abort=True)
@@ -175,11 +177,6 @@ def humanize_timestamp(timestamp: datetime.datetime) -> str:
         return "{} second{} ago".format(delta_s, "s" if delta_s > 1 else "")
 
 
-def get_cluster_config(config_path: str) -> Dict:
-    with open(config_path) as f:
-        return yaml.safe_load(f)
-
-
 def get_requirements(requirements_path: str) -> str:
     with open(requirements_path) as f:
         return f.read()
@@ -250,45 +247,6 @@ def get_availability_zones(region: str) -> List[str]:
     ]
 
 
-def launch_gcp_cloud_setup(
-    name: str,
-    region: str,
-    is_k8s: Optional[bool],
-    folder_id: Optional[int],
-    vpc_peering_ip_range: Optional[str],
-    vpc_peering_target_project_id: Optional[str],
-    vpc_peering_target_vpc_id: Optional[str],
-) -> None:
-    # TODO: enforce uniqueness for user's clouds
-    quote_safe_name = quote_sanitize(name, safe="")
-    query_params: Dict[str, Any] = {
-        "region": region,
-    }
-    if is_k8s is not None:
-        query_params["is_k8s"] = is_k8s
-    if folder_id:
-        query_params["folder_id"] = str(folder_id)
-    if (
-        vpc_peering_ip_range
-        and vpc_peering_target_project_id
-        and vpc_peering_target_vpc_id
-    ):
-        query_params.update(
-            {
-                "vpc_peering_ip_range": vpc_peering_ip_range,
-                "vpc_peering_target_project_id": vpc_peering_target_project_id,
-                "vpc_peering_target_vpc_id": vpc_peering_target_vpc_id,
-            }
-        )
-    # TODO: Replace this with a proper endpoint
-    endpoint = f"/api/v2/clouds/gcp/create/{quote_safe_name}?{urlencode(query_params)}"
-    full_url = get_endpoint(endpoint)
-    print(
-        f"Launching GCP Oauth Flow:\n{full_url}\n(If this window does not auto-launch, use the link above)"
-    )
-    webbrowser.open(full_url)
-
-
 def get_project_directory_name(project_id: str, api_client: ProductApi = None) -> str:
     if api_client is None:
         api_client = get_auth_api_client().api_client
@@ -396,6 +354,7 @@ def wait_for_session_start(
     if api_client is None:
         api_client = get_auth_api_client().api_client
 
+    start_time = time.time()
     with updating_printer() as print_status:
         while True:
             sessions = api_client.list_sessions_api_v2_sessions_get(
@@ -438,10 +397,15 @@ def wait_for_session_start(
                     and session.state in {"Terminated", "Stopped"}
                     and session.pending_state is None
                 ):
-                    # Cluster is created in Terminated state; Check pending state to see if it is pending transition.
-                    raise click.ClickException(
-                        f"Error while starting cluster {session_name}: Cluster is still in stopped/terminated state."
-                    )
+                    if time.time() - start_time < TERMINATED_STATE_GRACE_PERIOD_SECONDS:
+                        # The V2 stack has some delay before transitioning from Terminated to StartingUp.
+                        # Don't error until the grace period has passed
+                        print_status("Waiting for start up...")
+                    else:
+                        # Cluster is created in Terminated state; Check pending state to see if it is pending transition.
+                        raise click.ClickException(
+                            f"Error while starting cluster {session_name}: Cluster is still in stopped/terminated state."
+                        )
                 elif (
                     session.state_data
                     and session.state_data.startup
@@ -464,68 +428,12 @@ def wait_for_session_start(
             time.sleep(2)
 
 
-def _get_rsync_args(rsync_exclude: List[str], rsync_filter: List[str]) -> List[str]:
-    rsync_exclude_args = [["--exclude", exclude] for exclude in rsync_exclude]
-    rsync_filter_args = [["--filter", f"dir-merge,- {f}"] for f in rsync_filter]
-
-    # Combine and flatten the two lists
-    return [
-        arg for sublist in rsync_exclude_args + rsync_filter_args for arg in sublist
-    ]
-
-
-def get_rsync_command(  # noqa: PLR0913
-    ssh_command: List[str],
-    source: str,
-    ssh_user: str,
-    head_ip: str,
-    target: str,
-    delete: bool,
-    rsync_exclude: Optional[List[str]] = None,
-    rsync_filter: Optional[List[str]] = None,
-    dry_run: bool = False,
-) -> Tuple[List[str], Optional[Dict[str, Any]]]:
-    rsync_executable = "rsync"
-    env = None
-    if rsync_exclude is None:
-        rsync_exclude = []
-    if rsync_filter is None:
-        rsync_filter = []
-
-    command_list = [
-        rsync_executable,
-        "--rsh",
-        " ".join(ssh_command),
-        "-avz",
-    ]
-    command_list += _get_rsync_args(rsync_exclude, rsync_filter)
-
-    if delete:
-        # Deletes files in target that doesn't exist in source
-        command_list.append("--delete")
-
-    if dry_run:
-        command_list += [
-            "--dry-run",
-            "--itemize-changes",
-            "--out-format",
-            "%o %f",
-        ]
-
-    command_list += [
-        source,
-        f"{ssh_user}@{head_ip}:{target}",
-    ]
-    return command_list, env
-
-
 def populate_session_args(cluster_config_str: str, config_file_name: str) -> str:
     import jinja2
 
     env = jinja2.Environment()
     t = env.parse(cluster_config_str)
     for elem in t.body[0].nodes:  # type: ignore
-
         if isinstance(elem, jinja2.nodes.Getattr) and elem.attr not in os.environ:
             prefixed_command = " ".join(
                 [f"{elem.attr}=<value>", "anyscale"] + sys.argv[1:]
@@ -541,29 +449,6 @@ def populate_session_args(cluster_config_str: str, config_file_name: str) -> str
     return cluster_config_filled
 
 
-def canonicalize_remote_location(
-    cluster_config: Dict[str, Any], remote_location: Optional[str], project_id: str
-) -> Optional[str]:
-    """Returns remote_location, but changes it from being based
-    in "~/", "/root/" or "/ray" to match working_dir
-    """
-    # Include the /root path to ensure that absolute paths also work
-    # This is because of an implementation detail in OSS Ray's rsync
-    if bool(get_container_name(cluster_config)) and bool(remote_location):
-        remote_location = str(remote_location)
-        working_dir = get_working_dir(cluster_config, project_id)
-        for possible_name in ["root", "ray"]:
-            full_name = f"/{possible_name}/"
-            # TODO(ilr) upstream this to OSS Ray
-            if working_dir.startswith("~/") and remote_location.startswith(full_name):
-                return remote_location.replace(full_name, "~/", 1)
-
-            if working_dir.startswith(full_name) and remote_location.startswith("~/"):
-                return remote_location.replace("~/", full_name, 1)
-
-    return remote_location
-
-
 def get_user_info() -> Optional[UserInfo]:
     try:
         api_client = get_auth_api_client().api_client
@@ -702,6 +587,10 @@ def is_anyscale_workspace() -> bool:
     return "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID" in os.environ
 
 
+def is_anyscale_cluster() -> bool:
+    return "ANYSCALE_SESSION_ID" in os.environ
+
+
 def credentials_check_sanity(credentials_str: str) -> bool:
     """
     The main goal of this function is to perform a minimal sanity check
@@ -743,6 +632,7 @@ def prepare_cloudformation_template(
     subnet_templates: List[str] = []
     subnets_route_table_association: List[str] = []
     subnets_with_availability_zones: List[str] = []
+    efs_mount_targets: List[str] = []
 
     vpc_cidr = ipaddress.ip_network(VPC_CIDR_RANGE)
     if len(azs) > 4:
@@ -780,6 +670,17 @@ def prepare_cloudformation_template(
             f'{{"subnet_id": !Ref Subnet{i}, "availability_zone": !GetAtt Subnet{i}.AvailabilityZone}}'
         )
 
+        efs_mount_targets.append(
+            f"""
+  EFSMountTarget{i}:
+    Type: AWS::EFS::MountTarget
+    Properties:
+        FileSystemId: !Ref EFS
+        SecurityGroups:
+          - !Ref AnyscaleSecurityGroup
+        SubnetId: !Ref Subnet{i}"""
+        )
+
     body = body.replace("$SUBNETS_TEMPLATES", "\n".join(subnet_templates))
     body = body.replace(
         "$SUBNETS_ROUTE_TABLE_ASSOCIATION", "\n".join(subnets_route_table_association),
@@ -789,6 +690,7 @@ def prepare_cloudformation_template(
         f'[{",".join(subnets_with_availability_zones)}]',
     )
     body = body.replace("$ALLOWED_ORIGIN", shared_anyscale_conf.ANYSCALE_HOST)
+    body = body.replace("$EFSMountTargets", "\n".join(efs_mount_targets))
 
     return body
 
@@ -866,7 +768,7 @@ def populate_dict_with_workspace_config_if_exists(
     config_dict: Dict[str, Any], anyscale_api_client: BaseApi
 ) -> Dict[str, Any]:
     """
-    If the job is submitted from the workspace, we populate values from the workspace
+    If the service/job is submitted from the workspace, we populate values from the workspace
     if the config doesn't specify them, such as cluster env and compute config.
     """
     if (
@@ -876,14 +778,15 @@ def populate_dict_with_workspace_config_if_exists(
         cluster = anyscale_api_client.get_cluster(
             os.environ["ANYSCALE_SESSION_ID"]
         ).result
-        # If the job configs are not specified, infer them from the workspace:
+        # If the service/job configs are not specified, infer them from the workspace:
         if "build_id" not in config_dict and "cluster_env" not in config_dict:
             config_dict["build_id"] = cluster.cluster_environment_build_id
-        if "project_id" not in config_dict:
+        if "project_id" not in config_dict and "project" not in config_dict:
             config_dict["project_id"] = cluster.project_id
         if (
             "compute_config" not in config_dict
             and "compute_config_id" not in config_dict
+            and "cloud" not in config_dict
         ):
             config_dict["compute_config_id"] = cluster.cluster_compute_id
     return config_dict
@@ -959,3 +862,14 @@ T = TypeVar("T")
 
 def _coerce_to_list(maybe_list: Union[T, List[T]]) -> List[T]:
     return maybe_list if isinstance(maybe_list, list) else [maybe_list]
+
+
+@contextmanager
+def handle_and_log_lines_in_click_exception(
+    log_callback: Callable[[str], Any]
+) -> Generator[None, None, None]:
+    try:
+        yield
+    except click.ClickException as e:
+        for line in e.message.splitlines():
+            log_callback(line)
