@@ -1,5 +1,8 @@
 from typing import Optional, Tuple
 
+from click.exceptions import ClickException
+
+from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client.api.default_api import DefaultApi
 from anyscale.cloud import get_cloud_id_and_name
 
@@ -39,3 +42,16 @@ def get_default_cloud(
         if default_cloud_name:
             cloud_name = default_cloud_name
     return get_cloud_id_and_name(api_client, cloud_name=cloud_name)
+
+
+def verify_anyscale_access(
+    api_client: DefaultApi, cloud_id: str, logger: BlockLogger
+) -> bool:
+    try:
+        api_client.verify_access_api_v2_cloudsverify_access_cloud_id_get(cloud_id)
+        return True
+    except ClickException as e:
+        logger.error(
+            f"Anyscale's control plane is unable to access resources on your cloud provider.\n{e}"
+        )
+        return False
