@@ -323,7 +323,7 @@ def test_convert_cluster_env(mock_auth_api_client) -> None:
         ("3.7", "2.0.0dev0", False, True, "anyscale/ray:1.7.0-py37-gpu"),
     ],
 )
-def test_select_base_image(  # noqa: PLR0913
+def test_select_base_image(
     python_version: str,
     ray_version: str,
     ml: bool,
@@ -358,7 +358,7 @@ def test_select_base_image(  # noqa: PLR0913
         ("3.8", "1.9.2", False, False, "ANYSCALE_RAY_1_9_2_PY38"),
     ],
 )
-def test_find_enum_name(  # noqa: PLR0913
+def test_find_enum_name(
     python_version: str,
     ray_version: str,
     ml: bool,
