@@ -1,3 +1,4 @@
+import asyncio
 import json
 import os
 from pathlib import Path
@@ -10,26 +11,39 @@ from unittest.mock import MagicMock, Mock, mock_open, patch
 from aiohttp import ClientResponseError, ClientSession, WSMsgType
 from asynctest import CoroutineMock
 import click
+from pydantic import ValidationError
 import pytest
 import yaml
 
+from anyscale.api_utils.exceptions.log_retrieval_errors import (
+    ExpectedLogRetrievalError,
+    UnsupportedLogRetrievalMethodError,
+)
+from anyscale.api_utils.job_logs_util import _get_job_logs_from_storage_bucket
+from anyscale.api_utils.job_util import _is_production_job_on_v2_cloud
+from anyscale.api_utils.logs_util import _stream_log_from_ray_websocket
 from anyscale.client.openapi_client.models.create_internal_production_job import (
     CreateInternalProductionJob,
 )
-from anyscale.client.openapi_client.models.log_filter import LogFilter
+from anyscale.client.openapi_client.models.ha_job_states import HaJobStates
 from anyscale.client.openapi_client.models.production_job_config import (
     ProductionJobConfig,
 )
+from anyscale.controllers import job_controller as job_controller_module
 from anyscale.controllers.job_controller import (
-    _working_dir_is_remote_uri,
-    JobConfig,
     JobController,
     LogProvider,
     LogsLogger,
     MiniJobRun,
 )
+from anyscale.models.job_model import _working_dir_is_remote_uri, JobConfig
 from anyscale.sdk.anyscale_client.models.compute_template import ComputeTemplate
-from anyscale.util import PROJECT_NAME_ENV_VAR
+from anyscale.shared_anyscale_utils.test_util import (
+    get_args_from_call,
+    MockAsyncContextManagerReturnValue,
+)
+from anyscale.util import poll, PROJECT_NAME_ENV_VAR
+from anyscale.utils.runtime_env import is_dir_remote_uri
 
 
 CONDA_DICT = {"dependencies": ["pip", {"pip": ["pip-install-test==0.5"]}]}
@@ -103,9 +117,9 @@ def test_generate_config_from_entrypoint(
     mock_validate_successful_build = Mock()
     mock_get_default_cluster_env_build = Mock(return_value=Mock(id="mock_build_id"))
     with patch.multiple(
-        "anyscale.controllers.job_controller",
-        get_default_cluster_compute=mock_get_default_cluster_compute,
+        "anyscale.models.job_model",
         validate_successful_build=mock_validate_successful_build,
+        get_default_cluster_compute=mock_get_default_cluster_compute,
         get_default_cluster_env_build=mock_get_default_cluster_env_build,
     ):
         job_config = job_controller.generate_config_from_entrypoint(
@@ -118,6 +132,53 @@ def test_generate_config_from_entrypoint(
     assert job_config.workspace_id == workspace_id
 
 
+@pytest.mark.parametrize("add_extra_field", [True, False])
+def test_config_validation(add_extra_field, mock_auth_api_client):
+    """
+    This test checks that an error is thrown if extra/mispelled fields
+    are added into the config file
+    """
+    mock_logger = Mock()
+    job_controller = JobController(log=mock_logger)
+
+    name = "test_job_name"
+    description = "mock_description"
+    build_id = "test_build_id"
+    compute_config_id = "test_compute_config_id"
+    project_id = "test_project_id"
+    extra_field_value = "*****"
+    config_dict = {
+        "name": name,
+        "description": description,
+        "entrypoint": "python test.py",
+        "project_id": project_id,
+        "build_id": build_id,
+        "compute_config_id": compute_config_id,
+    }
+    if add_extra_field:
+        extra_field_value = "*****"
+        config_dict["extra_field"] = extra_field_value
+
+    mock_validate_successful_build = Mock()
+    with patch(
+        "builtins.open", mock_open(read_data=json.dumps(config_dict))
+    ), patch.multiple("os.path", exists=Mock(return_value=True)), patch.multiple(
+        "anyscale.models.job_model",
+        validate_successful_build=mock_validate_successful_build,
+    ), patch.multiple(
+        "anyscale.controllers.job_controller", validate_job_config_dict=Mock(),
+    ):
+        if add_extra_field:
+            with pytest.raises(ValidationError):
+                job_controller.generate_config_from_file(
+                    "test_job_config_file", name=name, description=description,
+                )
+        else:
+            job_controller.generate_config_from_file(
+                "test_job_config_file", name=name, description=description,
+            )
+
+
 @pytest.mark.parametrize(
     "config_dict",
     [
@@ -164,9 +225,9 @@ def test_submit_job(
     mock_project_definition = Mock()
     mock_project_definition.root = "/some/directory"
 
-    def infer_project_id_mock(project_id: str, *args: Any):
-        if project_id:
-            return project_id
+    def infer_project_id_mock(*args: Any, **kwargs: Any):
+        if kwargs.get("project_id"):
+            return kwargs.get("project_id")
         elif use_default_project:
             return "mock_default_project_id"
         else:
@@ -176,6 +237,9 @@ def test_submit_job(
     mock_get_build_from_cluster_env_identifier = Mock(
         return_value=Mock(id="mock_build_id")
     )
+
+    mock_override_runtime_env_config = Mock(return_value=None)
+
     job_controller.api_client.get_compute_template_api_v2_compute_templates_template_id_get = Mock(
         return_value=Mock(result=compute_template_test_data)
     )
@@ -188,17 +252,24 @@ def test_submit_job(
 
     mock_validate_successful_build = Mock()
     mock_get_default_cluster_env_build = Mock(return_value=Mock(id="mock_build_id"))
+    mock_get_parent_cloud_id_and_name_of_project = Mock(
+        return_value=("mock_parent_cloud_id", "mock_parent_cloud_name")
+    )
     with patch(
         "builtins.open", mock_open(read_data=json.dumps(config_dict))
     ), patch.multiple(
-        "anyscale.controllers.job_controller",
-        infer_project_id=mock_infer_project_id,
-        get_build_from_cluster_env_identifier=mock_get_build_from_cluster_env_identifier,
+        "anyscale.models.job_model",
+        validate_successful_build=mock_validate_successful_build,
         get_cluster_compute_from_name=mock_get_cluster_compute_from_name,
         get_default_cluster_compute=mock_get_default_cluster_compute,
         register_compute_template=mock_register_compute_template,
-        validate_successful_build=mock_validate_successful_build,
+        get_build_from_cluster_env_identifier=mock_get_build_from_cluster_env_identifier,
         get_default_cluster_env_build=mock_get_default_cluster_env_build,
+        get_parent_cloud_id_and_name_of_project=mock_get_parent_cloud_id_and_name_of_project,
+    ), patch.multiple(
+        "anyscale.controllers.job_controller",
+        infer_project_id=mock_infer_project_id,
+        override_runtime_env_config=mock_override_runtime_env_config,
     ), patch.object(
         JobController,
         "_get_maximum_uptime_output",
@@ -210,6 +281,15 @@ def test_submit_job(
             "mock_config_file", name="mock_name", description="mock_description"
         )
     mock_validate_successful_build.assert_called_once_with("mock_build_id")
+    mock_infer_project_id.assert_called_once_with(
+        job_controller.anyscale_api_client,
+        job_controller.api_client,
+        job_controller.log,
+        project_id=config_dict.get("project_id"),
+        cluster_compute_id=compute_template_test_data.id,
+        cluster_compute=config_dict.get("compute_config"),
+        cloud=config_dict.get("cloud"),
+    )
 
     if config_project_id:
         final_project_id = config_project_id
@@ -243,10 +323,14 @@ def test_submit_job(
             mock_register_compute_template.assert_called_once_with(compute_config)
     elif "cloud" in config_dict:
         mock_get_default_cluster_compute.assert_called_once_with(
-            config_dict["cloud"], None
+            cloud_name=config_dict["cloud"], project_id=None
         )
     elif "compute_config_id" in config_dict:
         mock_get_default_cluster_compute.assert_not_called()
+    if "project_id" in config_dict:
+        mock_get_parent_cloud_id_and_name_of_project.assert_called_once_with(
+            config_dict.get("project_id")
+        )
 
 
 @pytest.mark.parametrize("include_all_users", [False, True])
@@ -424,7 +508,7 @@ class TestValidateConda:
             entrypoint="ls",
             build_id="123",
             compute_config_id="test",
-            runtime_env={"conda": "env_name"},
+            runtime_env={"conda": "env_name", "working_dir": "s3://bucket"},
         )
         assert jc.runtime_env["conda"] == "env_name"
 
@@ -560,14 +644,50 @@ class TestValidateEnvVars:
 
 
 class TestValidateUploadPath:
-    def test_reject_upload_path_without_local_dir(self, patch_jobs_anyscale_api_client):
-        """If upload_path is specified, a local working_dir must also be specified."""
-        with pytest.raises(click.ClickException):
+    @pytest.mark.parametrize("working_dir", [None, ".", "s3://bucket", "gs://bucket"])
+    @pytest.mark.parametrize("upload_path", [None, "s3://bucket", "gs://bucket"])
+    def test_validate_working_dir_and_upload_path_combinations(
+        self, patch_jobs_anyscale_api_client, working_dir: str, upload_path: str
+    ):
+        """Test combinations of working_dir and upload_path
+        Exceptions:
+        1. Working_dir is a remote uri and upload_path is defined
+        2. Working_dir is None and upload_path is defined
+        """
+        runtime_env = {}
+        if working_dir:
+            runtime_env["working_dir"] = working_dir
+        if upload_path:
+            runtime_env["upload_path"] = upload_path
+
+        if is_dir_remote_uri(working_dir) and is_dir_remote_uri(upload_path):
+            with pytest.raises(
+                click.ClickException,
+                match=r"`upload_path` was specified, but `working_dir` is not a local directory",
+            ):
+                JobConfig(
+                    entrypoint="ls",
+                    build_id="123",
+                    compute_config_id="test",
+                    runtime_env=runtime_env,
+                )
+        elif working_dir is None and is_dir_remote_uri(upload_path):
+            with pytest.raises(
+                click.ClickException,
+                match=r"upload_path` was specified, but no `working_dir` is defined",
+            ):
+                JobConfig(
+                    entrypoint="ls",
+                    build_id="123",
+                    compute_config_id="test",
+                    runtime_env=runtime_env,
+                )
+        else:
             JobConfig(
                 entrypoint="ls",
                 build_id="123",
                 compute_config_id="test",
-                runtime_env={"upload_path": "s3://some_bucket"},
+                runtime_env=runtime_env,
             )
 
     def test_reject_non_s3_or_gs_upload_path(self, patch_jobs_anyscale_api_client):
@@ -607,49 +727,6 @@ class TestValidateWorkingDir:
         assert not _working_dir_is_remote_uri("/some/path")
         assert not _working_dir_is_remote_uri("some/path/")
 
-    def test_reject_local_dir_without_upload_path(self, patch_jobs_anyscale_api_client):
-        """If a local working_dir is specified, upload_path must also be specified."""
-        with pytest.raises(click.ClickException):
-            JobConfig(
-                entrypoint="ls",
-                build_id="123",
-                compute_config_id="test",
-                runtime_env={"working_dir": "."},
-            )
-
-        with pytest.raises(click.ClickException):
-            JobConfig(
-                entrypoint="ls",
-                build_id="123",
-                compute_config_id="test",
-                runtime_env={"working_dir": "/tmp/dir"},
-            )
-
-    def test_reject_remote_uri_with_upload_path(self, patch_jobs_anyscale_api_client):
-        """If a working_dir is specified as a remote URI, upload_path cannot be specified."""
-        with pytest.raises(click.ClickException):
-            JobConfig(
-                entrypoint="ls",
-                build_id="123",
-                compute_config_id="test",
-                runtime_env={"working_dir": "s3://test", "upload_path": "s3://another"},
-            )
-
-    def test_accept_local_dir_with_upload_path(self, patch_jobs_anyscale_api_client):
-        JobConfig(
-            entrypoint="ls",
-            build_id="123",
-            compute_config_id="test",
-            runtime_env={"working_dir": ".", "upload_path": "s3://fake"},
-        )
-
-        JobConfig(
-            entrypoint="ls",
-            build_id="123",
-            compute_config_id="test",
-            runtime_env={"working_dir": ".", "upload_path": "gs://fake/dir"},
-        )
-
     def test_reject_nonexistent_local_dir(self, patch_jobs_anyscale_api_client):
         """If a local working_dir is specified, it must exist."""
         with pytest.raises(click.ClickException):
@@ -702,6 +779,130 @@ class TestValidatePyModules:
         assert jc.runtime_env["py_modules"] == ["s3://path/to/archive.zip"]
 
 
+class TestComputeConfig:
+    @pytest.fixture()
+    def patch_calls_to_get_compute_config(self,):
+        mock_get_default_cluster_compute = Mock(
+            return_value=Mock(id="mock_default_compute_config_id")
+        )
+        mock_get_cluster_compute_from_name = Mock(
+            return_value=Mock(id="mock_existing_compute_config_id")
+        )
+        mock_register_compute_template = Mock(
+            return_value=Mock(id="mock_registered_compute_config_id")
+        )
+        mock_get_parent_cloud_id_and_name_of_project = Mock(
+            return_value=("mock_parent_cloud_id", "mock_parent_cloud_name")
+        )
+
+        with patch.multiple(
+            "anyscale.models.job_model",
+            get_default_cluster_compute=mock_get_default_cluster_compute,
+            get_cluster_compute_from_name=mock_get_cluster_compute_from_name,
+            register_compute_template=mock_register_compute_template,
+            get_parent_cloud_id_and_name_of_project=mock_get_parent_cloud_id_and_name_of_project,
+        ):
+            yield mock_get_default_cluster_compute, mock_get_cluster_compute_from_name, mock_register_compute_template, mock_get_parent_cloud_id_and_name_of_project
+
+    def test_fill_compute_config_id_from_compute_config_id(
+        self, patch_jobs_anyscale_api_client, patch_calls_to_get_compute_config
+    ):
+        (
+            mock_get_default_cluster_compute,
+            mock_get_cluster_compute_from_name,
+            mock_register_compute_template,
+            mock_get_parent_cloud_id_and_name_of_project,
+        ) = patch_calls_to_get_compute_config
+        jc = JobConfig(
+            entrypoint="ls", compute_config_id="mock_existing_compute_config_id",
+        )
+        assert jc.compute_config_id == "mock_existing_compute_config_id"
+        mock_get_default_cluster_compute.assert_not_called()
+        mock_get_cluster_compute_from_name.assert_not_called()
+        mock_register_compute_template.assert_not_called()
+        mock_get_parent_cloud_id_and_name_of_project.assert_not_called()
+
+    def test_fill_compute_config_id_from_compute_config_name(
+        self, patch_jobs_anyscale_api_client, patch_calls_to_get_compute_config
+    ):
+        (
+            mock_get_default_cluster_compute,
+            mock_get_cluster_compute_from_name,
+            mock_register_compute_template,
+            mock_get_parent_cloud_id_and_name_of_project,
+        ) = patch_calls_to_get_compute_config
+        jc = JobConfig(
+            entrypoint="ls", compute_config="mock_existing_compute_config_name",
+        )
+        assert jc.compute_config_id == "mock_existing_compute_config_id"
+        mock_get_default_cluster_compute.assert_not_called()
+        mock_get_cluster_compute_from_name.assert_called_once_with(
+            "mock_existing_compute_config_name"
+        )
+        mock_register_compute_template.assert_not_called()
+        mock_get_parent_cloud_id_and_name_of_project.assert_not_called()
+
+    def test_fill_compute_config_id_from_compute_config_dict(
+        self, patch_jobs_anyscale_api_client, patch_calls_to_get_compute_config
+    ):
+        (
+            mock_get_default_cluster_compute,
+            mock_get_cluster_compute_from_name,
+            mock_register_compute_template,
+            mock_get_parent_cloud_id_and_name_of_project,
+        ) = patch_calls_to_get_compute_config
+        compute_config_dict = {
+            "cloud_id": "mock_cloud_id",
+            "region": "mock_region",
+            "head_node_type": {"name": "head", "instance_type": "m5.large"},
+            "worker_node_types": [],
+        }
+        jc = JobConfig(entrypoint="ls", compute_config=compute_config_dict,)
+        assert jc.compute_config_id == "mock_registered_compute_config_id"
+        mock_get_default_cluster_compute.assert_not_called()
+        mock_get_cluster_compute_from_name.assert_not_called()
+        mock_register_compute_template.assert_called_once_with(compute_config_dict)
+        mock_get_parent_cloud_id_and_name_of_project.assert_not_called()
+
+    def test_fill_compute_config_id_from_cloud(
+        self, patch_jobs_anyscale_api_client, patch_calls_to_get_compute_config
+    ):
+        (
+            mock_get_default_cluster_compute,
+            mock_get_cluster_compute_from_name,
+            mock_register_compute_template,
+            mock_get_parent_cloud_id_and_name_of_project,
+        ) = patch_calls_to_get_compute_config
+        jc = JobConfig(entrypoint="ls", cloud="mock_cloud",)
+        assert jc.compute_config_id == "mock_default_compute_config_id"
+        mock_get_default_cluster_compute.assert_called_once_with(
+            cloud_name="mock_cloud", project_id=None
+        )
+        mock_get_cluster_compute_from_name.assert_not_called()
+        mock_register_compute_template.assert_not_called()
+        mock_get_parent_cloud_id_and_name_of_project.assert_not_called()
+
+    def test_fill_compute_config_id_from_project_with_cloud_isolation(
+        self, patch_jobs_anyscale_api_client, patch_calls_to_get_compute_config
+    ):
+        (
+            mock_get_default_cluster_compute,
+            mock_get_cluster_compute_from_name,
+            mock_register_compute_template,
+            mock_get_parent_cloud_id_and_name_of_project,
+        ) = patch_calls_to_get_compute_config
+        jc = JobConfig(entrypoint="ls", project_id="mock_project_id")
+        assert jc.compute_config_id == "mock_default_compute_config_id"
+        mock_get_default_cluster_compute.assert_called_once_with(
+            cloud_name="mock_parent_cloud_name", project_id=None
+        )
+        mock_get_cluster_compute_from_name.assert_not_called()
+        mock_register_compute_template.assert_not_called()
+        mock_get_parent_cloud_id_and_name_of_project.assert_called_once_with(
+            "mock_project_id"
+        )
+
+
 @pytest.mark.parametrize("project_id", [None, "proj_id"])
 @pytest.mark.parametrize("project_name", [None, "proj_name"])
 @pytest.mark.parametrize("project_name_env_var", [None, "proj_name_env"])
@@ -723,7 +924,7 @@ def test_validate_project_id_field(
         {PROJECT_NAME_ENV_VAR: project_name_env_var} if project_name_env_var else {}
     )
     with patch.multiple(
-        "anyscale.controllers.job_controller",
+        "anyscale.models.job_model",
         get_proj_id_from_name=mock_get_proj_id_from_name,
         validate_successful_build=mock_validate_successful_build,
     ), patch.dict(os.environ, mock_os_dict):
@@ -811,28 +1012,18 @@ def fake_log_provider(mode: str) -> Type[LogProvider]:
     return FakeLogProvider
 
 
-# For `async with a() as b:`
-class MockAsyncContextManagerReturnValue(MagicMock):
-    async def __aenter__(self, *args):
-        return self
-
-    async def __aexit__(self, *args):
-        return None
-
-
-# This also indirectly tests _read_websocket()
-def test_stream_logs(mock_auth_api_client) -> None:
+def test_print_logs_for_active_job_run(mock_auth_api_client) -> None:
     job_controller = JobController(log=FakeLogger())
-    prodjob_id = "prodjob_123"
     job_id = "job_123"
-    job_controller._wait_for_a_job_run = Mock(return_value=job_id)  # type: ignore
+    websocket_url = "http://localhost:5000/stream"
     job_controller.api_client = Mock(
         get_job_logs_stream_api_v2_logs_job_logs_stream_job_id_get=Mock(
-            return_value=Mock(result=Mock(stream_url="http://localhost:5000/stream",))
+            return_value=Mock(result=Mock(stream_url=websocket_url))
         )
     )
     mock_websocket = MockAsyncContextManagerReturnValue()
-    # Test logs successfully streamed
+
+    # Test logs successfully streamed (followed) from cluster
     mock_websocket.receive = CoroutineMock(
         side_effect=[
             Mock(type=WSMsgType.TEXT, data=" abc "),
@@ -843,25 +1034,12 @@ def test_stream_logs(mock_auth_api_client) -> None:
     with patch.object(
         ClientSession, ClientSession.ws_connect.__name__, return_value=mock_websocket
     ):
-        job_controller.stream_logs(prodjob_id)
-    job_controller._wait_for_a_job_run.assert_called_once_with(prodjob_id)
+        job_controller._print_logs_for_active_job_run(job_id, should_follow=True)
     job_controller.api_client.get_job_logs_stream_api_v2_logs_job_logs_stream_job_id_get.assert_called_once_with(
         job_id=job_id
     )
     assert mock_websocket.receive.call_count == 3
     assert job_controller.log.logs == ["abc"]  # type: ignore
-    # Test job does not exist
-    mock_job_run_not_found_exception = ClientResponseError(
-        request_info=Mock(), history=Mock(), status=404
-    )
-    mock_websocket.receive = CoroutineMock(side_effect=mock_job_run_not_found_exception)
-    with patch.object(
-        ClientSession, ClientSession.ws_connect.__name__, return_value=mock_websocket
-    ), pytest.raises(
-        Exception  # noqa: PT011
-    ) as e:
-        job_controller.stream_logs(job_id)
-    assert "Job run job_123 not found in cluster for current running job." in str(e)
 
     # Test other exception thrown
     mock_other_exception = ClientResponseError(
@@ -870,38 +1048,65 @@ def test_stream_logs(mock_auth_api_client) -> None:
     mock_websocket.receive = CoroutineMock(side_effect=mock_other_exception)
     with patch.object(
         ClientSession, ClientSession.ws_connect.__name__, return_value=mock_websocket
-    ), pytest.raises(
-        Exception  # noqa: PT011
-    ) as e:
-        job_controller.stream_logs(job_id)
+    ), pytest.raises(ClientResponseError):
+        job_controller._print_logs_for_active_job_run(job_id, should_follow=True)
 
-    # Test keyboard interrupt handled
-    job_controller._read_websocket = CoroutineMock(side_effect=KeyboardInterrupt)  # type: ignore
-    with patch.object(ClientSession, ClientSession.ws_connect.__name__):
-        job_controller.stream_logs(job_id)
-
-
-@pytest.mark.parametrize("cluster_id_found_in_last_job_run", [True, False])
-def test_download_logs(
-    mock_auth_api_client, cluster_id_found_in_last_job_run: bool
-) -> None:
-    prodjob_id = "prodjob_123"
-    cluster_id = "session_123"
-    job_controller = JobController(log=FakeLogger())
-    if cluster_id_found_in_last_job_run:
-        mock_prod_job = Mock(last_job_run=Mock(cluster_id=cluster_id), state=Mock())
-    else:
-        mock_prod_job = Mock(last_job_run=None, state=Mock(cluster_id=cluster_id))
-    job_controller._get_latest_job_run = Mock(  # type: ignore
-        return_value=mock_prod_job
+    # Test connection issues are raised and logged as error output
+    # Indirectly tests `handle_and_log_lines_in_click_exception()`
+    job_controller.log.error = Mock()  # type: ignore
+    with patch.object(ClientSession, ClientSession.ws_connect.__name__), patch.object(
+        job_controller_module,
+        _stream_log_from_ray_websocket.__name__,
+        new=CoroutineMock(side_effect=asyncio.TimeoutError),
+    ):
+        job_controller._print_logs_for_active_job_run(job_id, should_follow=True)
+    log_content_args = job_controller.log.error.call_args_list[0]
+    log_content: str = get_args_from_call(log_content_args)[0]
+    assert (
+        "only supported if the Anyscale client (SDK / CLI) can reach the cluster"
+        in log_content
     )
-    mock_download_logs = Mock()
-    job_controller.logs_controller.download_logs = mock_download_logs  # type: ignore
-    job_controller.download_logs(prodjob_id)
-    job_controller._get_latest_job_run.assert_called_once_with(prodjob_id)
-    mock_download_logs.assert_called_once_with(
-        filter=LogFilter(cluster_id=cluster_id, glob="/job-driver-"),
-        write_to_stdout=True,
+
+
+def test_print_logs_for_inactive_job_run(mock_auth_api_client) -> None:
+    job_id = "job_123"
+    job_controller = JobController(log=Mock())
+    job_controller.log.spinner = Mock(return_value=MockAsyncContextManagerReturnValue())  # type: ignore
+    log_call_return_vals = ["", "logs\nretrieved"]
+
+    # Test `_get_job_logs_from_storage_bucket`` called twice, and `log.log()` called twice with split logs
+    with patch.object(
+        job_controller_module, poll.__name__, return_value=["bucket", "hit", "twice"]
+    ) as mock_poll, patch.object(
+        job_controller_module,
+        _get_job_logs_from_storage_bucket.__name__,
+        new=CoroutineMock(side_effect=log_call_return_vals),
+    ) as mock_get_job_logs_from_storage_bucket:
+        job_controller._print_logs_for_inactive_job_run(job_id)
+    job_controller.log.spinner.assert_called_once_with("Retrieving logs...")
+    mock_poll.assert_called_once_with(interval_secs=5, max_iter=10)
+    assert mock_get_job_logs_from_storage_bucket.call_count == len(log_call_return_vals)
+    assert job_controller.log.log.call_count == 2  # type: ignore
+    assert get_args_from_call(job_controller.log.log.call_args_list[0])[0] == "logs"  # type: ignore
+    assert (
+        get_args_from_call(job_controller.log.log.call_args_list[1])[0] == "retrieved"  # type: ignore
+    )
+
+    # Test `log.error()` called when logs are empty after polling
+    empty_log_call_return_vals = ["", ""]
+    with patch.object(
+        job_controller_module, poll.__name__, return_value=["tried", "twice"]
+    ) as mock_poll, patch.object(
+        job_controller_module,
+        _get_job_logs_from_storage_bucket.__name__,
+        new=CoroutineMock(side_effect=empty_log_call_return_vals),
+    ) as mock_get_job_logs_from_storage_bucket:
+        job_controller._print_logs_for_inactive_job_run(job_id)
+    assert mock_get_job_logs_from_storage_bucket.call_count == len(
+        empty_log_call_return_vals
+    )
+    job_controller.log.error.assert_called_once_with(  # type: ignore
+        "Logs are not available for this job run."
     )
 
 
@@ -909,7 +1114,7 @@ def test_download_logs(
     "mode",
     ["empty", "single", "paginate", "follow", "flush_success", "wait_for_running"],
 )
-def test_loki_logs(mock_auth_api_client, capsys, mode) -> None:
+def test_print_logs_legacy(mock_auth_api_client, capsys, mode) -> None:
     job_controller = JobController(log=FakeLogger())
     fake_log_provider_cls = fake_log_provider(mode)
 
@@ -942,8 +1147,8 @@ def test_loki_logs(mock_auth_api_client, capsys, mode) -> None:
     )
 
     def do_loki_logs() -> None:
-        job_controller.loki_logs(
-            job_id="mock_job_id",
+        job_controller._print_logs_legacy(
+            job=Mock(id="mock_job_id"),
             should_follow=mode in ["follow", "flush_success", "wait_for_running"],
             log_provider_cls=fake_log_provider_cls,
         )
@@ -976,16 +1181,55 @@ def test_loki_logs(mock_auth_api_client, capsys, mode) -> None:
         raise Exception("unreachable")
 
 
+def test_print_logs_legacy_errors_when_job_on_v2_cloud(mock_auth_api_client) -> None:
+    job_controller = JobController()
+    mock_job = Mock(id="prodjob_123")
+    with patch.object(
+        job_controller_module,
+        _is_production_job_on_v2_cloud.__name__,  # type: ignore
+        return_value=True,
+    ) as mock_is_production_job_on_v2_cloud, pytest.raises(
+        UnsupportedLogRetrievalMethodError, match="Logs are not stored Anyscale-side"
+    ):
+        job_controller._print_logs_legacy(mock_job)
+    mock_is_production_job_on_v2_cloud.assert_called_once_with(
+        job_controller.anyscale_api_client, mock_job.id
+    )
+
+
+def test_print_logs_legacy_errors_when_not_logged_at_all(mock_auth_api_client) -> None:
+    job_controller = JobController()
+    mock_job = Mock(
+        state=Mock(current_state=HaJobStates.TERMINATED, error=None),
+        last_job_run_id="job_123",
+    )
+    fake_log_provider_cls = fake_log_provider("empty")
+    with patch.object(
+        job_controller_module,
+        _is_production_job_on_v2_cloud.__name__,  # type: ignore
+        return_value=False,
+    ), pytest.raises(ExpectedLogRetrievalError):
+        job_controller._print_logs_legacy(
+            mock_job, log_provider_cls=fake_log_provider_cls
+        )
+
+
 def test_logs(mock_auth_api_client) -> None:
     job_controller = JobController()
     prod_job_id = "prodjob_123"
-    job_controller._resolve_job_object = Mock(return_value=Mock(id=prod_job_id))  # type: ignore
-    job_controller.stream_logs = mock_stream_logs = Mock()  # type: ignore
-    job_controller.download_logs = mock_download_logs = Mock()  # type: ignore
-    job_controller.loki_logs = mock_loki_logs = Mock()  # type: ignore
+    last_job_run_id = "job_123"
+    mock_job = Mock(
+        id=prod_job_id,
+        last_job_run_id=last_job_run_id,
+        state=Mock(current_state=HaJobStates.TERMINATED),
+    )
+    job_controller._wait_for_a_job_run = Mock(return_value=mock_job)  # type: ignore
+    job_controller._print_logs_for_active_job_run = mock_stream_logs = Mock()  # type: ignore
+    job_controller._print_logs_legacy = mock_loki_logs = Mock()  # type: ignore
+    job_controller._print_logs_for_inactive_job_run = mock_download_logs = Mock()  # type: ignore
 
     def reset_mocks() -> None:
-        for mock in [mock_download_logs, mock_stream_logs, mock_loki_logs]:
+        for mock in [mock_stream_logs, mock_loki_logs, mock_download_logs]:
             mock.reset_mock()
 
     def do_assertions(
@@ -996,37 +1240,40 @@ def test_logs(mock_auth_api_client) -> None:
         if not not_called:
             not_called = []
         for mock in called:
-            mock.assert_called_once_with(prod_job_id)
+            mock.assert_called_once()
         for mock in not_called:
             mock.assert_not_called()
 
-    # Test download logs by default
+    # Test Loki logs and does not follow by default if job is not in RUNNING state
+    job_controller.logs(job_id=prod_job_id)
+    do_assertions(not_called=[mock_stream_logs, mock_download_logs])
+    mock_loki_logs.assert_called_once_with(mock_job, False)
+
+    # Test download logs (ie. falls back) if Loki has no logs (raises `ExpectedLogRetrievalError`)
+    reset_mocks()
+    job_controller._print_logs_legacy.side_effect = ExpectedLogRetrievalError
     job_controller.logs(job_id=prod_job_id)
     do_assertions(
-        called=[mock_download_logs], not_called=[mock_stream_logs, mock_loki_logs]
+        called=[mock_loki_logs, mock_download_logs], not_called=[mock_stream_logs]
     )
 
-    # Test stream logs if follow specified
+    # Test stream if job is RUNNING
     reset_mocks()
-    job_controller.logs(job_id=prod_job_id, should_follow=True)
+    mock_job.state.current_state = HaJobStates.RUNNING
+    job_controller.logs(job_id=prod_job_id)
     do_assertions(
-        called=[mock_stream_logs], not_called=[mock_download_logs, mock_loki_logs]
+        called=[mock_stream_logs], not_called=[mock_loki_logs, mock_download_logs]
     )
 
-    # Test download if stream fails
+    # Test `all_attempts = True`
     reset_mocks()
-    job_controller.stream_logs.side_effect = Exception
-    job_controller.logs(job_id=prod_job_id, should_follow=True)
-    do_assertions(
-        called=[mock_stream_logs, mock_download_logs], not_called=[mock_loki_logs]
+    job_controller._get_all_job_runs = Mock(  # type: ignore
+        return_value=[Mock(id="job_1"), Mock(id=last_job_run_id),]
     )
-
-    # Test loki logs if both stream and download fails
-    reset_mocks()
-    job_controller.download_logs.side_effect = Exception
-    job_controller.logs(job_id=prod_job_id, should_follow=True)
-    do_assertions(called=[mock_stream_logs, mock_download_logs])
-    mock_loki_logs.assert_called_once_with(prod_job_id, True)
+    job_controller.logs(job_id=prod_job_id, should_follow=True, all_attempts=True)
+    job_controller._get_all_job_runs.assert_called_once_with(prod_job_id)
+    mock_download_logs.assert_called_once_with("job_1")
+    mock_stream_logs.assert_called_once_with(last_job_run_id, True)
 
 
 def test_wait_for_job_running(mock_auth_api_client) -> None:
@@ -1046,7 +1293,7 @@ def test_wait_for_job_running(mock_auth_api_client) -> None:
         ret.state.current_state = status
         return ret
 
-    job_controller._get_latest_job_run = Mock(side_effect=get_latest_job_run)  # type: ignore
+    job_controller._get_job = Mock(side_effect=get_latest_job_run)  # type: ignore
     job_controller._wait_for_job_running("fake_job_id")
     assert i == 5
 
@@ -1183,3 +1430,57 @@ def test_job_wait(mock_auth_api_client) -> None:
     # Test parsing logic
     with pytest.raises(click.ClickException):
         job_controller.wait("name", target_state="FAKE_STATE")
+
+
+def test_override_runtime_env_config_and_infer_project_id(mock_auth_api_client):
+    """Test helper functions in submit_from_config function
+    Test that runtime_env is overwrriten from override_runtime_env_config
+    Test the project_id is autopopulated from infer_project_id
+    """
+    config_dict = {
+        "name": "mock_name",
+        "entrypoint": "mock_entrypoint",
+        "description": "mock_description",
+        "build_id": "mock_build_id",
+        "compute_config_id": "mock_compute_config_id",
+    }
+
+    assert "project_id" not in config_dict
+
+    mock_project_id = "default"
+
+    with patch.multiple(
+        "anyscale.models.job_model", validate_successful_build=Mock(),
+    ):
+        job_config = JobConfig.parse_obj(config_dict)
+
+    new_runtime_env = {"working_dir": "s3://bucket"}
+    mock_override_config = Mock(return_value=new_runtime_env)
+
+    mock_infer_project_id = Mock(return_value=mock_project_id)
+
+    with patch.multiple(
+        "anyscale.controllers.job_controller",
+        infer_project_id=mock_infer_project_id,
+        override_runtime_env_config=mock_override_config,
+    ):
+        job_controller = JobController()
+        job_controller.api_client.create_job_api_v2_decorated_ha_jobs_create_post = (
+            Mock()
+        )
+        job_controller._get_maximum_uptime_output = Mock()
+        job_controller.submit_from_config(job_config)
+        job_controller.api_client.create_job_api_v2_decorated_ha_jobs_create_post.assert_called_once_with(
+            CreateInternalProductionJob(
+                name=config_dict["name"],
+                description=config_dict["description"],
+                project_id=mock_project_id,
+                config=ProductionJobConfig(
+                    entrypoint=job_config.entrypoint,
+                    runtime_env=new_runtime_env,
+                    build_id=job_config.build_id,
+                    compute_config_id=job_config.compute_config_id,
+                    max_retries=job_config.max_retries,
+                ),
+            )
+        )
