@@ -1,16 +1,15 @@
+from copy import deepcopy
 import json
 import os
-from typing import Optional
-from unittest.mock import call, Mock, mock_open, patch
+from typing import Collection, Optional
+from unittest.mock import ANY, call, Mock, mock_open, patch
 
 import click
+from pydantic import ValidationError
 import pytest
 
-from anyscale.client.openapi_client.models.apply_production_service_v2_api_model import (
-    ApplyProductionServiceV2APIModel,
-)
-from anyscale.client.openapi_client.models.create_production_service import (
-    CreateProductionService,
+from anyscale.client.openapi_client.models.apply_production_service_v2_model import (
+    ApplyProductionServiceV2Model,
 )
 from anyscale.client.openapi_client.models.decoratedlistserviceapimodel_list_response import (
     DecoratedlistserviceapimodelListResponse,
@@ -18,10 +17,8 @@ from anyscale.client.openapi_client.models.decoratedlistserviceapimodel_list_res
 from anyscale.client.openapi_client.models.list_response_metadata import (
     ListResponseMetadata,
 )
-from anyscale.client.openapi_client.models.production_job_config import (
-    ProductionJobConfig,
-)
 from anyscale.controllers.service_controller import ServiceConfig, ServiceController
+from anyscale.utils.workload_types import Workload
 
 
 TEST_SERVICE_ID = "test-service-id"
@@ -30,15 +27,14 @@ TEST_SERVICE_NAME = "test-service-name"
 
 def test_service_config_model_with_entrypoint():
     config_dict = {
-        "entrypoint": "python test.py",
+        "name": "test_service",
         "project_id": "test_project_id",
         "build_id": "test_build_id",
         "compute_config_id": "test_compute_config_id",
-        "healthcheck_url": "test.url",
     }
     mock_validate_successful_build = Mock()
     with patch.multiple(
-        "anyscale.controllers.job_controller",
+        "anyscale.models.job_model",
         validate_successful_build=mock_validate_successful_build,
     ):
         service_config = ServiceConfig.parse_obj(config_dict)
@@ -46,167 +42,269 @@ def test_service_config_model_with_entrypoint():
 
 
 def test_service_config_model_with_ray_serve_config():
+    runtime_env = {
+        "pip": ["requests"],
+        "working_dir": "https://github.com/anyscale/docs_examples/archive/refs/heads/main.zip",
+    }
     config_dict = {
-        "ray_serve_config": {"runtime_env": {"pip": ["requests"]}},
+        "name": "test_service",
+        "ray_serve_config": {
+            "import_path": "serve_hello:entrypoint",
+            "runtime_env": runtime_env,
+        },
         "project_id": "test_project_id",
         "build_id": "test_build_id",
         "compute_config_id": "test_compute_config_id",
     }
     mock_validate_successful_build = Mock()
     with patch.multiple(
-        "anyscale.controllers.job_controller",
+        "anyscale.models.job_model",
         validate_successful_build=mock_validate_successful_build,
     ):
         service_config = ServiceConfig.parse_obj(config_dict)
-        assert service_config.entrypoint is None
+        assert service_config.runtime_env is None
+        assert service_config.ray_serve_config["runtime_env"] == runtime_env
 
 
-@pytest.mark.parametrize("use_default_project", [True, False])
-@pytest.mark.parametrize("access", ["public", "private"])
-@pytest.mark.parametrize("maximum_uptime_minutes", [60, None])
-def test_update_service(
-    mock_auth_api_client,
-    use_default_project: bool,
-    access: str,
-    maximum_uptime_minutes: Optional[int],
-) -> None:
+def test_service_config_model_with_ray_serve_config_working_dir():
+    runtime_env = {"pip": ["requests"], "working_dir": "."}
     config_dict = {
-        "entrypoint": "mock_entrypoint",
-        "build_id": "mock_build_id",
-        "compute_config_id": "mock_compute_config_id",
-        "healthcheck_url": "mock_healthcheck_url",
-        "access": access,
+        "name": "test_service",
+        "ray_serve_config": {
+            "import_path": "serve_hello:entrypoint",
+            "runtime_env": runtime_env,
+        },
+        "project_id": "test_project_id",
+        "build_id": "test_build_id",
+        "compute_config_id": "test_compute_config_id",
     }
-    mock_logger = Mock()
-    service_controller = ServiceController(log=mock_logger)
-    if use_default_project:
-        mock_infer_project_id = Mock(return_value="mock_default_project_id")
-    else:
-        mock_infer_project_id = Mock(return_value="mock_project_id")
-
     mock_validate_successful_build = Mock()
+    with patch.multiple(
+        "anyscale.models.job_model",
+        validate_successful_build=mock_validate_successful_build,
+    ):
+        service_config = ServiceConfig.parse_obj(config_dict)
+        assert service_config.ray_serve_config["runtime_env"] == runtime_env
+        assert service_config.ray_gcs_external_storage_config is None
 
+
+def test_service_config_model_with_ray_gcs_external_storage_config():
+    config_dict = {
+        "name": "test_service",
+        "ray_serve_config": {"runtime_env": {"pip": ["requests"], "working_dir": "."}},
+        "project_id": "test_project_id",
+        "build_id": "test_build_id",
+        "compute_config_id": "test_compute_config_id",
+        "ray_gcs_external_storage_config": {"address": "addr:port",},
+    }
+    mock_validate_successful_build = Mock()
+    with patch.multiple(
+        "anyscale.models.job_model",
+        validate_successful_build=mock_validate_successful_build,
+    ):
+        service_config = ServiceConfig.parse_obj(config_dict)
+        assert (
+            service_config.ray_gcs_external_storage_config
+            == config_dict["ray_gcs_external_storage_config"]
+        )
+        assert service_config.runtime_env is None
+
+
+@pytest.mark.parametrize(
+    ("ray_serve_config", "runtime_env"),
+    [[{"runtime_env": {"pip": ["requests"], "working_dir": "."}}, None],],
+)
+def test_generate_config_from_file(
+    mock_auth_api_client, ray_serve_config: Optional[str], runtime_env: Optional[str]
+):
+    """
+    This test checks that the config file for Service v2
+    can be properly parsed.
+    """
+    service_controller = ServiceController()
+
+    name = "test_service_name"
+    description = "mock_description"
+    build_id = "test_build_id"
+    compute_config_id = "test_compute_config_id"
+    project_id = "test_project_id"
+    rollout_strategy = "ROLLOUT"
+    canary_percent = 100
+    config_dict = {
+        "name": name,
+        "description": description,
+        "ray_serve_config": ray_serve_config,
+        "runtime_env": runtime_env,
+        "project_id": project_id,
+        "build_id": build_id,
+        "compute_config_id": compute_config_id,
+        "version": "test_abc",
+        "canary_percent": canary_percent,
+        "rollout_strategy": rollout_strategy,
+    }
+    mock_validate_successful_build = Mock()
     with patch(
         "builtins.open", mock_open(read_data=json.dumps(config_dict))
-    ), patch.multiple(
-        "anyscale.controllers.service_controller",
-        infer_project_id=mock_infer_project_id,
-    ), patch.object(
-        ServiceController,
-        "_get_maximum_uptime_minutes",
-        return_value=maximum_uptime_minutes,
-    ), patch.multiple(
-        "anyscale.controllers.job_controller",
+    ), patch.multiple("os.path", exists=Mock(return_value=True)), patch.multiple(
+        "anyscale.models.job_model",
         validate_successful_build=mock_validate_successful_build,
-    ), patch.multiple(
-        "os.path", exists=Mock(return_value=True)
     ):
-        service_controller.deploy(
-            "mock_config_file", name="mock_name", description="mock_description",
+        config = service_controller.generate_config_from_file(
+            "test_ser8vice_config_file",
         )
 
-    service_controller.api_client.apply_service_api_v2_decorated_ha_jobs_apply_service_put.assert_called_once_with(
-        CreateProductionService(
-            name="mock_name",
-            description="mock_description",
-            project_id="mock_default_project_id"
-            if use_default_project
-            else "mock_project_id",
-            config=ProductionJobConfig(  # noqa: PIE804
-                **{
-                    "entrypoint": "mock_entrypoint",
-                    "build_id": "mock_build_id",
-                    "compute_config_id": "mock_compute_config_id",
-                }
-            ),
-            healthcheck_url="mock_healthcheck_url",
-            access=access,
+        expected_service_config = ServiceConfig(
+            name=name,
+            description=description,
+            ray_serve_config=ray_serve_config,
+            runtime_env=runtime_env,
+            project_id=project_id,
+            build_id=build_id,
+            compute_config_id=compute_config_id,
+            version="test_abc",
+            canary_percent=canary_percent,
+            rollout_strategy=rollout_strategy,
         )
-    )
-    assert mock_logger.info.call_count == 4
+    assert config == expected_service_config
 
 
-def test_service_submit_parse_logic(mock_auth_api_client) -> None:
+@pytest.mark.parametrize("add_extra_field", [True, False])
+def test_config_validation(add_extra_field, mock_auth_api_client):
+    """
+    This test checks that an error is thrown if extra/mispelled fields
+    are added into the config file
+    """
     service_controller = ServiceController()
-    service_controller.generate_config_from_entrypoint = Mock()  # type: ignore
-    service_controller.generate_config_from_file = Mock()  # type: ignore
-    service_controller.deploy_from_config = Mock()  # type: ignore
 
-    # We are not in a workspace, so entrypoint should not be allowed
-    with pytest.raises(click.ClickException):
-        service_controller.deploy(
-            "file", None, None, entrypoint=["entrypoint"], is_entrypoint_cmd=False
-        )
+    name = "test_service_name"
+    description = "mock_description"
+    build_id = "test_build_id"
+    compute_config_id = "test_compute_config_id"
+    project_id = "test_project_id"
+    rollout_strategy = "ROLLOUT"
+    extra_field_value = "*****"
+    canary_percent = 100
+    config_dict = {
+        "name": name,
+        "description": description,
+        "ray_serve_config": {},
+        "project_id": project_id,
+        "build_id": build_id,
+        "compute_config_id": compute_config_id,
+        "version": "test_abc",
+        "canary_percent": canary_percent,
+        "rollout_strategy": rollout_strategy,
+    }
 
-    with pytest.raises(click.ClickException):
-        service_controller.deploy(
-            "file", None, None, entrypoint=["entrypoint"], is_entrypoint_cmd=True
-        )
+    if add_extra_field:
+        extra_field_value = "*****"
+        config_dict["extra_field"] = extra_field_value
 
-    with pytest.raises(click.ClickException):
-        service_controller.deploy(
-            "file",
-            None,
-            None,
-            entrypoint=["entrypoint", "commands"],
-            is_entrypoint_cmd=True,
-        )
-
-    # Simulate a workspace
-    with patch.dict(
-        os.environ, {"ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": "fake_workspace_id"}
-    ), patch.multiple(
-        "anyscale.controllers.service_controller",
-        override_runtime_env_for_local_working_dir=Mock(
-            return_value={"rewritten": True}
-        ),
+    mock_validate_successful_build = Mock()
+    with patch(
+        "builtins.open", mock_open(read_data=json.dumps(config_dict))
+    ), patch.multiple("os.path", exists=Mock(return_value=True)), patch.multiple(
+        "anyscale.models.job_model",
+        validate_successful_build=mock_validate_successful_build,
     ):
-        # Fails due to is_entrypoint_cmd being False
-        with pytest.raises(click.ClickException):
-            service_controller.deploy(
-                "file",
-                name=None,
-                description=None,
-                entrypoint=["entrypoint"],
-                is_entrypoint_cmd=False,
+        if add_extra_field:
+            with pytest.raises(ValidationError):
+                service_controller.generate_config_from_file(
+                    "test_service_config_file", name=name, description=description,
+                )
+        else:
+            service_controller.generate_config_from_file(
+                "test_service_config_file", name=name, description=description,
             )
 
-        mock_config = Mock()
-        service_controller.generate_config_from_file.return_value = mock_config
-        service_controller.deploy(
-            "file", name=None, description=None, entrypoint=[], is_entrypoint_cmd=False
-        )
-        service_controller.generate_config_from_file.assert_called_once_with(
-            "file", name=None, description=None, healthcheck_url=None
-        )
-        service_controller.deploy_from_config.assert_called_once_with(mock_config)
-        service_controller.generate_config_from_file.reset_mock()
-        service_controller.deploy_from_config.reset_mock()
 
-        mock_config = Mock()
-        service_controller.generate_config_from_entrypoint.return_value = mock_config
-        service_controller.deploy(
-            "file",
-            name=None,
-            description=None,
-            entrypoint=["entrypoint"],
-            is_entrypoint_cmd=True,
+@pytest.mark.parametrize(
+    ("working_dir"),
+    [
+        ("https://github.com/anyscale/docs_examples/archive/refs/heads/main.zip"),
+        ("s3://already-a-uri/path"),
+    ],
+)
+def test_generate_config_from_file_inside_workspace_no_working_dir_override(
+    mock_auth_api_client, working_dir: str,
+):
+    """
+    This test checks that if the service v2 is launched from the workspace with
+    a remote working dir being specified, the method would inspect the remote url
+    and not override it.
+    """
+    service_controller = ServiceController()
+
+    name = "test_service_name"
+    description = "mock_description"
+    ray_serve_config = {
+        "runtime_env": {"pip": ["requests"], "working_dir": working_dir,}
+    }
+    build_id = "test_build_id"
+    compute_config_id = "test_compute_config_id"
+    project_id = "test_project_id"
+    rollout_strategy = "ROLLOUT"
+    canary_percent = 100
+    config_dict = {
+        "name": name,
+        "description": description,
+        "ray_serve_config": ray_serve_config,
+        "project_id": project_id,
+        "build_id": build_id,
+        "compute_config_id": compute_config_id,
+        "version": "test_abc",
+        "canary_percent": canary_percent,
+        "rollout_strategy": rollout_strategy,
+    }
+    mock_validate_successful_build = Mock()
+
+    workspace_id = "Test_workspace_id"
+    with patch.dict(
+        os.environ,
+        {
+            "ANYSCALE_EXPERIMENTAL_WORKSPACE_ID": workspace_id,
+            "ANYSCALE_SESSION_ID": "test_session_id",
+        },
+    ), patch(
+        "builtins.open", mock_open(read_data=json.dumps(config_dict))
+    ), patch.multiple(
+        "os.path", exists=Mock(return_value=True)
+    ), patch.multiple(
+        "anyscale.models.job_model",
+        validate_successful_build=mock_validate_successful_build,
+    ):
+        config = service_controller.generate_config_from_file(
+            "test_service_config_file",
         )
-        service_controller.generate_config_from_entrypoint.assert_called_once_with(
-            ["file", "entrypoint"], name=None, description=None, healthcheck_url=None
+
+        expected_service_config = ServiceConfig(
+            name=name,
+            description=description,
+            ray_serve_config=ray_serve_config,
+            project_id=project_id,
+            build_id=build_id,
+            compute_config_id=compute_config_id,
+            version="test_abc",
+            canary_percent=canary_percent,
+            rollout_strategy=rollout_strategy,
         )
-        service_controller.deploy_from_config.assert_called_once_with(mock_config)
+
+    assert config == expected_service_config
 
 
 @pytest.mark.parametrize("override_name", [None, "override_service_name"])
 @pytest.mark.parametrize("override_version", [None, "override_version"])
-@pytest.mark.parametrize("override_canary_weight", [None, 40])
+@pytest.mark.parametrize(
+    ("override_canary_percent", "first_get_ha_job_throws_error"),
+    [[None, False], [0, False], [0, True], [40, False], [40, True]],
+)
 @pytest.mark.parametrize("override_rollout_strategy", [None, "IN_PLACE"])
 def test_service_rollout(
     mock_auth_api_client,
     override_name: Optional[str],
     override_version: Optional[str],
-    override_canary_weight: Optional[int],
+    override_canary_percent: Optional[int],
+    first_get_ha_job_throws_error: bool,
     override_rollout_strategy: Optional[str],
 ) -> None:
     """
@@ -215,10 +313,26 @@ def test_service_rollout(
     Please reference services_internal_api_models.py for more details.
     """
     service_controller = ServiceController()
+    service_controller.api_client.apply_service_v2_api_v2_services_v2_apply_put = Mock(
+        return_value=Mock(
+            result=(
+                Mock(
+                    versions=[
+                        Mock(),
+                        Mock(production_job_ids=["test_production_job_id"]),
+                    ]
+                )
+            )
+        )
+    )
+    if first_get_ha_job_throws_error:
+        service_controller.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get = Mock(
+            side_effect=[Exception("Error!"), Mock()]
+        )
 
     name = "test_service_name"
     description = "mock_description"
-    ray_serve_config = {"runtime_env": {"pip": ["requests"]}}
+    ray_serve_config = {"runtime_env": {"pip": ["requests"], "working_dir": "."}}
     build_id = "test_build_id"
     compute_config_id = "test_compute_config_id"
     project_id = "test_project_id"
@@ -232,25 +346,31 @@ def test_service_rollout(
         "build_id": build_id,
         "compute_config_id": compute_config_id,
         "version": "test_abc",
-        "canary_weight": 100,
+        "canary_percent": 100,
         "rollout_strategy": rollout_strategy,
     }
     mock_validate_successful_build = Mock()
+    mock_override_runtime_env_config = Mock(
+        return_value={"pip": ["requests"], "working_dir": "."}
+    )
     with patch(
         "builtins.open", mock_open(read_data=json.dumps(config_dict))
     ), patch.multiple("os.path", exists=Mock(return_value=True)), patch.multiple(
-        "anyscale.controllers.job_controller",
+        "anyscale.models.job_model",
         validate_successful_build=mock_validate_successful_build,
+    ), patch.multiple(
+        "anyscale.controllers.service_controller",
+        override_runtime_env_config=mock_override_runtime_env_config,
     ):
         service_controller.rollout(
             "test_ser8vice_config_file",
             name=override_name,
             version=override_version,
-            canary_percent=override_canary_weight,
+            canary_percent=override_canary_percent,
             rollout_strategy=override_rollout_strategy,
         )
 
-    service_v2_config = ApplyProductionServiceV2APIModel(
+    service_v2_config = ApplyProductionServiceV2Model(
         name=override_name if override_name else name,
         description=description,
         build_id=build_id,
@@ -258,7 +378,9 @@ def test_service_rollout(
         project_id=project_id,
         ray_serve_config=ray_serve_config,
         version=override_version if override_version else "test_abc",
-        canary_weight=override_canary_weight if override_canary_weight else 100,
+        canary_percent=override_canary_percent
+        if override_canary_percent is not None
+        else 100,
         rollout_strategy=override_rollout_strategy
         if override_rollout_strategy
         else "ROLLOUT",
@@ -699,3 +821,213 @@ def test_archive(mock_auth_api_client):
     service_controller.api_client.archive_job_api_v2_decorated_ha_jobs_production_job_id_archive_post.assert_called_once_with(
         "service_abc123",
     )
+
+
+@pytest.mark.parametrize("init_runtime_env", [None, {}, {"working_dir": "."}])
+def test_override_runtime_env_config_and_infer_project_id_v2(
+    mock_auth_api_client, init_runtime_env: Collection[str],
+):
+    """Test helper functions in rollout function (service v2 path)
+    Test that runtime_env is overwrriten from override_runtime_env_config
+    Test the project_id is autopopulated from infer_project_id
+    """
+    config_dict = {
+        "name": "mock_name",
+        "description": "mock_description",
+        "build_id": "mock_build_id",
+        "compute_config_id": "mock_compute_config_id",
+        "ray_serve_config": {
+            "import_path": "mock_import_path",
+            "runtime_env": init_runtime_env,
+        },
+    }
+
+    assert "project_id" not in config_dict
+
+    mock_project_id = "default"
+
+    with patch.multiple(
+        "anyscale.models.job_model", validate_successful_build=Mock(),
+    ):
+        service_config = ServiceConfig.parse_obj(config_dict)
+
+    new_runtime_env = {"working_dir": "s3://bucket"}
+    mock_override_config = Mock(return_value=new_runtime_env)
+
+    mock_infer_project_id = Mock(return_value=mock_project_id)
+
+    mock_generate_config_from_file = Mock(return_value=service_config)
+
+    with patch.multiple(
+        "anyscale.controllers.service_controller",
+        infer_project_id=mock_infer_project_id,
+        override_runtime_env_config=mock_override_config,
+    ), patch.object(
+        ServiceController, "generate_config_from_file", mock_generate_config_from_file
+    ):
+        service_controller = ServiceController()
+        service_controller.api_client = Mock()
+        service_controller.anyscale_api_client = Mock()
+        service_controller.log = Mock()
+        service_controller.api_client.apply_service_v2_api_v2_services_v2_apply_put = (
+            Mock()
+        )
+
+        service_controller.rollout(service_config)
+
+        mock_override_config.assert_called_once_with(
+            anyscale_api_client=service_controller.anyscale_api_client,
+            api_client=service_controller.api_client,
+            compute_config_id=config_dict["compute_config_id"],
+            runtime_env=init_runtime_env,
+            workload_type=Workload.SERVICES,
+            log=service_controller.log,
+        )
+
+        expected_ray_serve_config = {
+            "import_path": "mock_import_path",
+            "runtime_env": new_runtime_env,
+        }
+
+        service_controller.api_client.apply_service_v2_api_v2_services_v2_apply_put.assert_called_once_with(
+            ApplyProductionServiceV2Model(
+                name=config_dict["name"],
+                description=config_dict["description"],
+                project_id=mock_project_id,
+                canary_percent=None,
+                ray_serve_config=expected_ray_serve_config,
+                build_id=config_dict["build_id"],
+                compute_config_id=service_config.compute_config_id,
+                rollout_strategy=service_config.rollout_strategy,
+            )
+        )
+
+
+def test_overwrite_runtime_env_in_v2_ray_serve_config(mock_auth_api_client):
+    """Test that runtime env is overwritten in v2 single-app configs as expected.
+    Delegates the runtime env overwriting logic to override_runtime_env_config(), simply
+    makes sure that override_runtime_env_config is called with the correct arguments.
+    """
+    service_controller = ServiceController()
+
+    # Runtime environments
+    original_runtime_env = {
+        "pip": ["requests"],
+        "working_dir": ".",
+        "upload_path": "s3://already-a-uri/path",
+    }
+    rewritten_runtime_env = {
+        "pip": ["requests"],
+        "working_dir": "s3://bucket",
+    }
+
+    # Service configs
+    with patch.multiple(
+        "anyscale.models.job_model", validate_successful_build=Mock(),
+    ):
+        service_config = ServiceConfig.parse_obj(
+            {
+                "name": "mock_name",
+                "description": "mock_description",
+                "build_id": "mock_build_id",
+                "compute_config_id": "mock_compute_config_id",
+                "ray_serve_config": {
+                    "import_path": "a.b.c",
+                    "runtime_env": original_runtime_env,
+                },
+            }
+        )
+        expected_service_config = deepcopy(service_config)
+        expected_service_config.ray_serve_config["runtime_env"] = rewritten_runtime_env
+
+    # Perform test
+    override_runtime_env_config_mock = Mock(return_value=rewritten_runtime_env)
+    with patch.multiple(
+        "anyscale.controllers.service_controller",
+        override_runtime_env_config=override_runtime_env_config_mock,
+    ):
+        service_controller._overwrite_runtime_env_in_v2_ray_serve_config(service_config)
+
+    calls = [
+        call(
+            runtime_env=original_runtime_env,
+            anyscale_api_client=ANY,
+            api_client=ANY,
+            workload_type=Workload.SERVICES,
+            compute_config_id="mock_compute_config_id",
+            log=ANY,
+        )
+    ]
+    assert service_config == expected_service_config
+    override_runtime_env_config_mock.assert_has_calls(calls)
+
+
+def test_overwrite_runtime_env_in_v2_ray_serve_config_multi_app(mock_auth_api_client):
+    """Test that runtime env for each app is overwritten in v2 multi-app configs.
+    Delegates the runtime env overwriting logic to override_runtime_env_config(), simply
+    makes sure that override_runtime_env_config is called with the correct arguments.
+    """
+    service_controller = ServiceController()
+
+    # Runtime environments
+    original_runtime_env1 = {
+        "pip": ["requests"],
+        "working_dir": ".",
+        "upload_path": "s3://already-a-uri/path",
+    }
+    original_runtime_env2 = None
+    rewritten_runtime_env = {
+        "pip": ["requests"],
+        "working_dir": "s3://bucket",
+    }
+
+    # Service configs
+    with patch.multiple(
+        "anyscale.models.job_model", validate_successful_build=Mock(),
+    ):
+        service_config = ServiceConfig.parse_obj(
+            {
+                "name": "mock_name",
+                "description": "mock_description",
+                "build_id": "mock_build_id",
+                "compute_config_id": "mock_compute_config_id",
+                "ray_serve_config": {
+                    "applications": [
+                        {"import_path": "a.b", "runtime_env": original_runtime_env1},
+                        {"import_path": "c.d"},
+                    ]
+                },
+            }
+        )
+        expected_service_config = deepcopy(service_config)
+        for app in expected_service_config.ray_serve_config["applications"]:
+            app["runtime_env"] = rewritten_runtime_env
+
+    # Perform test
+    override_runtime_env_config_mock = Mock(return_value=rewritten_runtime_env)
+    with patch.multiple(
+        "anyscale.controllers.service_controller",
+        override_runtime_env_config=override_runtime_env_config_mock,
+    ):
+        service_controller._overwrite_runtime_env_in_v2_ray_serve_config(service_config)
+
+    calls = [
+        call(
+            runtime_env=original_runtime_env1,
+            anyscale_api_client=ANY,
+            api_client=ANY,
+            workload_type=Workload.SERVICES,
+            compute_config_id="mock_compute_config_id",
+            log=ANY,
+        ),
+        call(
+            runtime_env=original_runtime_env2,
+            anyscale_api_client=ANY,
+            api_client=ANY,
+            workload_type=Workload.SERVICES,
+            compute_config_id="mock_compute_config_id",
+            log=ANY,
+        ),
+    ]
+    assert service_config == expected_service_config
+    override_runtime_env_config_mock.assert_has_calls(calls)
