@@ -1,4 +1,5 @@
 import json
+import os
 import random
 import re
 import string
@@ -10,9 +11,22 @@ import boto3
 from moto import mock_cloudformation, mock_ec2, mock_efs, mock_iam, mock_s3
 import pytest
 
+from anyscale.aws_iam_policies import (
+    AMAZON_ECR_READONLY_ACCESS_POLICY_ARN,
+    AMAZON_S3_FULL_ACCESS_POLICY_ARN,
+    ANYSCALE_IAM_PERMISSIONS_EC2_INITIAL_RUN,
+    ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE,
+    ANYSCALE_IAM_PERMISSIONS_SERVICE_STEADY_STATE,
+    get_anyscale_aws_iam_assume_role_policy,
+)
+from anyscale.cli_logger import BlockLogger
+from anyscale.client.openapi_client.models.create_cloud_resource import (
+    CreateCloudResource,
+)
+from anyscale.client.openapi_client.models.subnet_id_with_availability_zone_aws import (
+    SubnetIdWithAvailabilityZoneAWS,
+)
 from anyscale.cloud_resource import (
-    associate_subnets_with_azs,
-    aws_subnet_has_enough_capacity,
     verify_aws_cloudformation_stack,
     verify_aws_efs,
     verify_aws_iam_roles,
@@ -24,17 +38,7 @@ from anyscale.cloud_resource import (
 from anyscale.conf import ANYSCALE_IAM_ROLE_NAME
 from anyscale.shared_anyscale_utils.aws import AwsRoleArn
 from anyscale.shared_anyscale_utils.conf import ANYSCALE_HOST
-from frontend.cli.anyscale.aws_iam_policies import (
-    AMAZON_ECR_READONLY_ACCESS_POLICY_ARN,
-    AMAZON_S3_FULL_ACCESS_POLICY_ARN,
-    ANYSCALE_IAM_PERMISSIONS_EC2_INITIAL_RUN,
-    ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE,
-    get_anyscale_aws_iam_assume_role_policy,
-)
-from frontend.cli.anyscale.cli_logger import BlockLogger
-from frontend.cli.anyscale.client.openapi_client.models.create_cloud_resource import (
-    CreateCloudResource,
-)
+from anyscale.utils.network_verification import AWS_SUBNET_CAPACITY
 
 
 DEFAULT_RAY_IAM_ROLE = "ray-autoscaler-v1"
@@ -82,7 +86,14 @@ def _delete_role(role_arn: str) -> None:
 def generate_cloud_resource_mock_aws() -> CreateCloudResource:
     return CreateCloudResource(
         aws_vpc_id="fake_aws_vpc_id",
-        aws_subnet_ids=["fake_aws_subnet_id_0", "fake_aws_subnet_id_1"],
+        aws_subnet_ids_with_availability_zones=[
+            SubnetIdWithAvailabilityZoneAWS(
+                subnet_id="fake_aws_subnet_id_0", availability_zone="fake_aws_az_0"
+            ),
+            SubnetIdWithAvailabilityZoneAWS(
+                subnet_id="fake_aws_subnet_id_1", availability_zone="fake_aws_az_1"
+            ),
+        ],
         aws_iam_role_arns=[
             "arn:aws:iam::123:role/mock_anyscale_role",
             "arn:aws:iam::123:role/mock_dataplane_role",
@@ -104,6 +115,12 @@ def _attach_iam_roles_to_role(role_arn: str) -> None:
     )["Policy"]["Arn"]
     iam.attach_role_policy(RoleName=role_name, PolicyArn=steady_state)
 
+    service_steady_state = iam.create_policy(
+        PolicyName="".join(random.choices(string.ascii_lowercase, k=10)),
+        PolicyDocument=json.dumps(ANYSCALE_IAM_PERMISSIONS_SERVICE_STEADY_STATE),
+    )["Policy"]["Arn"]
+    iam.attach_role_policy(RoleName=role_name, PolicyArn=service_steady_state)
+
     initial_run = iam.create_policy(
         PolicyName="".join(random.choices(string.ascii_lowercase, k=10)),
         PolicyDocument=json.dumps(ANYSCALE_IAM_PERMISSIONS_EC2_INITIAL_RUN),
@@ -120,6 +137,11 @@ def _attach_inline_iam_policy_to_role(role_arn: str) -> None:
         PolicyName="".join(random.choices(string.ascii_lowercase, k=10)),
         PolicyDocument=json.dumps(ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE),
     )
+    iam.put_role_policy(
+        RoleName=role_name,
+        PolicyName="".join(random.choices(string.ascii_lowercase, k=10)),
+        PolicyDocument=json.dumps(ANYSCALE_IAM_PERMISSIONS_SERVICE_STEADY_STATE),
+    )
     iam.put_role_policy(
         RoleName=role_name,
         PolicyName="".join(random.choices(string.ascii_lowercase, k=10)),
@@ -148,18 +170,29 @@ def _create_vpc(cidr_block: str = "10.0.0.0/16", region: str = "us-west-2") -> s
 
 
 def _create_subnets(
-    vpc_id: str, region: str, num_subnets: int = 2, assign_public_ip: bool = False
-) -> List[str]:
+    vpc_id: str,
+    region: str,
+    num_subnets: int = 2,
+    assign_public_ip: bool = False,
+    use_single_availability_zone: bool = False,
+) -> List[SubnetIdWithAvailabilityZoneAWS]:
     ec2 = boto3.client("ec2", region_name=region)
     subnet_ids = []
     for i in range(num_subnets):
         subnet = ec2.create_subnet(
             VpcId=vpc_id,
             CidrBlock=f"10.0.{i}.0/24",
-            AvailabilityZone=f"{region}{string.ascii_lowercase[i]}",
+            AvailabilityZone=f"{region}a"
+            if use_single_availability_zone
+            else f"{region}{string.ascii_lowercase[i]}",
         )
         subnet_id = subnet["Subnet"]["SubnetId"]
-        subnet_ids.append(subnet_id)
+        subnet_az = subnet["Subnet"]["AvailabilityZone"]
+        subnet_ids.append(
+            SubnetIdWithAvailabilityZoneAWS(
+                subnet_id=subnet_id, availability_zone=subnet_az
+            )
+        )
         # MapPublicIpOnLaunch is by default False.
         if assign_public_ip:
             ec2.modify_subnet_attribute(
@@ -192,6 +225,35 @@ def _create_security_group(region: str) -> str:
     return security_group_id
 
 
+def _create_security_group_given_permissions(
+    permissions: List[str], region: str
+) -> str:
+    ec2 = boto3.client("ec2", region_name=region)
+    vpc_id = _create_vpc(region=region)
+
+    security_group = ec2.create_security_group(
+        VpcId=vpc_id, GroupName="mock_security_group", Description="mock_security_group"
+    )
+    security_group_id = security_group["GroupId"]
+
+    port_number = {"https": 443, "ssh": 22, "self": -1, "excess": 123}
+
+    ec2.authorize_security_group_ingress(
+        GroupId=security_group_id,
+        IpPermissions=[
+            {
+                "IpProtocol": "-1" if permission == "self" else "tcp",
+                "FromPort": port_number[permission],
+                "ToPort": port_number[permission],
+                "UserIdGroupPairs": [{"GroupId": security_group_id}],
+            }
+            for permission in permissions
+        ],
+    )
+
+    return security_group_id
+
+
 def _create_security_group_inbound_permission_expected(region: str) -> str:
     ec2 = boto3.client("ec2", region_name=region)
     vpc_id = _create_vpc(region=region)
@@ -266,9 +328,15 @@ def _create_security_group_inbound_permission_excessive_permission(region: str)
     return security_group_id
 
 
-def _create_bucket(bucket_name: str = "mock_bucket", cors_rules: Any = None) -> Any:
+def _create_bucket(
+    bucket_name: str = "mock_bucket", cors_rules: Any = None, region: str = "us-east-1"
+) -> Any:
     s3 = boto3.client("s3")
-    s3.create_bucket(Bucket=bucket_name)
+    kwargs = {}
+    if region != "us-east-1":
+        kwargs["CreateBucketConfiguration"] = {"LocationConstraint": region}
+
+    s3.create_bucket(Bucket=bucket_name, **kwargs)  # type: ignore
     if cors_rules:
         s3.put_bucket_cors(
             Bucket=bucket_name, CORSConfiguration={"CORSRules": cors_rules}
@@ -276,15 +344,31 @@ def _create_bucket(bucket_name: str = "mock_bucket", cors_rules: Any = None) ->
     return boto3.resource("s3").Bucket(bucket_name)
 
 
-def _create_efs(region: str, subnet_ids: List[str], security_group_ids: List[str]):
+def _create_efs(
+    region: str,
+    subnet_ids: List[str],
+    security_group_ids: List[str],
+    mount_target_ips: Any = None,
+    backup: bool = False,
+):
     efs = boto3.client("efs", region_name=region)
-    create_resp = efs.create_file_system(CreationToken="mock_efs")
+    create_resp = efs.create_file_system(CreationToken="mock_efs", Backup=backup)
     efs_id = create_resp["FileSystemId"]
 
     for subnet_id in subnet_ids:
-        efs.create_mount_target(
-            FileSystemId=efs_id, SubnetId=subnet_id, SecurityGroups=security_group_ids,
-        )
+        if mount_target_ips:
+            efs.create_mount_target(
+                FileSystemId=efs_id,
+                SubnetId=subnet_id,
+                SecurityGroups=security_group_ids,
+                IpAddress=mount_target_ips[subnet_id],
+            )
+        else:
+            efs.create_mount_target(
+                FileSystemId=efs_id,
+                SubnetId=subnet_id,
+                SecurityGroups=security_group_ids,
+            )
     return efs_id
 
 
@@ -319,10 +403,10 @@ def _create_cloudformation_stack(region: str) -> str:
         pytest.param(True, "0.0.0.0/20", True, r"verification succeeded."),
         # Warn sizes
         pytest.param(
-            True, "0.0.3.0/21", True, r"but this vpc only supports up to \d+ addresses",
+            True, "0.0.3.0/21", True, r"but this VPC only supports up to \d+ addresses",
         ),
         pytest.param(
-            True, "0.0.4.0/24", True, r"but this vpc only supports up to \d+ addresses",
+            True, "0.0.4.0/24", True, r"but this VPC only supports up to \d+ addresses",
         ),
         # Error sizes
         pytest.param(
@@ -375,7 +459,7 @@ def test_verify_aws_subnets(capsys, is_private_network: bool):
     )
 
     cloud_resource_mock.aws_vpc_id = vpc_id
-    cloud_resource_mock.aws_subnet_ids = subnet_ids
+    cloud_resource_mock.aws_subnet_ids_with_availability_zones = subnet_ids
 
     result = verify_aws_subnets(
         cloud_resource=cloud_resource_mock,
@@ -403,7 +487,7 @@ def test_verify_aws_subnets_warn_about_not_assigning_public_ip_in_public_subnets
     subnet_ids = _create_subnets(vpc_id=vpc_id, region=region, assign_public_ip=False)
 
     cloud_resource_mock.aws_vpc_id = vpc_id
-    cloud_resource_mock.aws_subnet_ids = subnet_ids
+    cloud_resource_mock.aws_subnet_ids_with_availability_zones = subnet_ids
 
     result = verify_aws_subnets(
         cloud_resource=cloud_resource_mock,
@@ -432,7 +516,7 @@ def test_verify_aws_subnets_vpc_subnet_mismatch(capsys):
     irrevalent_vpc_id = _create_vpc(region=region)
 
     cloud_resource_mock.aws_vpc_id = irrevalent_vpc_id
-    cloud_resource_mock.aws_subnet_ids = subnet_ids
+    cloud_resource_mock.aws_subnet_ids_with_availability_zones = subnet_ids
 
     result = verify_aws_subnets(
         cloud_resource=cloud_resource_mock,
@@ -456,8 +540,7 @@ def test_verify_aws_subnets_insufficient_subnets(capsys):
     subnet_ids = _create_subnets(vpc_id=vpc_id, region=region, num_subnets=1)
 
     cloud_resource_mock.aws_vpc_id = vpc_id
-    cloud_resource_mock.aws_subnet_ids = subnet_ids
-    cloud_resource_mock.aws_subnet_ids_with_availability_zones = None
+    cloud_resource_mock.aws_subnet_ids_with_availability_zones = subnet_ids
 
     result = verify_aws_subnets(
         cloud_resource=cloud_resource_mock,
@@ -470,11 +553,34 @@ def test_verify_aws_subnets_insufficient_subnets(capsys):
     assert re.search(r"Need at least 2 subnets for a cloud.", stderr)
 
 
+@mock_ec2
+def test_verify_aws_subnets_insufficient_azs(capsys):
+    cloud_resource_mock = generate_cloud_resource_mock_aws()
+
+    region = "us-west-2"
+    vpc_id = _create_vpc(region=region)
+    subnet_ids = _create_subnets(
+        vpc_id=vpc_id, region=region, num_subnets=2, use_single_availability_zone=True
+    )
+
+    cloud_resource_mock.aws_vpc_id = vpc_id
+    cloud_resource_mock.aws_subnet_ids_with_availability_zones = subnet_ids
+
+    result = verify_aws_subnets(
+        cloud_resource=cloud_resource_mock,
+        region=region,
+        is_private_network=False,
+        logger=BlockLogger(),
+    )
+    assert result is False
+    _, stderr = capsys.readouterr()
+    assert re.search(r"Subnets should be in at least 2 Availability Zones", stderr)
+
+
 @mock_ec2
 def test_verify_aws_subnets_no_subnets():
     cloud_resource_mock = generate_cloud_resource_mock_aws()
 
-    cloud_resource_mock.aws_subnet_ids = None
     cloud_resource_mock.aws_subnet_ids_with_availability_zones = None
 
     result = verify_aws_subnets(
@@ -494,8 +600,8 @@ def test_verify_aws_subnets_no_subnets():
         ("0.0.0.0/0", True, None),
         ("0.0.1.0/24", True, None),
         # Warn sizes
-        ("0.0.2.0/25", True, r"but this subnet only supports up to \d+ addresses"),
-        ("0.0.3.0/28", True, r"but this subnet only supports up to \d+ addresses"),
+        ("0.0.2.0/25", True, r"but this Subnet only supports up to \d+ addresses"),
+        ("0.0.3.0/28", True, r"but this Subnet only supports up to \d+ addresses"),
         # Error sizes
         ("0.0.3.0/29", False, r"Please reach out to support if this is an issue!"),
     ],
@@ -507,7 +613,12 @@ def test_aws_subnet_has_enough_capacity(
     subnet.id = "vpc-fake_id"
     subnet.cidr_block = subnet_cidr
 
-    assert aws_subnet_has_enough_capacity(subnet, BlockLogger()) == has_capacity
+    assert (
+        AWS_SUBNET_CAPACITY.verify_network_capacity(
+            cidr_block_str=subnet_cidr, resource_name="Subnet", logger=BlockLogger()
+        )
+        == has_capacity
+    )
 
     stdout, stderr = capsys.readouterr()
     sys.stdout.write(stdout)
@@ -540,6 +651,7 @@ def test_verify_aws_iam_roles(
         boto3_session=boto3.Session(),
         anyscale_aws_account="fake_anyscale_aws_account",
         logger=mock_logger,
+        cloud_id="fake_cloud_id",
     )
 
     if expect_warning:
@@ -550,6 +662,56 @@ def test_verify_aws_iam_roles(
     assert len(mock_logger.info.mock_calls) == 2
 
 
+@mock_iam
+@pytest.mark.parametrize("include_service_permissions", [True, False])
+def test_verify_aws_iam_service_permissions(include_service_permissions: bool,):
+    """Test service permission validation
+    """
+    cloud_resource_mock = generate_cloud_resource_mock_aws()
+    cp_role_arn, dp_role_arn = _create_roles("fake_anyscale_aws_account")
+    _create_instance_profile_for_role(dp_role_arn, None)
+    cloud_resource_mock.aws_iam_role_arns = [cp_role_arn, dp_role_arn]
+
+    iam = boto3.client("iam")
+    role_name = AwsRoleArn.from_string(cp_role_arn).to_role_name()
+
+    iam.put_role_policy(
+        RoleName=role_name,
+        PolicyName="".join(random.choices(string.ascii_lowercase, k=10)),
+        PolicyDocument=json.dumps(ANYSCALE_IAM_PERMISSIONS_EC2_STEADY_STATE),
+    )
+    if include_service_permissions:
+        iam.put_role_policy(
+            RoleName=role_name,
+            PolicyName="".join(random.choices(string.ascii_lowercase, k=10)),
+            PolicyDocument=json.dumps(ANYSCALE_IAM_PERMISSIONS_SERVICE_STEADY_STATE),
+        )
+    iam.put_role_policy(
+        RoleName=role_name,
+        PolicyName="".join(random.choices(string.ascii_lowercase, k=10)),
+        PolicyDocument=json.dumps(ANYSCALE_IAM_PERMISSIONS_EC2_INITIAL_RUN),
+    )
+
+    mock_logger = Mock()
+    test_status = verify_aws_iam_roles(
+        cloud_resource=cloud_resource_mock,
+        boto3_session=boto3.Session(),
+        anyscale_aws_account="fake_anyscale_aws_account",
+        logger=mock_logger,
+        cloud_id="fake_cloud_id",
+    )
+
+    assert test_status
+
+    if include_service_permissions:
+
+        mock_logger.warning.assert_not_called()
+        assert len(mock_logger.info.mock_calls) == 2
+
+    else:
+        assert len(mock_logger.confirm_missing_permission.mock_calls) == 1
+
+
 @mock_iam
 @pytest.mark.parametrize(
     ("cp_role_exist", "dp_role_exist"),
@@ -570,6 +732,7 @@ def test_verify_aws_iam_roles_not_exist(cp_role_exist: bool, dp_role_exist: bool
         boto3_session=boto3.Session(),
         anyscale_aws_account="fake_anyscale_aws_account",
         logger=BlockLogger(),
+        cloud_id="fake_cloud_id",
     )
 
 
@@ -588,6 +751,7 @@ def test_verify_aws_iam_roles_instance_profiles():
         boto3_session=boto3.Session(),
         anyscale_aws_account="fake_anyscale_aws_account",
         logger=mock_logger,
+        cloud_id="fake_cloud_id",
     )
 
     # Create an instance profile for the role, but with the wrong name
@@ -597,6 +761,7 @@ def test_verify_aws_iam_roles_instance_profiles():
         boto3_session=boto3.Session(),
         anyscale_aws_account="fake_anyscale_aws_account",
         logger=mock_logger,
+        cloud_id="fake_cloud_id",
     )
 
     # Create a correct insance profile with the role!
@@ -606,38 +771,57 @@ def test_verify_aws_iam_roles_instance_profiles():
         boto3_session=boto3.Session(),
         anyscale_aws_account="fake_anyscale_aws_account",
         logger=mock_logger,
+        cloud_id="fake_cloud_id",
     )
 
 
+@mock_iam
+def test_verify_aws_iam_roles_different_account(capsys):
+    """Tests that the IAM roles must be in the same account."""
+    cloud_resource_mock = generate_cloud_resource_mock_aws()
+    cp_role_arn, _ = _create_roles("fake_anyscale_aws_account")
+    with patch.dict(os.environ, {"MOTO_ACCOUNT_ID": "112233445566"}):
+        _, dp_role_arn = _create_roles("fake_anyscale_aws_account")
+    cloud_resource_mock.aws_iam_role_arns = [cp_role_arn, dp_role_arn]
+
+    assert not verify_aws_iam_roles(
+        cloud_resource=cloud_resource_mock,
+        boto3_session=boto3.Session(),
+        anyscale_aws_account="fake_anyscale_aws_account",
+        logger=BlockLogger(),
+        cloud_id="fake_cloud_id",
+    )
+
+    _, stderr = capsys.readouterr()
+    assert "All IAM roles must be in the same AWS account:" in stderr
+
+
 @pytest.mark.parametrize(
-    ("sg_exists", "sg_valid", "expected_result", "expected_log_message"),
+    ("sg_list", "expected_result", "expected_log_message"),
     [
+        pytest.param([], False, r"Missing security group IDs", id="SG-Does-Not-Exist"),
+        pytest.param([False], False, r"Could not find", id="SG-Exists-Not-Valid"),
+        pytest.param([True], True, None, id="SG-Exists-And-Valid"),
         pytest.param(
-            False, False, False, r"Missing security group IDs", id="SG-Does-Not-Exist"
+            [True, False], False, r"Could not find", id="2SG-Exists-Not-Valid"
         ),
-        pytest.param(True, False, False, r"Could not find", id="SG-Exists-Not-Valid"),
-        pytest.param(True, True, True, None, id="SG-Exists-And-Valid"),
+        pytest.param([True, True], True, None, id="2SG-Exists-And-Valid"),
     ],
 )
 @mock_ec2
 def test_verify_aws_security_groups_exist_and_valid(
-    capsys,
-    sg_exists: bool,
-    sg_valid: bool,
-    expected_result: bool,
-    expected_log_message: str,
+    capsys, sg_list: List[bool], expected_result: bool, expected_log_message: str,
 ):
     cloud_resource_mock = generate_cloud_resource_mock_aws()
 
     region = "us-west-2"
-    if sg_exists:
-        if sg_valid:
+    cloud_resource_mock.aws_security_groups = []
+    for sg_valid in sg_list:
+        if sg_valid:  # True means valid sg, False throws not found error
             sg_id = _create_security_group(region)
         else:
             sg_id = "Invalid_SG"
-        cloud_resource_mock.aws_security_groups = [sg_id]
-    else:
-        cloud_resource_mock.aws_security_groups = []
+        cloud_resource_mock.aws_security_groups.append(sg_id)
 
     result = verify_aws_security_groups(
         cloud_resource=cloud_resource_mock,
@@ -661,7 +845,7 @@ def test_verify_aws_security_groups_exist_and_valid(
         pytest.param(
             True,
             False,
-            r"does not contain inbound permission for ports",
+            r"not contain inbound permission for ports",
             id="Not_Enough_Permission",
         ),
         pytest.param(
@@ -703,6 +887,73 @@ def test_verify_aws_security_groups_inbound_permissions(
         assert re.search(expected_log_message, stderr)
 
 
+@pytest.mark.parametrize(
+    ("security_groups_permissions", "expected_result", "expected_log_message"),
+    [
+        pytest.param(
+            [["https"], ["ssh"], ["self"]], True, None, id="Expected_Permission"
+        ),
+        pytest.param(
+            [["https", "ssh", "self"], ["ssh", "self"], ["https"]],
+            True,
+            None,
+            id="Expected_Permission_With_Duplicates",
+        ),
+        pytest.param(
+            [["https", "self"], ["self"]],
+            True,
+            r"not contain inbound permission for ports",
+            id="Missing_Permission",
+        ),
+        pytest.param(
+            [["https", "ssh"], ["https"]],
+            False,
+            r"not contain inbound permission for all ports for traffic from the same",
+            id="Missing_Self_Permission",
+        ),
+        pytest.param(
+            [["https", "excess"], ["ssh"], ["self"]],
+            True,
+            r"allows access to more than",
+            id="Excessive_Permission",
+        ),
+    ],
+)
+@mock_ec2
+def test_verify_aws_security_groups_multiple_security_groups(
+    capsys,
+    security_groups_permissions: List[List[str]],
+    expected_result,
+    expected_log_message: str,
+):
+    """Tests permission check is correct for multiple security groups"""
+    cloud_resource_mock = generate_cloud_resource_mock_aws()
+
+    region = "us-west-2"
+    sg_ids = []
+
+    for security_group_permission in security_groups_permissions:
+        sg_id = _create_security_group_given_permissions(
+            security_group_permission, region
+        )
+        sg_ids.append(sg_id)
+
+    cloud_resource_mock.aws_security_groups = sg_ids
+
+    result = verify_aws_security_groups(
+        cloud_resource=cloud_resource_mock,
+        boto3_session=boto3.Session(region_name=region),
+        logger=BlockLogger(),
+    )
+    stdout, stderr = capsys.readouterr()
+    sys.stdout.write(stdout)
+    sys.stderr.write(stderr)
+
+    assert result == expected_result
+    if expected_log_message:
+        assert re.search(expected_log_message, stderr)
+
+
 @pytest.mark.parametrize(
     ("s3_exists", "expected_result"),
     [
@@ -752,10 +1003,11 @@ def test_verify_aws_s3(
 
     region = "us-west-2"
     if s3_exists:
-        bucket = _create_bucket(cors_rules=cors_rules)
+        bucket = _create_bucket(cors_rules=cors_rules, region=region)
         cloud_resource_mock.aws_s3_id = bucket.name
 
     cp_role_arn, dp_role_arn = _create_roles("fake_anyscale_aws_account")
+
     cloud_resource_mock.aws_iam_role_arns = [cp_role_arn, dp_role_arn]
 
     mock_logger = Mock()
@@ -766,6 +1018,7 @@ def test_verify_aws_s3(
         result = verify_aws_s3(
             cloud_resource=cloud_resource_mock,
             boto3_session=boto3.Session(region_name=region),
+            region=region,
             logger=mock_logger,
         )
 
@@ -778,6 +1031,33 @@ def test_verify_aws_s3(
     assert result == expected_result
 
 
+@mock_s3
+@mock_iam
+def test_verify_aws_s3_wrong_region(capsys):
+    cloud_resource_mock = generate_cloud_resource_mock_aws()
+
+    region = "us-west-2"
+    bucket = _create_bucket(region="us-east-1")
+    cloud_resource_mock.aws_s3_id = bucket.name
+
+    cp_role_arn, dp_role_arn = _create_roles("fake_anyscale_aws_account")
+    cloud_resource_mock.aws_iam_role_arns = [cp_role_arn, dp_role_arn]
+
+    with patch.multiple(
+        "anyscale.cloud_resource", verify_s3_access=Mock(return_value=True),
+    ):
+        verify_aws_s3(
+            cloud_resource=cloud_resource_mock,
+            boto3_session=boto3.Session(region_name=region),
+            region=region,
+            logger=BlockLogger(),
+        )
+
+    _, stderr = capsys.readouterr()
+
+    assert re.search("S3 bucket .* is in region us-east-1, .* us-west-2", stderr)
+
+
 @pytest.mark.parametrize(
     (
         "efs_exists",
@@ -785,14 +1065,16 @@ def test_verify_aws_s3(
         "use_security_group",
         "expected_result",
         "warning",
+        "enable_backup",
     ),
     [
-        pytest.param(False, 2, True, False, False),
-        pytest.param(True, 0, False, False, False),
-        pytest.param(True, 1, True, True, True),
-        pytest.param(True, 1, False, True, True),
-        pytest.param(True, 2, True, True, False),
-        pytest.param(True, 2, False, True, True),
+        pytest.param(False, 2, True, False, False, True),
+        pytest.param(True, 0, False, False, False, True),
+        pytest.param(True, 1, True, True, True, True),
+        pytest.param(True, 1, False, True, True, True),
+        pytest.param(True, 2, True, True, False, True),
+        pytest.param(True, 2, False, True, True, True),
+        pytest.param(True, 2, False, True, True, False),
     ],
 )
 @mock_efs
@@ -804,19 +1086,24 @@ def test_verify_aws_efs(
     use_security_group: bool,
     expected_result: bool,
     warning: bool,
+    enable_backup: bool,
 ):
     cloud_resource_mock = generate_cloud_resource_mock_aws()
 
     region = "us-west-2"
     if efs_exists:
         vpc_id = _create_vpc(region=region)
-        subnet_ids = _create_subnets(vpc_id, region, num_subnets=2)
-        assert len(subnet_ids) == 2
+        aws_subnet_ids = _create_subnets(vpc_id, region, num_subnets=2)
+        assert len(aws_subnet_ids) == 2
         security_group_id = _create_security_group(region)
-        subnet_ids_in_efs = subnet_ids[:num_subnets_to_use]
+        subnet_ids_in_efs = [
+            subnet.subnet_id for subnet in aws_subnet_ids[:num_subnets_to_use]
+        ]
         security_groups_in_efs = [security_group_id] if use_security_group else []
-        efs_id = _create_efs(region, subnet_ids_in_efs, security_groups_in_efs)
-        cloud_resource_mock.aws_subnet_ids = subnet_ids
+        efs_id = _create_efs(
+            region, subnet_ids_in_efs, security_groups_in_efs, backup=enable_backup
+        )
+        cloud_resource_mock.aws_subnet_ids_with_availability_zones = aws_subnet_ids
         cloud_resource_mock.aws_security_groups = [security_group_id]
         cloud_resource_mock.aws_efs_id = efs_id
 
@@ -825,12 +1112,76 @@ def test_verify_aws_efs(
         boto3_session=boto3.Session(region_name=region),
         logger=BlockLogger(),
     )
+    _, stderr = capsys.readouterr()
     if warning:
-        _, stderr = capsys.readouterr()
         assert re.search(r"does not contain network interface with subnet id", stderr)
+    if enable_backup:
+        assert not re.search(r"backup policy is not enabled", stderr)
+        assert not re.search(r"backup policy not found", stderr)
+    else:
+        assert re.search(r"backup policy is not enabled", stderr) or re.search(
+            r"backup policy not found", stderr
+        )
     assert result == expected_result
 
 
+@pytest.mark.parametrize(
+    ("valid_ip", "expected_result", "expected_log_message",),
+    [
+        pytest.param(
+            False, False, "Mount target registered with the cloud no longer exists."
+        ),
+        pytest.param(True, True, None),
+    ],
+)
+@mock_efs
+@mock_ec2
+def test_verify_aws_efs_valid_ip(
+    capsys, valid_ip: bool, expected_result: bool, expected_log_message: str,
+):
+    cloud_resource_mock = generate_cloud_resource_mock_aws()
+
+    region = "us-west-2"
+
+    vpc_id = _create_vpc(region=region)
+    subnet_ids_with_az = _create_subnets(vpc_id, region, num_subnets=2)
+    subnet_ids = [
+        subnet_id_with_az.subnet_id for subnet_id_with_az in subnet_ids_with_az
+    ]
+    security_groups = [_create_security_group(region)]
+
+    mount_target_ips = {
+        subnet_id_with_az.subnet_id: f"10.0.{i}.123"
+        for i, subnet_id_with_az in enumerate(subnet_ids_with_az)
+    }
+
+    efs_id = _create_efs(
+        region, subnet_ids, security_groups, mount_target_ips, backup=True
+    )
+    cloud_resource_mock.aws_subnet_ids = subnet_ids
+    cloud_resource_mock.aws_security_groups = security_groups
+    cloud_resource_mock.aws_efs_id = efs_id
+
+    if valid_ip:
+        cloud_resource_mock.aws_efs_mount_target_ip = mount_target_ips[subnet_ids[0]]
+    else:
+        cloud_resource_mock.aws_efs_mount_target_ip = "1.2.3.4"
+
+    result = verify_aws_efs(
+        cloud_resource=cloud_resource_mock,
+        boto3_session=boto3.Session(region_name=region),
+        logger=BlockLogger(),
+    )
+
+    stdout, stderr = capsys.readouterr()
+    sys.stdout.write(stdout)
+    sys.stderr.write(stderr)
+
+    assert result == expected_result
+    if expected_log_message:
+        assert re.search(expected_log_message, stderr)
+
+
 @pytest.mark.parametrize(
     ("cloudformation_stack_exists", "expected_result"),
     [pytest.param(False, False), pytest.param(True, True)],
@@ -853,18 +1204,3 @@ def test_verify_aws_cloudformation_stack(
         logger=BlockLogger(),
     )
     assert result == expected_result
-
-
-@mock_ec2
-def test_associate_subnets_with_azs():
-    cloud_resource_mock = generate_cloud_resource_mock_aws()
-    region = "us-west-2"
-    vpc_id = _create_vpc(region=region)
-    subnet_ids = _create_subnets(vpc_id=vpc_id, region=region)
-
-    cloud_resource_mock.aws_subnet_ids = subnet_ids
-    cloud_resource_mock.aws_subnet_ids_with_availability_zones = None
-
-    new_cloud_resource = associate_subnets_with_azs(cloud_resource_mock, region)
-    assert new_cloud_resource.aws_subnet_ids is None
-    assert len(new_cloud_resource.aws_subnet_ids_with_availability_zones) == 2
