@@ -258,10 +258,6 @@ def _make_test_builder(
             _ignore_version_check=False,
             auth_api_client=_auth_api_client,
         )
-    builder._exec_controller.api_client = _api_client
-    builder._exec_controller.anyscale_api_client = _anyscale_api_client
-    builder._session_controller.api_client = _api_client
-    builder._session_controller.anyscale_api_client = _anyscale_api_client
     if setup_project_dir:
         builder.project_dir(scratch.absolute().as_posix())
     else:
@@ -423,7 +419,7 @@ def test_autosuspend_from_env(mock_auth_api_client) -> None:
         auth_api_client=Mock(api_client=Mock(), anyscale_api_client=Mock()),
     )
     unconfigured_builder._fill_unset_configs_from_env()
-    assert unconfigured_builder._autosuspend_timeout == 120  # default
+    assert unconfigured_builder._autosuspend_timeout is None  # Not set in environment
 
     unconfigured_builder = ClientBuilder(
         anyscale_sdk=Mock(),
